@@ -4,7 +4,7 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: config.c,v 1.103 2011/03/20 21:31:36 phintuka Exp $
+ * $Id: config.c,v 1.110 2012/01/30 10:41:56 phintuka Exp $
  *
  */
 
@@ -115,7 +115,18 @@ const char * const config_t::s_audioDriverNames[ AUDIO_DRIVER_count+1 ] = {
 };
 
 const char * const config_t::s_videoDriversX11[ X11_DRIVER_count+1 ] =  {
-  "auto", "xshm", "xv", "xvmc", "xxmc", "vidix", "XDirectFB", "opengl", "sdl", "none",
+  "auto",
+  "xshm",
+  "xv",
+  "vaapi",
+  "vdpau",
+  "opengl",
+  "xvmc",
+  "xxmc",
+  "sdl",
+  "XDirectFB",
+  "vidix",
+  "none",
   NULL
 };
 
@@ -123,12 +134,14 @@ const char * const config_t::s_videoDriverNamesX11[ X11_DRIVER_count+1 ] =  {
   trNOOP("automatic"),
   "XShm",
   "Xv",
+  "VAAPI",
+  "VDPAU",
+  "OpenGL",
   "XvMC",
   "XvMC+VLD",
-  "Vidix",
-  "XDirectFB",
-  "OpenGL",
   "SDL",
+  "XDirectFB",
+  "Vidix",
   trNOOP("no video"),
   NULL
 };
@@ -211,8 +224,8 @@ const char * const config_t::s_subtitleSizes[ SUBTITLESIZE_count+1 ] = {
 };
 
 const char * const config_t::s_subExts[] = {
-  ".sub", ".srt", ".txt", ".ssa",
-  ".SUB", ".SRT", ".TXT", ".SSA", 
+  ".sub", ".srt", ".txt", ".ssa", "ass", "smi",
+  ".SUB", ".SRT", ".TXT", ".SSA", "ASS", "SMI",
   NULL
 };
 
@@ -421,6 +434,17 @@ bool config_t::IsDvdImage(const char *fname)
   return (ext && !strcasecmp(ext, "iso")) ? true : false;
 }
 
+bool config_t::IsBluRayImage(const char *fname)
+{
+  if (IsDvdImage(fname)) {
+    struct stat st;
+    if (!stat(fname, &st)) {
+      return st.st_size > 10*1024*1024*1024ll;
+    }
+  }
+  return false;
+}
+
 bool config_t::IsDvdFolder(const char *fname)
 {
   struct stat st;
@@ -604,7 +628,7 @@ config_t::config_t() {
   alpha_correction_abs = 0;
 
   fullscreen     = 0;
-  modeswitch     = 1;
+  modeswitch     = 0;
   width          = 720;
   height         = 576;
   scale_video    = 0;
@@ -694,6 +718,7 @@ config_t::config_t() {
   enable_id3_scanner = 1;
   dvd_arrow_keys_control_playback = 1;
   media_menu_items = ~0;
+  media_enable_delete = 0;
 
   main_menu_mode = ShowMenu;
   force_primary_device = 0;
@@ -716,7 +741,7 @@ bool config_t::ProcessArg(const char *Name, const char *Value)
 
 bool config_t::ProcessArgs(int argc, char *argv[])
 {
-  static const char short_options[] = "fDw:h:l:r:A:V:d:P:C:pc";
+  static const char short_options[] = "fDw:h:l:mr:A:V:d:P:C:pc";
 
   static const struct option long_options[] = {
       { "fullscreen",   no_argument,       NULL, 'f' },
@@ -734,6 +759,7 @@ bool config_t::ProcessArgs(int argc, char *argv[])
       { "video",        required_argument, NULL, 'V' },
       { "display",      required_argument, NULL, 'd' },
       { "window",       required_argument, NULL, 'W' },
+      { "modeswitch",   no_argument,       NULL, 'm' },
       { "post",         required_argument, NULL, 'P' },
       { "config",       required_argument, NULL, 'C' },
       { "primary",      no_argument,       NULL, 'p' },
@@ -748,6 +774,11 @@ bool config_t::ProcessArgs(int argc, char *argv[])
               break;
     case 'W': ProcessArg("X11.WindowId", optarg);
               break;
+    case 'm': ProcessArg("VideoModeSwitching", "1");
+#ifndef HAVE_XRANDR
+              LOGMSG("Video mode switching not supported");
+#endif
+              break;
     case 'f': ProcessArg("Fullscreen", "1");
               break;
     case 'D': ProcessArg("X11.HUDOSD", "1");
@@ -764,12 +795,13 @@ bool config_t::ProcessArgs(int argc, char *argv[])
 #endif
               }
 #ifndef HAVE_XRENDER
-              LOGMSG("HUD OSD not supported\n");
+              else
+                LOGMSG("HUD OSD not supported\n");
 #endif
               break;
     case 'O': ProcessArg("OpenglAlways", "1");
 #ifndef HAVE_OPENGL
-              LOGMSG("OpenGL HUD OSD not supported\n");
+              LOGMSG("OpenGL not supported\n");
 #endif
               break;
     case 'w': //ProcessArg("Fullscreen", "0");
@@ -1003,6 +1035,7 @@ bool config_t::SetupParse(const char *Name, const char *Value)
   else if (!strcasecmp(Name, "Media.EnableID3Scanner"))  enable_id3_scanner = atoi(Value);
   else if (!strcasecmp(Name, "Media.DVD.ArrowKeysControlPlayback")) dvd_arrow_keys_control_playback = atoi(Value);
   else if (!strcasecmp(Name, "Media.MenuItems"))         media_menu_items = atoi(Value);
+  else if (!strcasecmp(Name, "Media.EnableDelete"))      media_enable_delete = atoi(Value);
 
   else if (!strcasecmp(Name, "Playlist.Tracknumber")) playlist_tracknumber = atoi(Value);
   else if (!strcasecmp(Name, "Playlist.Artist"))      playlist_artist = atoi(Value);
