@@ -4,7 +4,7 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: config.h,v 1.78 2011/03/20 21:31:36 phintuka Exp $
+ * $Id: config.h,v 1.83 2012/01/19 08:18:26 phintuka Exp $
  *
  */
 
@@ -76,14 +76,16 @@
 #define X11_DRIVER_AUTO          0
 #define X11_DRIVER_XSHM          1
 #define X11_DRIVER_XV            2
-#define X11_DRIVER_XVMC          3
-#define X11_DRIVER_XXMC          4
-#define X11_DRIVER_VIDIX         5
-#define X11_DRIVER_DIRECTFB      6
-#define X11_DRIVER_OPENGL        7
+#define X11_DRIVER_VAAPI         3
+#define X11_DRIVER_VDPAU         4
+#define X11_DRIVER_OPENGL        5
+#define X11_DRIVER_XVMC          6
+#define X11_DRIVER_XXMC          7
 #define X11_DRIVER_SDL           8
-#define X11_DRIVER_NONE          9
-#define X11_DRIVER_count         10
+#define X11_DRIVER_DIRECTFB      9
+#define X11_DRIVER_VIDIX         10
+#define X11_DRIVER_NONE          11
+#define X11_DRIVER_count         12
 
 #define FB_DRIVER_AUTO           0
 #define FB_DRIVER_FB             1
@@ -346,6 +348,7 @@ class config_t {
     int  playlist_album;
     int  dvd_arrow_keys_control_playback;
     uint media_menu_items;         // enabled items in media player menu (bitmask)
+    int  media_enable_delete;      // enable Delete in file browser
 
     // Audio visualization
     char audio_visualization[64];
@@ -453,6 +456,7 @@ class config_t {
     static bool IsDvdFolder(const char *);
     static bool IsBluRayFolder(const char *);
     static bool IsDvdImage(const char *);
+    static bool IsBluRayImage(const char *);
 
     cString AutocropOptions(void);
     cString SwScaleOptions(void);
