@@ -4,7 +4,7 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: device.c,v 1.57.2.11 2009/09/14 13:09:23 phintuka Exp $
+ * $Id: device.c,v 1.122 2012/12/09 21:37:53 rofafor Exp $
  *
  */
 
@@ -36,6 +36,7 @@
 #include "tools/pes.h"
 #include "tools/ts.h"
 #include "tools/functor.h"
+#include "tools/section_lock.h"
 
 #include "frontend_local.h"
 #include "frontend_svr.h"
@@ -46,6 +47,10 @@
 #define LOCAL_INIT_TIMEOUT        20  // seconds
 #define SERVER_INIT_TIMEOUT       5   // seconds
 
+#if (VDRVERSNUM > 10700) && (VDRVERSNUM < 10711)
+# error VDR versions 1.7.1 ... 1.7.10 are not supported !
+#endif
+
 #ifdef LOG_TRICKSPEED
 #  define LOGTRICKSPEED(x...) LOGMSG("trs: " x)
 #else
@@ -72,7 +77,11 @@ class cXinelibStatusMonitor : public cStatus
     };
 
   protected:
+#if VDRVERSNUM < 10726
     virtual void ChannelSwitch(const cDevice *Device, int ChannelNumber);
+#else
+    virtual void ChannelSwitch(const cDevice *Device, int ChannelNumber, bool LiveView);
+#endif
     virtual void Replaying(const cControl *Control, const char *Name, 
 			   const char *FileName, bool On);
 
@@ -107,7 +116,11 @@ class cXinelibStatusMonitor : public cStatus
 };
 
 void cXinelibStatusMonitor::ChannelSwitch(const cDevice *Device, 
+#if VDRVERSNUM < 10726
 					  int ChannelNumber) 
+#else
+					  int ChannelNumber, bool LiveView) 
+#endif
 {
   TRACEF("cXinelibStatusMonitor::ChannelSwitch");
   TRACK_TIME(200);
@@ -187,40 +200,29 @@ cXinelibDevice::cXinelibDevice()
   m_OriginalPrimaryDevice = 0;
   m_ForcePrimaryDeviceCnt = 0;
 
-  if(*xc.local_frontend && strncmp(xc.local_frontend, "none", 4))
-    m_clients.Add(m_local = new cXinelibLocal(xc.local_frontend));
-  if(xc.remote_mode && xc.listen_port>0)
-    m_clients.Add(m_server = new cXinelibServer(xc.listen_port));
-
-  m_ac3Present  = false;
-  m_spuPresent  = false;
-
-#if VDRVERSNUM < 10515
-  m_CurrentDvdSpuTrack = ttXSubtitleNone;
-  m_ForcedDvdSpuTrack = false;
-  ClrAvailableDvdSpuTracks();
-#endif
-
   memset(m_MetaInfo, 0, sizeof(m_MetaInfo));
-    
+
   m_PlayMode = pmNone;
   m_AudioChannel = 0;
 
   m_liveMode    = true;
   m_TrickSpeed  = -1;
-  m_TrickSpeedPts = 0;
   m_TrickSpeedMode = 0;
+#if VDRVERSNUM < 10705
+  m_TrickSpeedPts = 0;
   m_TrickSpeedDelay = 0;
-  m_VDR_TrickSpeedIBP = 0;
+#endif
   m_SkipAudio   = false;
   m_PlayingFile = pmNone;
   m_StreamStart = true;
   m_RadioStream = false;
   m_AudioCount  = 0;
   m_FreeBufs = 0;
-  m_h264 = false;
 
   m_VideoSize = (video_size_t*)calloc(1, sizeof(video_size_t));
+  m_tssVideoSize = NULL;
+
+  TsBufferClear();
 }
 
 cXinelibDevice::~cXinelibDevice() 
@@ -232,6 +234,24 @@ cXinelibDevice::~cXinelibDevice()
   m_pInstance = NULL;
 
   free (m_VideoSize);
+  ts_state_dispose(m_tssVideoSize);
+}
+
+bool cXinelibDevice::InitDevice()
+{
+  TRACEF("cXinelibDevice::InitDevice");
+
+  if (m_local || m_server) {
+    LOGMSG("cXinelibDevice::InitDevice() called twice");
+    return false;
+  }
+
+  if (*xc.local_frontend && strncmp(xc.local_frontend, "none", 4))
+    m_clients.Add(m_local = new cXinelibLocal(this, xc.local_frontend));
+  if (xc.remote_mode && xc.listen_port > 0)
+    m_clients.Add(m_server = new cXinelibServer(this, xc.listen_port));
+
+  return true;
 }
 
 bool cXinelibDevice::StartDevice()
@@ -259,6 +279,9 @@ bool cXinelibDevice::StartDevice()
     }
     if(xc.force_primary_device)
       ForcePrimaryDevice(true);
+    else if (cDevice::PrimaryDevice() && this != cDevice::PrimaryDevice()) {
+      LOGMSG("WARNING: xineliboutput is not the primary device !");
+    }
   }
 
   if(m_server) {
@@ -320,7 +343,7 @@ void cXinelibDevice::StopDevice(void)
 // Primary device switching
 //
 
-void cXinelibDevice::MakePrimaryDevice(bool On) 
+void cXinelibDevice::MakePrimaryDevice(bool On)
 {
   TRACEF("cXinelibDevice::MakePrimaryDevice");
 
@@ -330,16 +353,19 @@ void cXinelibDevice::MakePrimaryDevice(bool On)
     new cXinelibOsdProvider(this);
 }
 
-void cXinelibDevice::ForcePrimaryDevice(bool On) 
+bool cXinelibDevice::ForcePrimaryDevice(bool On)
 {
   TRACEF("cXinelibDevice::ForcePrimaryDevice");
 
   m_MainThreadLock.Lock();
   m_MainThreadFunctors.Add(CreateFunctor(this, &cXinelibDevice::ForcePrimaryDeviceImpl, On));
   m_MainThreadLock.Unlock();
+
+  return xc.force_primary_device ||
+         (cDevice::PrimaryDevice() && this == cDevice::PrimaryDevice());
 }
 
-void cXinelibDevice::ForcePrimaryDeviceImpl(bool On) 
+void cXinelibDevice::ForcePrimaryDeviceImpl(bool On)
 {
   TRACEF("cXinelibDevice::ForcePrimaryDeviceImpl");
   ASSERT(cThread::IsMainThread(), false);
@@ -469,8 +495,7 @@ void cXinelibDevice::ConfigureDecoder(int pes_buffers)
 
 void cXinelibDevice::ConfigureWindow(int fullscreen, int width, int height, 
 				     int modeswitch, const char *modeline, 
-				     int aspect, int scale_video, 
-				     int field_order)
+				     int aspect, int scale_video)
 {
   TRACEF("cXinelibDevice::ConfigureWindow");
 
@@ -486,10 +511,10 @@ void cXinelibDevice::ConfigureWindow(int fullscreen, int width, int height,
 
   if(m_local)
     m_local->ConfigureWindow(fullscreen, width, height, modeswitch, modeline, 
-			     aspect, scale_video, field_order);
+			     aspect, scale_video);
 
   else if(*xc.local_frontend && strncmp(xc.local_frontend, "none", 4)) {
-    cXinelibThread *tmp = new cXinelibLocal(xc.local_frontend);
+    cXinelibThread *tmp = new cXinelibLocal(this, xc.local_frontend);
     tmp->Start();
     m_clients.Add(m_local = tmp);
 
@@ -506,7 +531,7 @@ void cXinelibDevice::ConfigureWindow(int fullscreen, int width, int height,
 	ForcePrimaryDevice(true);
   
       m_local->ConfigureWindow(fullscreen, width, height, modeswitch, modeline,
-			       aspect, scale_video, field_order);
+			       aspect, scale_video);
     }
   }
 }
@@ -517,7 +542,7 @@ void cXinelibDevice::Listen(bool activate, int port)
 
   if(activate && port>0) {
     if(!m_server) {
-      cXinelibThread *tmp = new cXinelibServer(port);
+      cXinelibThread *tmp = new cXinelibServer(this, port);
       tmp->Start();
       m_clients.Add(m_server = tmp);
 
@@ -573,10 +598,6 @@ void cXinelibDevice::StopOutput(void)
   Clear();
   ForEach(m_clients, &cXinelibThread::QueueBlankDisplay);
   ForEach(m_clients, &cXinelibThread::SetNoVideo, false);
-#if VDRVERSNUM < 10515
-  ClrAvailableDvdSpuTracks();
-  m_ForcedDvdSpuTrack = false;
-#endif
 }
 
 void cXinelibDevice::SetTvMode(cChannel *Channel)
@@ -596,7 +617,6 @@ void cXinelibDevice::SetTvMode(cChannel *Channel)
   m_TrickSpeed = -1;
   m_SkipAudio  = false;
   m_AudioCount = 0;
-  m_spuPresent = false;
 
   Clear();
   ForEach(m_clients, &cXinelibThread::SetNoVideo, m_RadioStream);
@@ -611,6 +631,7 @@ void cXinelibDevice::SetReplayMode(void)
   m_RadioStream = true; // first seen replayed video packet resets this
   m_AudioCount  = 15;
   m_StreamStart = true;
+  m_TrickSpeed = -1;
 
   ForEach(m_clients, &cXinelibThread::SetLiveMode, false);
   Clear();
@@ -643,12 +664,6 @@ bool cXinelibDevice::SetPlayMode(ePlayMode PlayMode)
   }
 #endif
 
-  m_ac3Present = false;
-  m_spuPresent = false;
-
-#if VDRVERSNUM < 10515
-  ClrAvailableDvdSpuTracks();
-#endif
   m_PlayMode = PlayMode;
 
   TrickSpeed(-1);
@@ -675,28 +690,13 @@ bool cXinelibDevice::SetPlayMode(ePlayMode PlayMode)
 // m_TrickSpeedMode flags
 #define trs_IPB_frames 0x01  // stream has all frame types
 #define trs_I_frames   0x02  // stream has only I-frames
-#define trs_PTS_recalc 0x04  // PTS must be re-calculated
 #define trs_NoAudio    0x08  // no audio in trick speed mode
-#define trs_PTS_check  0x80  // detect in PlayVideo if PTS must be recalculated
-
-bool cXinelibDevice::HasIBPTrickSpeed(void)
-{
-  TRACEF("cXinelibDevice::HasIBPTrickSpeed");
-
-#ifndef DEVICE_SUPPORTS_IBP_TRICKSPEED
-#warning VDR has not been patched for smooth fast forward. Disabling smooth fast forward.
-  return false;
+#if VDRVERSNUM < 10705
+#define trs_PTS_recalc 0x10  // PTS must be re-calculated
+#define trs_PTS_check  0x20  // detect in PlayVideo if PTS must be recalculated
 #else
-  m_VDR_TrickSpeedIBP = true;
-
-  return xc.ibp_trickspeed;
+#define trs_Backward   0x40  // palying backwards -- same as regen pts ???
 #endif
-}
-
-bool cXinelibDevice::UseIBPTrickSpeed(void) 
-{ 
-  return m_VDR_TrickSpeedIBP && xc.ibp_trickspeed; 
-}
 
 void cXinelibDevice::TrickSpeed(int Speed) 
 {
@@ -706,9 +706,11 @@ void cXinelibDevice::TrickSpeed(int Speed)
     int RealSpeed = abs(Speed);
     LOGTRICKSPEED("TrickSpeed changed from %d to %d [%d]", m_TrickSpeed, Speed, RealSpeed);
 
-    m_TrickSpeedPts = 0;
     m_TrickSpeed = Speed;
+#if VDRVERSNUM < 10705
+    m_TrickSpeedPts = 0;
     m_TrickSpeedDelay = 0;
+#endif
 
     //  Possible transitions:
     //     fast <-> play
@@ -739,31 +741,55 @@ void cXinelibDevice::TrickSpeed(int Speed)
       //  ForEach(m_clients, &cXinelibThread::Clear);
       //}
 
+#if VDRVERSNUM < 10705
       // only I-frames, backwards, pts must be re-generated
       m_TrickSpeedMode = trs_I_frames | trs_PTS_recalc | trs_NoAudio;
 
       // change decoder and UDP/RTP scheduler clock rates
       ForEach(m_clients, &cXinelibThread::TrickSpeed, RealSpeed);
+#else
+      m_TrickSpeedMode = trs_I_frames | trs_Backward | trs_NoAudio;
+      ForEach(m_clients, &cXinelibThread::TrickSpeed, RealSpeed, true);
+#endif
     }
 
     else if(Speed == 6 || Speed == 3 || Speed == 1) {
       RealSpeed = 12/Speed;
       LOGTRICKSPEED("    Fast (%dx speed), direction unknown", RealSpeed);
 
+      if (m_StreamStart) {
+        LOGTRICKSPEED("    Fast (%dx speed), STREAM START --> BACKWARDS", RealSpeed);
+      }
+
       if(RealSpeed > xc.max_trickspeed) {
 	RealSpeed = xc.max_trickspeed;
 	LOGTRICKSPEED("    Trick speed limited to %dx speed", RealSpeed);
       }
 
+#if VDRVERSNUM < 10705
       /* only I-frames, backwards, pts must be re-generated if playing backwards */
       m_TrickSpeedMode |= trs_PTS_check;
 
       /* backward/forward state is unknown until first PTS is seen 
 	 so, clear() must be done in PlayVideo. */
       /* previous trick speed state is not overwritten yet ... ! */
-      
+
       // change decoder and UDP/RTP scheduler clock rates
       ForEach(m_clients, &cXinelibThread::TrickSpeed, -RealSpeed);
+#else
+      if (m_StreamStart || (m_TrickSpeedMode & trs_Backward)) {
+        m_TrickSpeedMode |= trs_I_frames | trs_Backward | trs_NoAudio;
+
+        ForEach(m_clients, &cXinelibThread::TrickSpeed, -RealSpeed, true);
+        ForEach(m_clients, &cXinelibThread::Sync);
+
+      } else {
+
+        m_TrickSpeedMode |= trs_IPB_frames;
+
+        ForEach(m_clients, &cXinelibThread::TrickSpeed, -RealSpeed);
+      }
+#endif
     }
 
     else if(Speed==-1 || Speed == 0) {
@@ -786,13 +812,16 @@ void cXinelibDevice::TrickSpeed(int Speed)
       //  LOGMSG("    -> Clear");
       //  ForEach(m_clients, &cXinelibThread::Clear);
       //}
+
       m_TrickSpeedMode = 0;
-    } 
+    }
 
     else {
       LOGTRICKSPEED("    Unknown trickspeed %d !", Speed);
+
       m_TrickSpeedMode = 0;
       m_TrickSpeed = -1;
+
       ForEach(m_clients, &cXinelibThread::TrickSpeed, -1);
     }
   }
@@ -803,8 +832,9 @@ void cXinelibDevice::Clear(void)
   TRACEF("cXinelibDevice::Clear");
   TRACK_TIME(100);
 
+  TsBufferClear();
+
   m_StreamStart = true;
-  m_h264 = false;
   m_FreeBufs = 0;
   TrickSpeed(-1);
   ForEach(m_clients, &cXinelibThread::Clear);
@@ -816,16 +846,20 @@ void cXinelibDevice::Play(void)
   TRACEF("cXinelibDevice::Play");
 
   m_SkipAudio  = false;
-  
+
   ForEach(m_clients, &cXinelibThread::SetLiveMode, false);
+  ForEach(m_clients, &cXinelibThread::SetStillMode, false);
   TrickSpeed(-1);
+  cDevice::Play();
 }
 
 void cXinelibDevice::Freeze(void) 
 {
   TRACEF("cXinelibDevice::Freeze");
 
+  TsBufferFlush();
   TrickSpeed(0);
+  cDevice::Freeze();
 }
 
 int64_t cXinelibDevice::GetSTC(void)
@@ -844,6 +878,8 @@ bool cXinelibDevice::Flush(int TimeoutMs)
   TRACEF("cXinelibDevice::Flush");
   TRACK_TIME(500);
 
+  TsBufferFlush();
+
   if(m_TrickSpeed == 0) {
     ForEach(m_clients, &cXinelibThread::SetLiveMode, false);
     TrickSpeed(-1);
@@ -862,7 +898,7 @@ bool cXinelibDevice::Flush(int TimeoutMs)
 
 int cXinelibDevice::PlayFileCtrl(const char *Cmd, int TimeoutMs)
 {
-  TRACEF("cXinelibDevice::PlayFile");
+  TRACEF("cXinelibDevice::PlayFileCtrl");
   int result = -1;
 
   /*if(m_PlayingFile != pmNone)*/ {
@@ -925,6 +961,7 @@ bool cXinelibDevice::PlayFile(const char *FileName, int Position,
 
 int cXinelibDevice::PlayTrickSpeed(const uchar *buf, int length) 
 {
+#if VDRVERSNUM < 10705
   if(abs(m_TrickSpeed) > 1 && (m_TrickSpeedMode & trs_I_frames)) {
     uint8_t PictureType = pes_get_picture_type(buf, length);
 #ifdef LOG_TRICKSPEED
@@ -993,7 +1030,7 @@ int cXinelibDevice::PlayTrickSpeed(const uchar *buf, int length)
 	  m_TrickSpeedMode = trs_I_frames | trs_PTS_recalc | trs_NoAudio;	  
 	} else {
 	  LOGTRICKSPEED("    Detected fast forward mode");
-          if(UseIBPTrickSpeed())
+          if(xc.ibp_trickspeed)
             m_TrickSpeedMode = trs_IPB_frames;
           else
             m_TrickSpeedMode = trs_I_frames;
@@ -1035,6 +1072,8 @@ int cXinelibDevice::PlayTrickSpeed(const uchar *buf, int length)
   }
 #endif
 
+#endif /* VDRVERSNUM < 10705 */
+
   return 0;
 }
 
@@ -1088,11 +1127,11 @@ int cXinelibDevice::PlayAny(const uchar *buf, int length)
 
   if(m_local) {
     length = (isMpeg1 ? m_local->Play_Mpeg1_PES(buf,length) : 
-	                m_local->Play_PES(buf,length));
+                        m_local->Play(buf, length));
   } 
   if(m_server && length > 0) {
     int length2 = isMpeg1 ? m_server->Play_Mpeg1_PES(buf, length) : 
-                            m_server->Play_PES(buf, length);   
+                            m_server->Play(buf, length);
     if(!m_local)
       return length2;
   }
@@ -1100,12 +1139,205 @@ int cXinelibDevice::PlayAny(const uchar *buf, int length)
   return length;
 }
 
+#if VDRVERSNUM >= 10701 || defined(TSPLAY_PATCH_VERSION)
+/*
+ * hook to PlayTs() to get PAT and PMT
+ */
+int cXinelibDevice::PlayTs(const uchar *Data, int Length, bool VideoOnly)
+{
+  if (!Data || Length < TS_SIZE) {
+    TsBufferFlush();
+    return cDevice::PlayTs(Data, Length, VideoOnly);
+  }
+
+  /* Play single TS pack */
+  int Result = cDevice::PlayTs(Data, TS_SIZE, VideoOnly);
+  if (Result != TS_SIZE)
+    return Result;
+
+  /* Grab PAT and PMT */
+  if (TsHasPayload(Data) && TsPayloadOffset(Data) < TS_SIZE) {
+
+    int Pid = TsPid(Data);
+#if VDRVERSNUM < 10733
+    if (Pid == 0 || Pid == PatPmtParser()->PmtPid()) {
+#else
+    if (Pid == PATPID || PatPmtParser()->IsPmtPid(Pid)) {
+#endif
+      if (m_server)
+        m_server->SetHeader(Data, Result, Pid == 0);
+
+      if (PlayTsAny(Data, Result) != Result)
+        LOGMSG("Lost PAT/PMT fragment !");
+
+      TsBufferFlush();
+
+      /* detect radio streams */
+      int patv, pmtv;
+      if (PatPmtParser()->GetVersions(patv, pmtv)) {
+        if (!PatPmtParser()->Vpid() && PatPmtParser()->Apid(0)) {
+          m_RadioStream = true;
+          m_AudioCount  = 0;
+          ForEach(m_clients, &cXinelibThread::SetNoVideo, m_RadioStream);
+        }
+      }
+    }
+  }
+
+  return Result;
+}
+
+/*
+ * TS buffer
+ */
+
+#define TS_BUFFER_LOCK cSectionLock SectionLock(m_TsBufLock)
+
+int cXinelibDevice::TsBufferFlush(void)
+{
+  TS_BUFFER_LOCK;
+
+  if (m_TsBufSize) {
+    int n;
+    if ((n = PlayAny(m_TsBuf, m_TsBufSize)) == (int)m_TsBufSize) {
+      m_TsBufSize = 0;
+      return n;
+    }
+    if (n)
+      LOGMSG("cXinelibDevice::TsBufferFlush: error: cache not flushed (%d %d)", n, m_TsBufSize);
+    errno = EAGAIN;
+  }
+  return 0;
+}
+
+void cXinelibDevice::TsBufferClear(void)
+{
+  TS_BUFFER_LOCK;
+  m_TsBufSize = 0;
+}
+
+int cXinelibDevice::PlayTsAny(const uchar *buf, int length)
+{
+  if (!DATA_IS_TS(buf))
+    LOGMSG("PlayTsAny(): TS SYNC byte missing !");
+  if (length != TS_SIZE)
+    LOGMSG("PlayTsAny(): length == %d !", length);
+
+  TS_BUFFER_LOCK;
+
+  // cache full ? try to flush it
+  if (m_TsBufSize >= 2048)
+    if (!TsBufferFlush())
+      return 0;
+
+  // add packet to cache
+  memcpy(m_TsBuf + m_TsBufSize, buf, length);
+  m_TsBufSize += length;
+
+  // time to flush ?
+  if (m_TsBufSize >= 2048-TS_SIZE-1)
+    TsBufferFlush();
+
+  return length;
+}
+
+int cXinelibDevice::PlayTsSubtitle(const uchar *Data, int Length)
+{
+  if (!xc.dvb_subtitles)
+    return cDevice::PlayTsSubtitle(Data, Length);
+
+  return PlayTsAny(Data, Length);
+}
+
+int cXinelibDevice::PlayTsAudio(const uchar *Data, int Length)
+{
+  if (!AcceptAudioPacket(Data, Length))
+    return Length;
+
+  return PlayTsAny(Data, Length);
+}
+
+int cXinelibDevice::PlayTsVideo(const uchar *Data, int Length)
+{
+  if (ts_PID(Data) == PatPmtParser()->Vpid()) {
+
+    if (!AcceptVideoPacket(Data, Length))
+      return Length;
+
+    if (m_StreamStart) {
+      if (!m_tssVideoSize)
+        m_tssVideoSize = ts_state_init(4096);
+
+      if (ts_get_video_size(m_tssVideoSize, Data, m_VideoSize,
+                            (PatPmtParser()->Vtype() == ISO_14496_PART10_VIDEO))) {
+
+        m_StreamStart = false;
+        LOGMSG("Detected video size %dx%d", m_VideoSize->width, m_VideoSize->height);
+        ForEach(m_clients, &cXinelibThread::SetHDMode, (m_VideoSize->width > 800));
+
+        ts_state_dispose(m_tssVideoSize);
+        m_tssVideoSize = NULL;
+      }
+    }
+  }
+
+  return PlayTsAny(Data, Length);
+}
+#endif // VDRVERSNUM >= 10701 || defined(TSPLAY_PATCH_VERSION)
+
+bool cXinelibDevice::AcceptVideoPacket(const uchar *Data, int Length)
+{
+  if (!Data || Length < 6)
+    return false;
+
+  if (m_PlayMode == pmAudioOnlyBlack)
+    return false;
+
+  if (m_PlayingFile && (m_PlayingFile == pmAudioVideo || m_PlayingFile == pmVideoOnly))
+    return false;
+
+  if (m_RadioStream) {
+    m_RadioStream = false;
+    m_AudioCount  = 0;
+    ForEach(m_clients, &cXinelibThread::SetNoVideo, m_RadioStream);
+  }
+
+  return true;
+}
+
+bool cXinelibDevice::AcceptAudioPacket(const uchar *Data, int Length)
+{
+  if (!Data || Length < 6)
+    return false;
+
+  if (m_PlayingFile && (m_PlayingFile == pmAudioVideo || m_PlayingFile == pmAudioOnly))
+    return false;
+
+  // strip audio in trick speed modes and when displaying still images
+  if (m_SkipAudio)
+    return false;
+  if (m_TrickSpeedMode & trs_NoAudio)
+    return false;
+
+  if (m_RadioStream) {
+    if (m_AudioCount > 0) {
+      m_AudioCount--;
+      if (m_AudioCount <= 0) {
+	LOGDBG("PlayAudio detected radio stream");
+	ForEach(m_clients, &cXinelibThread::SetNoVideo, m_RadioStream);
+      }
+    }
+  }
+
+  return true;
+}
+
 int cXinelibDevice::PlayVideo(const uchar *buf, int length) 
 {
   TRACEF("cXinelibDevice::PlayVideo");
   TRACK_TIME(100);
 
-  if(m_PlayMode == pmAudioOnlyBlack)
+  if (!AcceptVideoPacket(buf, length))
     return length;
 
   if (!DATA_IS_PES(buf)) {
@@ -1113,29 +1345,13 @@ int cXinelibDevice::PlayVideo(const uchar *buf, int length)
     return length;
   }
 
-  if(m_RadioStream) {
-    m_RadioStream = false;
-    m_AudioCount  = 0;
-    ForEach(m_clients, &cXinelibThread::SetNoVideo, m_RadioStream);
-  }
-
   if(m_StreamStart) {
-#ifdef START_IFRAME
-    // Start with I-frame if stream has video
-    // wait for first I-frame
-    if (pes_get_picture_type(buf, length) == I_FRAME) {
-      m_StreamStart = false;
-    } else {
-      return length;
-    }
-#endif
-
-    if (!m_h264 && pes_is_frame_h264(buf, length)) {
+    bool h264 = pes_is_frame_h264(buf, length);
+    if (h264) {
       LOGMSG("cXinelibDevice::PlayVideo: Detected H.264 video");
-      m_h264 = true;
     }
 
-    if (pes_get_video_size(buf, length, m_VideoSize, m_h264 ? 1:0)) {
+    if (pes_get_video_size(buf, length, m_VideoSize, h264)) {
       m_StreamStart = false;
       LOGDBG("Detected video size %dx%d", m_VideoSize->width, m_VideoSize->height);
       ForEach(m_clients, &cXinelibThread::SetHDMode, (m_VideoSize->width > 800));
@@ -1149,9 +1365,6 @@ int cXinelibDevice::PlayVideo(const uchar *buf, int length)
   }
 #endif
 
-  if(m_PlayingFile && (m_PlayingFile == pmAudioVideo || m_PlayingFile == pmVideoOnly))
-    return length;
-
   return PlayAny(buf, length);
 }
 
@@ -1177,16 +1390,23 @@ void cXinelibDevice::StillPicture(const uchar *Data, int Length)
   bool isPes   = DATA_IS_PES(Data) && ((Data[3] & 0xF0) == 0xE0);
   bool isMpeg1 = isPes && ((Data[6] & 0xC0) != 0x80);
   bool isH264  = isPes && pes_is_frame_h264(Data, Length);
+#if VDRVERSNUM >= 10701 || defined(TSPLAY_PATCH_VERSION)
+  bool isTs    = DATA_IS_TS(Data);
+#endif
+
   int i;
 
   if(m_PlayingFile && (m_PlayingFile == pmAudioVideo || m_PlayingFile == pmVideoOnly))
     return;
 
+  TsBufferFlush();
+
   ForEach(m_clients, &cXinelibThread::Clear);
   ForEach(m_clients, &cXinelibThread::SetNoVideo, false);
   ForEach(m_clients, &cXinelibThread::SetLiveMode, false);
+  ForEach(m_clients, &cXinelibThread::ResumeOutput);
   ForEach(m_clients, &cXinelibThread::SetStillMode, true);
-  ForEach(m_clients, &cXinelibThread::TrickSpeed, 1);
+  ForEach(m_clients, &cXinelibThread::Sync);
 
   m_TrickSpeed = -1; // to make Poll work ...
   m_SkipAudio = 1;   // enables audio and pts stripping
@@ -1197,21 +1417,34 @@ void cXinelibDevice::StillPicture(const uchar *Data, int Length)
 	      &mmin<int>, Length);
     } else if(isPes) {
       /*cDevice::*/PlayPes(Data, Length, m_SkipAudio);
+#if VDRVERSNUM >= 10701 || defined(TSPLAY_PATCH_VERSION)
+    } else if(isTs) {
+      int written = 0, total = (Length/TS_SIZE)*TS_SIZE;
+      while (written < total) {
+        int n = PlayTs(Data+written, Length-written, m_SkipAudio);
+        if (n > 0)
+          written += n;
+        else
+          cCondWait::SleepMs(5);
+      }
+      TsBufferFlush();
+#endif
     } else {
-      ForEach(m_clients, &cXinelibThread::Play_Mpeg2_ES, 
-	      Data, Length, VIDEO_STREAM,
-	      &mand<bool>, true);
+      ForEach(m_clients, &cXinelibThread::Play_Mpeg2_ES,
+              Data, Length, VIDEO_STREAM, isH264,
+              &mand<bool>, true);
     }
 
-  if(!isH264) {
-    // creates empty video PES with pseudo-pts
-    ForEach(m_clients, &cXinelibThread::Play_Mpeg2_ES,
-	    Data, 0, VIDEO_STREAM,
-	    &mand<bool>, true);
-  }
+  TsBufferFlush();
 
-  ForEach(m_clients, &cXinelibThread::Flush, 60, 
-	  &mand<bool>, true);
+#if 0
+  // creates empty video PES with pseudo-pts
+  ForEach(m_clients, &cXinelibThread::Play_Mpeg2_ES,
+          Data, 0, VIDEO_STREAM, isH264,
+          &mand<bool>, true);
+#endif
+
+  ForEach(m_clients, &cXinelibThread::Flush, 60, &mand<bool>, true);
 
   m_TrickSpeed = 0;
   m_SkipAudio = 0;
@@ -1222,77 +1455,18 @@ int cXinelibDevice::PlayAudio(const uchar *buf, int length, uchar Id)
   TRACEF("cXinelibDevice::PlayAudio");
   TRACK_TIME(100);
 
-  if(!buf || length < 6)
-    return length;
-
-#ifdef SKIP_AC3_AUDIO
-    // skip AC3 audio
-  if(((unsigned char *)buf)[3] == PRIVATE_STREAM1) {
-    TRACE("cXinelibDevice::PlayVideo: PRIVATE_STREAM1 discarded");
-    return length;
-  }
-#endif
-
-  // strip audio in trick speed modes and when displaying still images
-  if(m_SkipAudio /*|| m_TrickSpeed > 0*/)
-    return length;
-  if(m_TrickSpeedMode & trs_NoAudio)
-    return length;
-
-  if(m_RadioStream) {
-    if(m_AudioCount) {
-      m_AudioCount--;
-      if(!m_AudioCount) {
-	LOGDBG("PlayAudio detected radio stream");
-	ForEach(m_clients, &cXinelibThread::SetNoVideo, m_RadioStream);
-      }
-    }
-  }
-
-  if(m_PlayingFile && (m_PlayingFile == pmAudioVideo || m_PlayingFile == pmAudioOnly))
+  if (!AcceptAudioPacket(buf, length))
     return length;
 
   return PlayAny(buf, length);
 }
 
-#if VDRVERSNUM < 10510
-int cXinelibDevice::PlaySubtitle(const uchar *buf, int length) 
-{
-  TRACEF("cXinelibDevice::PlaySpu");
-
-  if(!buf || length < 6)
-    return length;
-
-  if(((unsigned char *)buf)[3] == PRIVATE_STREAM1) {
-
-    int PayloadOffset = buf[8] + 9;
-    uchar SubStreamId = buf[PayloadOffset];
-    //uchar SubStreamType = SubStreamId & 0xF0;
-    uchar SubStreamIndex = SubStreamId & 0x1F;
-
-    if(!m_spuPresent) {
-      TRACE("cXinelibDevice::PlaySpu first DVD SPU frame");
-      Skins.QueueMessage(mtInfo,"DVD Subtitles");
-      m_spuPresent = true;
-      
-      ForEach(m_clients, &cXinelibThread::SpuStreamChanged, (int)SubStreamIndex);
-    }
-
-    // Strip all but selected SPU track
-    if(SubStreamIndex != m_CurrentDvdSpuTrack)
-      return length;
-  }
-
-  return PlayAny(buf, length);
-}
-#else
 int cXinelibDevice::PlaySubtitle(const uchar *Data, int Length)
 {
   if(!xc.dvb_subtitles)
     return cDevice::PlaySubtitle(Data, Length);
   return PlayAny(Data, Length);
 }
-#endif
 
 bool cXinelibDevice::Poll(cPoller &Poller, int TimeoutMs) 
 {
@@ -1314,12 +1488,14 @@ bool cXinelibDevice::Poll(cPoller &Poller, int TimeoutMs)
     return true;
   }
 
+#if VDRVERSNUM < 10705
   if(m_TrickSpeed > 1 && m_TrickSpeedDelay > 20) {
     LOGTRICKSPEED("    Poll: m_TrickSpeedDelay=%d.", m_TrickSpeedDelay);
     cCondWait::SleepMs(20);
     m_TrickSpeedDelay -= 20;
     return false;
   }
+#endif
 
   if(m_FreeBufs < 1) {
     int result = DEFAULT_POLL_SIZE;
@@ -1451,6 +1627,19 @@ eVideoSystem cXinelibDevice::GetVideoSystem(void)
   return cDevice::GetVideoSystem();
 }
 
+#if VDRVERSNUM >= 10708
+void cXinelibDevice::GetVideoSize(int &Width, int &Height, double &VideoAspect)
+{
+  Width  = m_VideoSize->width;
+  Height = m_VideoSize->height;
+  VideoAspect = 1.0;
+  if (m_VideoSize->pixel_aspect.den) {
+    VideoAspect = (double)m_VideoSize->pixel_aspect.num / (double)m_VideoSize->pixel_aspect.den;
+    VideoAspect *= (double)Width / (double)Height;
+  }
+}
+#endif
+
 void cXinelibDevice::GetOsdSize(int &Width, int &Height, double &PixelAspect)
 {
   switch (xc.osd_size) {
@@ -1466,6 +1655,12 @@ void cXinelibDevice::GetOsdSize(int &Width, int &Height, double &PixelAspect)
       Width  = 1920;
       Height = 1080;
       break;
+    case OSD_SIZE_stream:
+      if (m_VideoSize->width > 0 && m_VideoSize->height > 0) {
+        Width  = max<int>(m_VideoSize->width,  512);
+        Height = max<int>(m_VideoSize->height, 480);
+        break;
+      }
     case OSD_SIZE_auto:
       if (xc.osd_width_auto > 0 && xc.osd_height_auto > 0) {
         Width  = xc.osd_width_auto;
@@ -1481,6 +1676,30 @@ void cXinelibDevice::GetOsdSize(int &Width, int &Height, double &PixelAspect)
   PixelAspect = 16.0 / 9.0 / (double)Width * (double)Height;
 }
 
+bool cXinelibDevice::SupportsTrueColorOSD(void)
+{
+  switch (xc.osd_color_depth) {
+    default:
+    case OSD_DEPTH_LUT8:      return false;
+    case OSD_DEPTH_TRUECOLOR: return true;
+    case OSD_DEPTH_auto:;
+  }
+
+  // Automatic mode:
+  // Use TrueColor if all clients support it.
+  // Special handling when no remote clients:
+  //  - local frontend supports TrueColor --> use TrueColor
+  //  - no local frontend --> use LUT8
+
+  if (m_local) {
+    if (!m_local->SupportsTrueColorOSD())
+      return false;
+    return !m_server || !!m_server->SupportsTrueColorOSD();
+  }
+
+  return m_server && (m_server->SupportsTrueColorOSD() == 1);
+}
+
 //
 // SPU decoder
 //
@@ -1519,9 +1738,6 @@ uchar *cXinelibDevice::GrabImage(int &Size, bool Jpeg,
 {
   TRACEF("cXinelibDevice::GrabImage");
 
-  if (Quality < 0)
-    Quality = 100;
-
   if(m_local)
     return m_local->GrabImage(Size, Jpeg, Quality, SizeX, SizeY);
   if(m_server)
@@ -1531,178 +1747,15 @@ uchar *cXinelibDevice::GrabImage(int &Size, bool Jpeg,
 }
 
 
-//
-// DVD SPU support in VDR recordings
-//
-//   - override cDevice::PlayPesPacket to get DVD SPUs
-//
-
-#if VDRVERSNUM < 10510
-int cXinelibDevice::PlayPesPacket(const uchar *Data, int Length, 
-				  bool VideoOnly)
-{
-  switch (Data[3]) {
-    case 0xBD: { // private stream 1
-      int PayloadOffset = Data[8] + 9;
-      uchar SubStreamId = Data[PayloadOffset];
-      uchar SubStreamType = SubStreamId & 0xF0;
-      uchar SubStreamIndex = SubStreamId & 0x1F;
-      switch (SubStreamType) {
-        case 0x20: // SPU
-        case 0x30: // SPU
-	  SetAvailableDvdSpuTrack(SubStreamIndex);
-	  return PlaySubtitle(Data, Length);
-	  break;
-        default:
-	  ;
-      }
-    }
-    default:
-      ;
-  }
-  return cDevice::PlayPesPacket(Data, Length, VideoOnly);
-}
-#endif
-
 //
 // Available DVD SPU tracks
 //
 
-#if VDRVERSNUM < 10515
-bool cXinelibDevice::SetCurrentDvdSpuTrack(int Type, bool Force)
-{
-  if(Type == ttXSubtitleNone || 
-     ( Type >= 0 && 
-       Type < 64 &&
-       m_DvdSpuTrack[Type].id != 0xffff)) {
-    m_CurrentDvdSpuTrack = Type;
-    ForEach(m_clients, &cXinelibThread::SpuStreamChanged, Type);
-    if(Force)
-      m_ForcedDvdSpuTrack = true;
-    return true;
-  }
-  return false;
-}
-#endif
-
-#if VDRVERSNUM >= 10515
 void cXinelibDevice::SetSubtitleTrackDevice(eTrackType Type)
 {
   if (m_PlayingFile == pmAudioVideo || m_PlayingFile == pmVideoOnly)
     ForEach(m_clients, &cXinelibThread::SetSubtitleTrack, Type);
 }
-#endif
-
-#if VDRVERSNUM < 10515
-void cXinelibDevice::ClrAvailableDvdSpuTracks(bool NotifyFrontend)
-{
-  for(int i=0; i<64; i++)
-    m_DvdSpuTrack[i].id = 0xffff;
-  if(m_CurrentDvdSpuTrack >= 0 ) {
-    m_CurrentDvdSpuTrack = ttXSubtitleNone;
-    if(NotifyFrontend)
-      ForEach(m_clients, &cXinelibThread::SpuStreamChanged, m_CurrentDvdSpuTrack);
-  }
-}
-#endif
-
-#if VDRVERSNUM < 10515
-int cXinelibDevice::NumDvdSpuTracks(void) const
-{ 
-  int DvdSpuTracks = 0;
-  for(int i=0; i<64; i++)
-    if(m_DvdSpuTrack[i].id != 0xffff)
-      DvdSpuTracks++;
-  return DvdSpuTracks; 
-}
-#endif
-
-#if VDRVERSNUM < 10515
-const tTrackId *cXinelibDevice::GetDvdSpuTrack(int Type) const
-{
-  if(Type >= 0 && Type < 64 &&
-     m_DvdSpuTrack[Type].id != 0xffff)
-    return &m_DvdSpuTrack[Type];
-  return NULL;
-}
-#endif
-
-#if VDRVERSNUM < 10515
-const char *cXinelibDevice::GetDvdSpuLang(int Type) const
-{
-  const tTrackId *track = GetDvdSpuTrack(Type);
-  if(track)
-    return track->language[0] ? track->language : NULL;
-  return NULL;
-}
-#endif
-
-#if VDRVERSNUM < 10515
-bool cXinelibDevice::SetAvailableDvdSpuTrack(int Type, const char *lang, bool Current)
-{
-  if(Type >= 0 && Type < 64) {
-
-    m_DvdSpuTrack[Type].id = Type;
-    m_DvdSpuTrack[Type].language[0] = '\0';
-    if(lang) 
-      strn0cpy(m_DvdSpuTrack[Type].language, lang, MAXLANGCODE2);
-    if(Current)
-      m_CurrentDvdSpuTrack = Type;
-
-    return true;
-  }
-  return false;
-}
-#endif
-
-#if VDRVERSNUM < 10515
-void cXinelibDevice::EnsureDvdSpuTrack(void)
-{
-  if(!m_ForcedDvdSpuTrack &&
-     NumDvdSpuTracks() > 0 &&
-     (m_DvdSpuTrack[0].id == 0xffff ||
-      strcmp(m_DvdSpuTrack[0].language, "menu"))) {
-
-    if(xc.spu_autoshow) {
-      int pref, track;
-      size_t len;
-      for(pref = 0; pref < 4; pref++)
-	for(track = 0; track < 64; track++)
-	  if(m_DvdSpuTrack[track].id != 0xffff)    
-	    if((len=strlen(xc.spu_lang[pref])) > 0)
-	      //if(!strncmp(m_DvdSpuTrack[track].language,
-	      //            xc.spu_lang[pref], len)) {
-	      if(!strcmp(m_DvdSpuTrack[track].language,
-			 xc.spu_lang[pref])) {
-		if(m_CurrentDvdSpuTrack != track) {
-		  LOGMSG("Auto-selecting %d. SPU track \'%s\'  (%d. preferred language is \'%s\')", 
-			 track, m_DvdSpuTrack[track].language, pref+1, xc.spu_lang[pref]);
-		  cXinelibDevice::SetCurrentDvdSpuTrack(track);
-		  cString msg = cString::sprintf("Subtitles: %s", m_DvdSpuTrack[track].language);
-		  Skins.QueueMessage(mtInfo, msg);
-		}
-		m_spuPresent = true;
-		track = 64;
-		pref = 99;
-	      }
-      if(pref < 99) {
-	for(track = 0; track < 64; track++)
-	  if(m_DvdSpuTrack[track].id != 0xffff)
-	    if(m_CurrentDvdSpuTrack != track) {
-	      LOGMSG("Auto-selecting (non-matching) %d. SPU track \'%s\'",
-		     track, m_DvdSpuTrack[track].language);
-	      cXinelibDevice::SetCurrentDvdSpuTrack(track);
-	    }
-      }
-    }
-
-    if(!m_spuPresent) {
-      Skins.QueueMessage(mtInfo, "Subtitles present");
-      m_spuPresent = true;
-    }
-  }
-}
-#endif
 
 //
 // Metainfo
@@ -1737,3 +1790,59 @@ void  cXinelibDevice::SetMetaInfo(eMetainfoType Type, const char *Value)
   }
 }
 
+//
+// Picture-In-Picture
+//
+
+int cXinelibDevice::Pip_Open(void)
+{
+  for (int i = 0; i < MAX_NUM_PIP; i++)
+    if (!m_PipPid[i]) {
+      m_PipPid[i] = i;
+      ForEach(m_clients, &cXinelibThread::Pip_Config, i, -1, -1, -1, -1);
+      return i;
+    }
+  return -1;
+}
+
+void cXinelibDevice::Pip_Config(int Index, int X, int Y, int W, int H)
+{
+  if (Index >= 0 && Index < MAX_NUM_PIP) {
+    if (m_PipPid[Index]) {
+      ForEach(m_clients, &cXinelibThread::Pip_Config, Index, X, Y, W, H);
+    }
+  }
+}
+
+int  cXinelibDevice::Pip_Play(int Index, uint8_t *Data, int Length)
+{
+  if (Index >= 0 && Index < MAX_NUM_PIP) {
+    if (m_PipPid[Index] && Data && Length > 0) {
+
+      int len = Length;
+
+      if (m_local)
+        len = m_local->Play(Data, len, eStreamId(sidPipFirst + Index));
+
+      if (m_server) {
+        int len2 = m_server->Play(Data, len, eStreamId(sidPipFirst + Index));
+        if (!m_local)
+          return len2;
+      }
+
+      return len;
+    }
+  }
+  return Length;
+}
+
+void cXinelibDevice::Pip_Close(int Index)
+{
+  if (Index >= 0 && Index < MAX_NUM_PIP) {
+    if (m_PipPid[Index]) {
+      ForEach(m_clients, &cXinelibThread::Pip_Close, Index);
+      m_PipPid[Index] = 0;
+    }
+  }
+}
+
