@@ -4,7 +4,7 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: frontend.c,v 1.59.2.6 2009/09/14 10:10:20 phintuka Exp $
+ * $Id: frontend.c,v 1.95 2011/03/19 17:20:53 phintuka Exp $
  *
  */
 
@@ -57,8 +57,8 @@
 //
 
 /*static*/
-void cXinelibThread::KeypressHandler(const char *keymap, const char *key, 
-				     bool repeat, bool release)
+void cXinelibThread::KeypressHandler(const char *keymap, const char *key,
+                                     bool repeat, bool release)
 {
 #ifdef XINELIBOUTPUT_LOG_KEYS
   static FILE *flog = fopen("/video/keys.log","w");
@@ -69,24 +69,46 @@ void cXinelibThread::KeypressHandler(const char *keymap, const char *key,
 
   TRACE("keypress_handler: " << (keymap?keymap:"") << " " << key);
 
-  if(!key)
+  // check if key exists.
+  // Note: empty key ("") is used to trigger learning; it only creates the cRemote object.
+  if (!key)
     return;
 
-  if(keymap) {
-    cRemote *item = Remotes.First();
-    while(item) {
-      if(!strcmp(item->Name(), keymap)) {
-	// dirty... but only way to support learning ...
-	((cGeneralRemote*)item)->Put(key, repeat, release);
-	return;
-      }
-      item = Remotes.Next(item);
-    }
-    cGeneralRemote *r = new cGeneralRemote(keymap);
-    if(*key)
-      r->Put(key, repeat, release);
-  } else {
+  if (!keymap) {
+    // raw VDR key
     cRemote::Put(cKey::FromString(key));
+    return;
+  }
+
+  // find correct remote
+  cGeneralRemote *remote = NULL;
+  for (cRemote *item = Remotes.First(); item; item = Remotes.Next(item)) {
+    if (!strcmp(item->Name(), keymap)) {
+      // dirty... but using protected cRemote::Put() is the only way to support learning ...
+      remote = (cGeneralRemote*)item;
+      break;
+    }
+  }
+
+  // not found ? create new one
+  if (!remote)
+    remote = new cGeneralRemote(keymap);
+
+  // put key to remote queue
+  if (key[0]) {
+    if (!remote->Put(key, repeat, release)) {
+      if (!strcmp(keymap, "KBD")) {
+        uint64_t value = 0;
+        sscanf(key, "%"PRIX64, &value);
+        if (value) {
+          remote->cRemote::Put(KBDKEY(value));
+          return;
+        }
+      }
+      if (!key[1]) {
+        remote->cRemote::Put(KBDKEY(key[0]));
+      }
+    }
   }
 }
 
@@ -109,9 +131,6 @@ void cXinelibThread::InfoHandler(const char *info)
   if(!strncmp(info, "TRACKMAP SPU", 12)) {
     int CurrentTrack = ttXSubtitleAuto;
     map += 12;
-#if VDRVERSNUM < 10515 && !defined(VDRSPUPATCH)
-    cXinelibDevice::Instance().ClrAvailableDvdSpuTracks(false);
-#endif
     while(*map) {
       bool Current = false;
       while(*map == ' ') map++;
@@ -131,25 +150,17 @@ void cXinelibThread::InfoHandler(const char *info)
 	char *lang = map;
 	while(*map && *map != ' ') map++;
 	if(*map == ' ') { *map = 0; map++; };
-#if VDRVERSNUM < 10515 && !defined(VDRSPUPATCH)
-	cXinelibDevice::Instance().SetAvailableDvdSpuTrack(id, iso639_2_to_iso639_1(lang), Current);
-#else
-	cXinelibDevice::Instance().SetAvailableTrack(ttSubtitle, id, id+1, iso639_2_to_iso639_1(lang) ?: *cString::sprintf("%03d", id+1));
+	cXinelibDevice::Instance().SetAvailableTrack(ttSubtitle, id, id+1, iso639_1_to_iso639_2(lang) ?: *cString::sprintf("%03d", id+1));
 	if (Current)
 	  CurrentTrack = id;
-#endif
       }
     }
-#if VDRVERSNUM < 10515 && !defined(VDRSPUPATCH)
-    cXinelibDevice::Instance().EnsureDvdSpuTrack();
-#else
     if (CurrentTrack == ttXSubtitleAuto)
       cXinelibDevice::Instance().EnsureSubtitleTrack();
     else if (CurrentTrack == ttXSubtitleNone)
       cXinelibDevice::Instance().SetCurrentSubtitleTrack(ttNone, true);
     else
       cXinelibDevice::Instance().SetCurrentSubtitleTrack(eTrackType(CurrentTrack+ttSubtitleFirst), true);
-#endif
   }
 
   else if(!strncmp(info, "TRACKMAP AUDIO", 14)) {
@@ -168,7 +179,7 @@ void cXinelibThread::InfoHandler(const char *info)
       char *lang = map;
       while(*map && *map != ' ') map++;
       if(*map == ' ') { *map = 0; map++; };
-      cXinelibDevice::Instance().SetAvailableTrack(ttDolby, id, ttDolby+id, iso639_2_to_iso639_1(lang) ?: *cString::sprintf("%03d", id+1));
+      cXinelibDevice::Instance().SetAvailableTrack(ttDolby, id, ttDolby+id, iso639_1_to_iso639_2(lang) ?: *cString::sprintf("%03d", id+1));
       if(Current)
 	cXinelibDevice::Instance().SetCurrentAudioTrack((eTrackType)(ttDolby+id));
     }
@@ -216,13 +227,8 @@ void cXinelibThread::InfoHandler(const char *info)
     map += 9;
     while(*map == ' ') map++;
     cXinelibDevice::Instance().SetMetaInfo(miDvdTitleNo, map);
-#if VDRVERSNUM < 10515 && !defined(VDRSPUPATCH)
-    if (*map == '0')  // DVD Menu, set spu track to 0
-      cXinelibDevice::Instance().SetCurrentDvdSpuTrack(0);
-#else
     if (*map == '0')  // DVD Menu, set spu track to 0
       cXinelibDevice::Instance().SetCurrentSubtitleTrack(ttSubtitleFirst);
-#endif
   }
 
   else if (!strncmp(info, "WINDOW ", 7)) {
@@ -245,10 +251,13 @@ cXinelibThread::cXinelibThread(const char *Description) : cThread(Description)
   m_Volume = 255;
   m_bReady = false;
   m_bNoVideo = true;
+  m_bHDMode = false;
   m_bLiveMode = true; /* can't be replaying when there is no output device */
   m_StreamPos = 0;
   m_LastClearPos = 0;
   m_Frames = 0;
+  m_TrickSpeed = -1;
+  m_bTrickSpeedBack = false;
   m_bEndOfStreamReached = false;
   m_bPlayingFile = false;
   m_StatusMonitor = NULL;
@@ -281,16 +290,22 @@ bool cXinelibThread::IsReady(void)
 void cXinelibThread::SetVolume(int NewVolume)
 {
   m_Volume = NewVolume;
-  cString str = cString::sprintf("VOLUME %d%s", NewVolume * 100 / 255, 
-				 xc.sw_volume_control ? " SW" : "");
+  cString str = cString::sprintf("VOLUME %d%s", NewVolume * 100 / 255,
+                                 xc.sw_volume_control ? " SW" : "");
   Xine_Control(str);
 }
 
-void cXinelibThread::TrickSpeed(int Speed)
+void cXinelibThread::TrickSpeed(int Speed, bool Backwards)
 {
   TRACEF("cXinelibThread::TrickSpeed");
 
-  Xine_Control("TRICKSPEED", Speed);
+  Lock();
+  m_TrickSpeed      = Speed;
+  m_bTrickSpeedBack = Backwards;
+  Unlock();
+
+  cString str = cString::sprintf("TRICKSPEED %d%s", Speed, Backwards ? " Backwards" : "");
+  Xine_Control(str);
 }
 
 void cXinelibThread::SetLiveMode(bool LiveModeOn)
@@ -328,12 +343,14 @@ void cXinelibThread::SetNoVideo(bool bVal)
 
   Xine_Control("NOVIDEO", m_bNoVideo ? 1 : 0);
 
-  char *opts = NULL;
-  if(xc.audio_vis_goom_opts[0] && !strcmp(xc.audio_visualization, "goom"))
-    opts = xc.audio_vis_goom_opts;
+  if(m_bNoVideo && strcmp(xc.audio_visualization, "none") && strcmp(xc.audio_visualization, "image")) {
+
+    char *opts = NULL;
+    if(xc.audio_vis_goom_opts[0] && !strcmp(xc.audio_visualization, "goom"))
+      opts = xc.audio_vis_goom_opts;
 
-  if(m_bNoVideo && strcmp(xc.audio_visualization, "none")) {
     ConfigurePostprocessing(xc.audio_visualization, true, opts);
+
   } else {
     ConfigurePostprocessing("AudioVisualization", false, NULL);
   }
@@ -348,13 +365,6 @@ void cXinelibThread::AudioStreamChanged(bool ac3, int StreamId)
     Xine_Control("AUDIOSTREAM", StreamId);
 }
 
-#if VDRVERSNUM < 10515
-void cXinelibThread::SpuStreamChanged(int StreamId)
-{
-  TRACEF("cXinelibThread::SpuStreamChanged");
-  Xine_Control("SPUSTREAM", StreamId);
-}
-#else
 void cXinelibThread::SetSubtitleTrack(eTrackType Track)
 {
   TRACEF("cXinelibThread::SetSubtitleTrack");
@@ -363,7 +373,16 @@ void cXinelibThread::SetSubtitleTrack(eTrackType Track)
 				 m_SpuLangAuto ? " auto" : "");
   Xine_Control(buf);
 }
-#endif
+
+void cXinelibThread::Pip_Config(int Index, int X, int Y, int W, int H)
+{
+  Xine_Control(cString::sprintf("PIP %d %d %d %d %d", Index, X, Y, W, H));
+}
+
+void cXinelibThread::Pip_Close(int Index)
+{
+  Xine_Control(cString::sprintf("PIP %d Close", Index));
+}
 
 void cXinelibThread::Clear(void)
 {
@@ -417,13 +436,14 @@ int cXinelibThread::Poll(cPoller& Poller, int TimeoutMs)
 // Data transfer
 //
 
-int cXinelibThread::Play_PES(const uchar *data, int len)
+int cXinelibThread::Play(const uchar *data, int len, eStreamId StreamId)
 {
-  Lock();
-  m_StreamPos += len;
-  m_Frames++;
-  /*m_bEndOfStreamReached = false;*/
-  Unlock();
+  if (StreamId == sidVdr && len >= 0) {
+    Lock();
+    m_StreamPos += len;
+    m_Frames++;
+    Unlock();
+  }
   return len;
 }
 
@@ -502,7 +522,7 @@ int cXinelibThread::Play_Mpeg1_PES(const uchar *data1, int len)
       while(!Poll(p,100) && loops++ < 10) {
 	LOGDBG("Play_Mpeg1_PES: poll failed");
       }
-      r = Play_PES(data2,newlen+6);
+      r = Play(data2, newlen + 6);
     } 
   
     delete data2;
@@ -513,22 +533,21 @@ int cXinelibThread::Play_Mpeg1_PES(const uchar *data1, int len)
 
 // Pack elementary MPEG stream to PES
 
-bool cXinelibThread::Play_Mpeg2_ES(const uchar *data, int len, int streamID)
+bool cXinelibThread::Play_Mpeg2_ES(const uchar *data, int len, int streamID, bool h264)
 {
   static uchar hdr_vid[] = {0x00,0x00,0x01,0xe0, 0x00,0x00,0x80,0x00,0x00}; /* mpeg2 */
   static uchar hdr_pts[] = {0x00,0x00,0x01,0xe0, 0x00,0x08,0x80,0x80,
                             0x05,0x00,0x00,0x00, 0x00,0x00}; /* mpeg2 */
   static uchar seq_end[] = {0x00,0x00,0x01,0xe0, 0x00,0x07,0x80,0x00,
 			    0x00,
-			    0x00,0x00,0x01,0xB7}; /* mpeg2 */
+			    0x00,0x00,0x01,SC_SEQUENCE_END}; /* mpeg2 */
   int todo = len, done = 0, hdrlen = 9/*sizeof(hdr)*/;
   uchar *frame = new uchar[PES_CHUNK_SIZE+32];
   cPoller p;
-  bool h264 = IS_NAL_AUD(data);
 
   hdr_pts[3] = (uchar)streamID;
   Poll(p, 100);
-  Play_PES(hdr_pts, sizeof(hdr_pts));
+  Play(hdr_pts, sizeof(hdr_pts));
 
   hdr_vid[3] = (uchar)streamID;
   while(todo) {
@@ -546,7 +565,7 @@ bool cXinelibThread::Play_Mpeg2_ES(const uchar *data, int len, int streamID)
 
     Poll(p, 100);
 
-    if(blocklen+hdrlen != Play_PES(frame,blocklen+hdrlen)) {      
+    if (blocklen + hdrlen != Play(frame, blocklen + hdrlen)) {
       delete frame;
       return false;
     }
@@ -555,9 +574,9 @@ bool cXinelibThread::Play_Mpeg2_ES(const uchar *data, int len, int streamID)
   // append sequence end code to video 
   if((streamID & 0xF0) == 0xE0) { 
     seq_end[3] = (uchar)streamID;
-    seq_end[12] = h264 ? NAL_END_SEQ : 0xB7;
+    seq_end[12] = h264 ? NAL_END_SEQ : SC_SEQUENCE_END;
     Poll(p, 100);
-    Play_PES(seq_end, sizeof(seq_end));
+    Play(seq_end, sizeof(seq_end));
   }
 
   delete[] frame;
@@ -689,7 +708,7 @@ bool cXinelibThread::PlayFile(const char *FileName, int Position,
 {
   TRACEF("cXinelibThread::PlayFile");
 
-  char vis[256];
+  char vis[256+4096];
 
   switch(PlayMode) {
     case pmVideoOnly:
@@ -706,19 +725,22 @@ bool cXinelibThread::PlayFile(const char *FileName, int Position,
       break;
     case pmAudioVideo:
     default:
-      if(xc.audio_vis_goom_opts[0] && !strcmp(xc.audio_visualization, "goom"))
+      if (xc.audio_vis_goom_opts[0] && !strcmp(xc.audio_visualization, "goom")) {
 	snprintf(vis, sizeof(vis), "%s:%s", xc.audio_visualization, xc.audio_vis_goom_opts);
-      else
+      } else if (xc.audio_vis_image_mrl[0] && !strcmp(xc.audio_visualization, "image")) {
+	snprintf(vis, sizeof(vis), "%s:%s", xc.audio_visualization, xc.audio_vis_image_mrl);
+      } else {
 	strn0cpy(vis, xc.audio_visualization, sizeof(vis));
+      }
       vis[sizeof(vis)-1] = 0;
       break;
   }
 
-  char buf[4096];
+  char buf[4096+4096+256];
   m_bEndOfStreamReached = false;
   if(snprintf(buf, sizeof(buf), "PLAYFILE %s %d %s %s",
 	      LoopPlay ? "Loop" : "", Position, vis, FileName ? FileName : "")
-     >= 4096) {
+     >= 4096+4096+256) {
     LOGMSG("PlayFile: message too long !");
     return 0;
   }
@@ -728,11 +750,9 @@ bool cXinelibThread::PlayFile(const char *FileName, int Position,
     m_FileName = FileName;
     m_bPlayingFile = true;
     m_SpuLangAuto = true;
-#if VDRVERSNUM >= 10515
     if (m_StatusMonitor)
       DELETENULL(m_StatusMonitor);
     m_StatusMonitor = new cFrontendStatusMonitor(m_SpuLangAuto);
-#endif
     Unlock();
   }
 
@@ -749,15 +769,15 @@ bool cXinelibThread::PlayFile(const char *FileName, int Position,
     if(xc.extsub_size >= 0)
       Xine_Control("EXTSUBSIZE", xc.extsub_size);
 
-#if VDRVERSNUM >= 10515
     // set preferred subtitle language 
     if (Setup.DisplaySubtitles) {
       const char *langs = I18nLanguageCode(Setup.SubtitleLanguages[0]);
       if (langs) {
 	char lang1[5];
 	strn0cpy(lang1, langs, 4); /* truncate */
-	const char *spu_lang = iso639_1_to_iso639_2(lang1);
+	const char *spu_lang = iso639_2_to_iso639_1(lang1);
 	LOGMSG("Preferred SPU language: %s (%s)", lang1, spu_lang);
+        Xine_Control(cString::sprintf("SPUSTREAM %s", lang1));
 	if (spu_lang && spu_lang[0] && spu_lang[1] && !spu_lang[2])
 	  Xine_Control(cString::sprintf("SPUSTREAM %s", spu_lang));
       }
@@ -765,7 +785,6 @@ bool cXinelibThread::PlayFile(const char *FileName, int Position,
       LOGMSG("Preferred SPU language: (none)");
       Xine_Control(cString::sprintf("SPUSTREAM %d", ttXSubtitleNone));
     }
-#endif
   }
 
   return Running() && !result;
@@ -801,6 +820,9 @@ void cXinelibThread::Configure(void)
     Xine_Control(cString::sprintf("SCR %s %d", 
 				  xc.live_mode_sync ? "Sync"    : "NoSync",
 				  xc.scr_tuning     ? xc.scr_hz : 90000));
+    Xine_Control("HDMODE", m_bHDMode);
+
+    Xine_Control("CONFIG END");
 }
 
 int cXinelibThread::ConfigurePostprocessing(const char *deinterlace_method, 
@@ -829,7 +851,7 @@ int cXinelibThread::ConfigurePostprocessing(const char *deinterlace_method,
 	  audio_equalizer[8],audio_equalizer[9]);
   r = Xine_Control(buf) && r;
 
-  if(m_bNoVideo && strcmp(xc.audio_visualization, "none")) {
+  if (m_bNoVideo && strcmp(xc.audio_visualization, "none") && strcmp(xc.audio_visualization, "image")) {
     char *opts = NULL;
     if(xc.audio_vis_goom_opts[0] && !strcmp(xc.audio_visualization, "goom"))
       opts = xc.audio_vis_goom_opts;
