@@ -4,7 +4,7 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: frontend_local.c,v 1.31.2.5 2009/09/13 18:48:11 phintuka Exp $
+ * $Id: frontend_local.c,v 1.60 2012/03/19 11:57:18 phintuka Exp $
  *
  */
 
@@ -21,6 +21,7 @@
 #include <vdr/tools.h>
 #include <vdr/shutdown.h>
 #include <vdr/plugin.h>
+#include <vdr/device.h>
 
 #include "logdefs.h"
 #include "config.h"
@@ -50,21 +51,26 @@ class cRwLockBlock
 //----------------- keyboard control handler (C callback) --------------------
 
 extern "C" {
-  static void keypress_handler(const char *keymap, const char *key)
+  static void keypress_handler(void *handle, const char *keymap, const char *key)
   {
-    if(!strncmp("INFO ", keymap, 5)) {
-      
-      cXinelibThread::InfoHandler(keymap+5);
+    if (!handle) {
+      LOGMSG("keypress_handler(): missing handle");
+      return;
+    }
 
-    } else if(!xc.use_x_keyboard || !key) {
+    cXinelibThread *t = (cXinelibThread *)handle;
+
+    if (!strncmp("INFO ", keymap, 5)) {
+      t->InfoHandler(keymap+5);
+
+    } else if (!xc.use_x_keyboard || !key) {
 
       /* Only X11 key events came this way in local mode.
 	 Keyboard is handled by vdr. */
       LOGMSG("keypress_handler(%s): X11 Keyboard disabled in config", key);
 
     } else {
-
-      cXinelibThread::KeypressHandler(keymap, key, false, false);
+      t->KeypressHandler(keymap, key, false, false);
 
     }
   }
@@ -72,8 +78,8 @@ extern "C" {
 
 //----------------------------- cXinelibLocal --------------------------------
 
-cXinelibLocal::cXinelibLocal(const char *frontend_name) :
-  cXinelibThread("Local decoder/display (cXinelibThread)"), m_feLock(true)
+cXinelibLocal::cXinelibLocal(cXinelibDevice *Dev, const char *frontend_name) :
+  cXinelibThread(Dev, "Local decoder/display (cXinelibThread)"), m_feLock(true)
 {
   fe = NULL;
   h_fe_lib = NULL;
@@ -116,20 +122,16 @@ cXinelibLocal::~cXinelibLocal()
 // Data transfer
 //
 
-int cXinelibLocal::Play_PES(const uchar *data, int len)
+int cXinelibLocal::Play(const uchar *data, int len, eStreamId StreamId)
 {
-  TRACEF("cXinelibLocal::Play_PES");
+  TRACEF("cXinelibLocal::Play");
 
   {
     LOCK_FE;
     if (fe && Running()) {
-      int done = fe->xine_queue_pes_packet(fe, (char*)data, len);
-      if (done >= 0) {
-	Lock();
-	m_StreamPos += done;
-	Unlock();
-        return done;
-      }
+      int done = fe->xine_queue_pes_packet(fe, StreamId, m_StreamPos, (const char *)data, len);
+      if (done >= 0)
+        return cXinelibThread::Play(data, done, StreamId);
     }
   }
 
@@ -193,14 +195,13 @@ bool cXinelibLocal::EndOfStreamReached(void)
 
 void cXinelibLocal::ConfigureWindow(int fullscreen, int width, int height, 
 				    int modeswitch, const char *modeline, 
-				    int aspect, int scale_video, 
-				    int field_order) 
+				    int aspect, int scale_video)
 {
   LOCK_FE;
   if(fe)
     fe->fe_display_config(fe, -1, -1, width, height,
                           fullscreen, modeswitch, modeline,
-                          aspect, scale_video, field_order);
+                          aspect, scale_video);
 }
 
 void cXinelibLocal::ConfigureDecoder(int pes_buffers)
@@ -222,6 +223,11 @@ void cXinelibLocal::ConfigureDecoder(int pes_buffers)
   cCondWait::SleepMs(100);
 }
 
+int cXinelibLocal::SupportsTrueColorOSD(void)
+{
+  return !!xc.hud_osd;
+}
+
 //
 // Xine control
 //
@@ -256,24 +262,24 @@ frontend_t *cXinelibLocal::load_frontend(const char *fe_name)
   fe_creator_f *fe_creator = NULL;
   static int my_marker = 0;
 
-  if(!dladdr((void *)&my_marker, &info)) {
-    LOGERR("Error searching plugin: dladdr() returned false (%s)",dlerror());
+  if (!dladdr((void *)&my_marker, &info)) {
+    LOGERR("Error searching plugin: dladdr() returned false (%s)", dlerror());
     return NULL;
   }
   LOGDBG("xineliboutput: plugin file is %s", info.dli_fname);
 
   int  fe_ind = strstra(fe_name, xc.s_frontends, FRONTEND_NONE);
   bool fe_try = false;
-  if(fe_ind == FRONTEND_NONE) {
+  if (fe_ind == FRONTEND_NONE) {
     LOGMSG("Front-end %s unknown!", fe_name);
     fe_ind = 0;
     fe_try = true;
   }
 
   strn0cpy(libname, info.dli_fname, sizeof(libname) - 128);
-  if(strrchr(libname, '/')) 
+  if (strrchr(libname, '/'))
     *(strrchr(libname, '/')+1) = 0;
-  
+
   LOGDBG("Searching frontend %s from %s", xc.s_frontends[fe_ind], libname);
 
   do {
@@ -281,29 +287,28 @@ frontend_t *cXinelibLocal::load_frontend(const char *fe_name)
     LOGDBG("Probing %s", libname);
 
     if (stat(libname, &statbuffer)) {
-      LOGERR("load_frontend: can't stat %s",libname);
+      LOGERR("load_frontend: can't stat %s", libname);
     } else if((statbuffer.st_mode & S_IFMT) != S_IFREG) {
       LOGMSG("load_frontend: %s not regular file ! trying to load anyway ...",
 	     libname);
     }
 
-    if( !(lib = dlopen (libname, RTLD_LAZY | RTLD_GLOBAL))) {
-      char *dl_error_msg = dlerror();
-      LOGERR("load_frontend: cannot dlopen file %s: %s", 
-	     libname, dl_error_msg);
+    if ( !(lib = dlopen (libname, RTLD_LAZY | RTLD_GLOBAL))) {
+      LOGERR("load_frontend: cannot dlopen file %s: %s", libname, dlerror());
+
     } else if ( (fe_creator = (fe_creator_f*)dlsym(lib, "fe_creator"))) {
       LOGDBG("load_frontend: entry at %p", fe_creator);
       frontend_t *fe = (**fe_creator)();
 
-      if(fe) {
-	if(h_fe_lib)
+      if (fe) {
+	if (h_fe_lib)
 	  dlclose(h_fe_lib);
 	h_fe_lib = lib;
 
-	LOGDBG("Using frontend %s (%s) from %s", 
-	       xc.s_frontends[fe_ind], xc.s_frontendNames[fe_ind], 
+	LOGDBG("Using frontend %s (%s) from %s",
+	       xc.s_frontends[fe_ind], xc.s_frontendNames[fe_ind],
 	       xc.s_frontend_files[fe_ind]);
-	
+
 	return fe;
       } else {
 	LOGMSG("Frontend %s (%s) creation failed",
@@ -313,12 +318,12 @@ frontend_t *cXinelibLocal::load_frontend(const char *fe_name)
       LOGERR("Frontend entry point not found");
       dlclose(lib);
     }
-    
+
     fe_ind++;  // try next frontend ...
 
-  } while(fe_try && fe_ind < FRONTEND_count);
+  } while (fe_try && fe_ind < FRONTEND_count);
 
-  LOGMSG("No usable frontends found, giving up !");  
+  LOGMSG("No usable frontends found, giving up !");
   return NULL;
 }
 
@@ -342,13 +347,18 @@ void cXinelibLocal::Action(void)
       Cancel(-1);
     } else {
       LOGDBG("cXinelibLocal::Action - fe created");
-      if(!curr_fe->fe_display_open(curr_fe, 0, 0, xc.width, xc.height, xc.fullscreen, xc.hud_osd,
+      curr_fe->fe_message_cb = keypress_handler;
+      curr_fe->fe_message_h  = this;
+      if(!curr_fe->fe_display_open(curr_fe,
+                                   xc.xpos, xc.ypos, xc.width, xc.height, xc.fullscreen,
+                                   xc.hud_osd,
+                                   xc.opengl,
                                    xc.modeswitch, xc.modeline, xc.display_aspect,
-                                   keypress_handler, 0/*no_x_kbd*/, 0/*gui_hotkeys*/,
+                                   0/*no_x_kbd*/, 0/*gui_hotkeys*/,
                                    xc.video_port,
                                    xc.scale_video,
-                                   xc.field_order,
-                                   NULL, -1)) {
+                                   NULL,
+                                   xc.window_id)) {
 	LOGMSG("cXinelibLocal: Error initializing display");
         Cancel(-1);
       } else {
@@ -446,17 +456,15 @@ void cXinelibLocal::Action(void)
 
     if(!m_bReconfigRequest && xc.exit_on_close) {
       LOGMSG("Shutting down VDR");
-#if VDRVERSNUM >= 10501 || (defined(PATCH_SHUTDOWN_REWRITE) && PATCH_SHUTDOWN_REWRITE >= 100)
       ShutdownHandler.RequestEmergencyExit();
-#else
-      cThread::EmergencyExit(true);
-#endif
       break;
     }
   }
 
   if(curr_fe) {
-    curr_fe->xine_exit(fe);
+    LOCK_FE_WR;
+    fe = NULL;
+    curr_fe->xine_exit(curr_fe);
     curr_fe->fe_display_close(curr_fe);
     curr_fe->fe_free(curr_fe);
   }
