@@ -4,20 +4,26 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: setup_menu.c,v 1.62.2.3 2009/11/16 11:34:49 phintuka Exp $
+ * $Id: setup_menu.c,v 1.84 2012/01/16 11:57:53 phintuka Exp $
  *
  */
 
+#include "features.h"
+
 #include <vdr/config.h>
 #include <vdr/plugin.h>
 #include <vdr/remote.h>
+#include <vdr/i18n.h>
 
 #include "config.h"
 #include "device.h"
 #include "menuitems.h"
-#include "i18n.h"      // trVDR for vdr-1.4.x
 #include "osd.h"       // cXinelibOsdProvider::RefreshOsd()
 #include "setup_menu.h"
+#include "tools/playlist.h"
+
+#define indent(x) Label_Ident(x)
+#define inden2(x) Label_Ident(Label_Ident(x))
 
 
 namespace XinelibOutputSetupMenu {
@@ -95,17 +101,6 @@ static int INDEX_TO_CONTROL(int ind)
 }
 #endif
 
-static cOsdItem *NewTitle(const char *s)
-{
-  char str[128];
-  cOsdItem *tmp;
-  snprintf(str, sizeof(str), "----- %s -----", s);
-  str[sizeof(str)-1] = 0;
-  tmp = new cOsdItem(str);
-  tmp->SetSelectable(false);
-  return tmp;
-}
-
 //--- cMenuSetupAudio --------------------------------------------------------
 
 class cMenuSetupAudio : public cMenuSetupPage 
@@ -115,7 +110,6 @@ class cMenuSetupAudio : public cMenuSetupPage
     int visualization;
     int goom_width, goom_height, goom_fps;
 
-    cOsdItem *audio_ctrl_speakers;
     cOsdItem *audio_ctrl_volume;
     cOsdItem *audio_ctrl_delay;
     cOsdItem *audio_ctrl_compression;
@@ -176,11 +170,7 @@ void cMenuSetupAudio::Set(void)
   int current = Current();
   Clear();
 
-  Add(NewTitle(tr("Audio")));
-
-  Add(audio_ctrl_speakers =
-      new cMenuEditStraI18nItem(tr("Speakers"), &newconfig.speaker_type, 
-				SPEAKERS_count, xc.s_speakerArrangements));
+  Add(SeparatorItem(tr("Audio")));
 
   Add(audio_ctrl_volume =
       new cMenuEditBoolItem(tr("Volume control"), 
@@ -212,12 +202,16 @@ void cMenuSetupAudio::Set(void)
 				AUDIO_VIS_count, 
 				xc.s_audioVisualizationNames));
   if(visualization == AUDIO_VIS_GOOM) {
-    Add(new cMenuEditTypedIntItem(tr("  Width"), tr("px"), &goom_width,
-				  320, 1920));
-    Add(new cMenuEditTypedIntItem(tr("  Height"),tr("px"), &goom_height,
-				  240, 1280));
-    Add(new cMenuEditTypedIntItem(tr("  Speed"), tr("fps"), &goom_fps,
-				  1, 100));
+    Add(new cMenuEditTypedIntItem(indent(tr("Width")), tr("px"),
+                                  &goom_width, 320, 1920));
+    Add(new cMenuEditTypedIntItem(indent(tr("Height")),tr("px"),
+                                  &goom_height, 240, 1280));
+    Add(new cMenuEditTypedIntItem(indent(tr("Speed")), tr("fps"),
+                                  &goom_fps, 1, 100));
+  } else if(visualization == AUDIO_VIS_IMAGE) {
+    Add(new cMenuEditStrItem(indent(tr("Background image MRL")),
+			     newconfig.audio_vis_image_mrl,
+			     sizeof(newconfig.audio_vis_image_mrl)));
   }
 
   if(current<1) current=1; /* first item is not selectable */
@@ -245,17 +239,6 @@ eOSState cMenuSetupAudio::ProcessKey(eKeys Key)
   else if(item == audio_ctrl_vis) {
     Set();
   }
-  else if(item == audio_ctrl_speakers) {
-    cXinelibDevice::Instance().ConfigurePostprocessing(
-          xc.deinterlace_method, newconfig.audio_delay, 
-	  newconfig.audio_compression, newconfig.audio_equalizer,
-	  newconfig.audio_surround, newconfig.speaker_type);
-    if(newconfig.speaker_type <= SPEAKERS_STEREO &&
-       newconfig.audio_upmix) {
-      newconfig.audio_upmix = false;
-      Set();
-    }
-  }
   else if(item == audio_ctrl_surround) {
     cXinelibDevice::Instance().ConfigurePostprocessing(
           xc.deinterlace_method, newconfig.audio_delay, 
@@ -301,7 +284,9 @@ void cMenuSetupAudio::Store(void)
 	   goom_width, goom_height, goom_fps);
   xc.audio_vis_goom_opts[sizeof(xc.audio_vis_goom_opts)-1] = 0;
 
-  SetupStore("Audio.Speakers", xc.s_speakerArrangements[xc.speaker_type]);
+  if(xc.audio_vis_image_mrl[0] == '/')
+    snprintf(xc.audio_vis_image_mrl, sizeof(xc.audio_vis_image_mrl), "%s", *cPlaylist::BuildMrl("file", xc.audio_vis_image_mrl));
+
   SetupStore("Audio.Delay",    xc.audio_delay);
   SetupStore("Audio.Compression",  xc.audio_compression);
   SetupStore("Audio.Surround",     xc.audio_surround);
@@ -309,6 +294,7 @@ void cMenuSetupAudio::Store(void)
   SetupStore("Audio.Headphone",    xc.headphone);
   SetupStore("Audio.Visualization",xc.audio_visualization);
   SetupStore("Audio.Visualization.GoomOpts",xc.audio_vis_goom_opts);
+  SetupStore("Audio.Visualization.ImageMRL",xc.audio_vis_image_mrl);
   SetupStore("Audio.SoftwareVolumeControl", xc.sw_volume_control);
   Setup.Save();
 }
@@ -350,7 +336,7 @@ void cMenuSetupAudioEq::Set(void)
   int current = Current();
   Clear();
 
-  Add(NewTitle(tr("Audio Equalizer")));
+  Add(SeparatorItem(tr("Audio Equalizer")));
   for(int i=0; i<AUDIO_EQ_count; i++)
     Add(new cMenuEditTypedIntItem(config_t::s_audioEqNames[i], "%", 
 				  &newconfig.audio_equalizer[i],
@@ -498,6 +484,20 @@ class cMenuSetupVideo : public cMenuSetupPage
     config_t newconfig;
 
     cOsdItem *ctrl_autocrop;
+    cOsdItem *ctrl_autocrop_autodetect;
+    cOsdItem *ctrl_autocrop_autodetect_rate;
+    cOsdItem *ctrl_autocrop_soft;
+    cOsdItem *ctrl_autocrop_soft_start_step;
+    cOsdItem *ctrl_autocrop_fixedsize;
+    cOsdItem *ctrl_autocrop_stabilize_time;
+    cOsdItem *ctrl_autocrop_subs;
+    cOsdItem *ctrl_autocrop_subs_detect_lifetime;
+    cOsdItem *ctrl_autocrop_subs_detect_stabilize_time;
+    cOsdItem *ctrl_autocrop_logo_width;
+    cOsdItem *ctrl_autocrop_use_driver_crop;
+    cOsdItem *ctrl_autocrop_use_avards_analysis;
+    cOsdItem *ctrl_autocrop_overscan_compensate;
+    cOsdItem *ctrl_autocrop_bar_tone_tolerance;
     cOsdItem *ctrl_swscale;
     cOsdItem *ctrl_swscale_resize;
     cOsdItem *ctrl_swscale_aspect;
@@ -576,7 +576,7 @@ void cMenuSetupVideo::Set(void)
   int current = Current();
   Clear();
 
-  Add(NewTitle(tr("Video")));
+  Add(SeparatorItem(tr("Video")));
 
   Add(ctrl_vo_aspect_ratio =
       new cMenuEditStraI18nItem(tr("Aspect ratio"), &newconfig.vo_aspect_ratio,
@@ -586,15 +586,61 @@ void cMenuSetupVideo::Set(void)
       new cMenuEditBoolItem(tr("Crop letterbox 4:3 to 16:9"), 
 			    &newconfig.autocrop));
   if(newconfig.autocrop) {
-    Add(new cMenuEditBoolItem(tr("  Autodetect letterbox"), 
+    Add(ctrl_autocrop_use_driver_crop =
+                    new cMenuEditBoolItem(indent(tr("Use driver crop")),
+                              &newconfig.autocrop_use_driver_crop));
+
+    Add(ctrl_autocrop_autodetect =
+                    new cMenuEditBoolItem(indent(tr("Autodetect letterbox")),
 			      &newconfig.autocrop_autodetect));
-    Add(new cMenuEditBoolItem(tr("  Soft start"), 
-			      &newconfig.autocrop_soft));
-    Add(new cMenuEditBoolItem(tr("  Crop to"),
-			      &newconfig.autocrop_fixedsize,
-			      "4:3...20:9", "14:9/16:9"));
-    Add(new cMenuEditBoolItem(tr("  Detect subtitles"),
-			      &newconfig.autocrop_subs));
+    if(newconfig.autocrop_autodetect) {
+      Add(ctrl_autocrop_fixedsize =
+                      new cMenuEditBoolItem(indent(tr("Crop to")),
+                                &newconfig.autocrop_fixedsize,
+                                "4:3...20:9", "14:9/16:9"));
+      Add(ctrl_autocrop_autodetect_rate =
+                      new cMenuEditIntItem(indent(tr("Autodetect rate")),
+                                &newconfig.autocrop_autodetect_rate, 1, 30));
+      Add(ctrl_autocrop_stabilize_time =
+                      new cMenuEditIntItem(indent(tr("Stabilize time")),
+                                &newconfig.autocrop_stabilize_time, 1, 9999));
+      Add(ctrl_autocrop_logo_width =
+                      new cMenuEditIntItem(indent(tr("Maximum logo width [%]")),
+                                &newconfig.autocrop_logo_width, 0, 99));
+      Add(ctrl_autocrop_overscan_compensate =
+                      new cMenuEditIntItem(indent(tr("Overscan compensate [%1000]")),
+                                &newconfig.autocrop_overscan_compensate, 0, 9999));
+
+      Add(ctrl_autocrop_soft =
+                      new cMenuEditBoolItem(indent(tr("Soft start")),
+                                &newconfig.autocrop_soft));
+      if(newconfig.autocrop_soft) {
+        Add(ctrl_autocrop_soft_start_step =
+                        new cMenuEditIntItem(indent(tr("Soft start step")),
+                                  &newconfig.autocrop_soft_start_step, 1, 999));
+      }
+
+      Add(ctrl_autocrop_subs =
+                      new cMenuEditBoolItem(indent(tr("Detect subtitles")),
+                                &newconfig.autocrop_subs));
+      if(newconfig.autocrop_subs) {
+        Add(ctrl_autocrop_subs_detect_stabilize_time =
+                        new cMenuEditIntItem(indent(tr("Subs detect stabilize time")),
+                                  &newconfig.autocrop_subs_detect_stabilize_time, 0, 9999));
+        Add(ctrl_autocrop_subs_detect_lifetime =
+                        new cMenuEditIntItem(indent(tr("Subs detect lifetime")),
+                                  &newconfig.autocrop_subs_detect_lifetime, 0, 9999));
+      }
+
+      Add(ctrl_autocrop_use_avards_analysis =
+                      new cMenuEditBoolItem(indent(tr("Use avards analysis")),
+                                &newconfig.autocrop_use_avards_analysis));
+      if (newconfig.autocrop_use_avards_analysis) {
+        Add(ctrl_autocrop_bar_tone_tolerance =
+                        new cMenuEditIntItem(indent(tr("Bar tone tolerance")),
+                                  &newconfig.autocrop_bar_tone_tolerance, 0, 255));
+      }
+    }
   }
 
   ctrl_swscale_resize = ctrl_swscale_aspect = ctrl_swscale_width = ctrl_swscale_height = NULL;
@@ -603,19 +649,19 @@ void cMenuSetupVideo::Set(void)
 			    &newconfig.swscale));
   if(newconfig.swscale) {
     Add(ctrl_swscale_aspect =
-	new cMenuEditBoolItem(tr("  Change aspect ratio"), 
+	new cMenuEditBoolItem(indent(tr("Change aspect ratio")),
 			      &newconfig.swscale_change_aspect));
     Add(ctrl_swscale_resize = 
-	new cMenuEditBoolItem(tr("  Change video size"), 
+	new cMenuEditBoolItem(indent(tr("Change video size")),
 			      &newconfig.swscale_resize));
     if(newconfig.swscale_resize) {
       Add(ctrl_swscale_width =
-	  new cMenuEditIntItem( tr("  Width"), 
+	  new cMenuEditIntItem( indent(tr("Width")),
 				&newconfig.swscale_width, 360, 2000));
       Add(ctrl_swscale_height =
-	  new cMenuEditIntItem( tr("  Height"),
+	  new cMenuEditIntItem( indent(tr("Height")),
 				&newconfig.swscale_height, 288, 1200));
-      Add(new cMenuEditBoolItem(tr("  Allow downscaling"),
+      Add(new cMenuEditBoolItem(indent(tr("Allow downscaling")),
 				&newconfig.swscale_downscale));
     }
   }
@@ -628,9 +674,10 @@ void cMenuSetupVideo::Set(void)
   Add(ctrl_pp = new cMenuEditBoolItem(tr("Post processing (ffmpeg)"), 
 				      &newconfig.ffmpeg_pp));
   if(newconfig.ffmpeg_pp) {
-    Add(new cMenuEditIntItem( tr("  Quality"), 
+    Add(new cMenuEditIntItem( indent(tr("Quality")),
 			      &newconfig.ffmpeg_pp_quality, 0, 6));
-    Add(new cMenuEditStrItem( tr("  Mode"), newconfig.ffmpeg_pp_mode, 
+    Add(new cMenuEditStrItem( indent(tr("Mode")),
+                              newconfig.ffmpeg_pp_mode,
 			      255, OptionsChars));
   }
 
@@ -642,47 +689,49 @@ void cMenuSetupVideo::Set(void)
   ctrl_tvtime_method = NULL;
   if(deinterlace == DEINTERLACE_TVTIME) {
     Add(ctrl_tvtime_method =
-	new cMenuEditStraI18nItem(tr("  Method"), &tvtime.method,
-				  tvtime_methods_count, tvtime_method_name));
-    Add(new cMenuEditBoolItem(tr("  Cheap mode"), &tvtime.cheap_mode));
-    Add(new cMenuEditStraI18nItem(tr("  Pulldown"), &tvtime.pulldown,
-				  2, tvtime_pulldown_name));
-    Add(new cMenuEditStraI18nItem(tr("  Frame rate"), &tvtime.framerate,
-				  3, tvtime_framerate_name));
-    Add(new cMenuEditBoolItem(tr("  Judder Correction"), &tvtime.judder_correction));
-    Add(new cMenuEditBoolItem(tr("  Use progressive frame flag"), 
-			      &tvtime.use_progressive_frame_flag));
-    Add(new cMenuEditBoolItem(tr("  Chroma Filter"), 
-			      &tvtime.chroma_filter));
+	new cMenuEditStraI18nItem(indent(tr("Method")),
+                                  &tvtime.method, tvtime_methods_count, tvtime_method_name));
+    Add(new cMenuEditBoolItem(    indent(tr("Cheap mode")),
+                                  &tvtime.cheap_mode));
+    Add(new cMenuEditStraI18nItem(indent(tr("Pulldown")),
+                                  &tvtime.pulldown, 2, tvtime_pulldown_name));
+    Add(new cMenuEditStraI18nItem(indent(tr("Frame rate")),
+                                  &tvtime.framerate, 3, tvtime_framerate_name));
+    Add(new cMenuEditBoolItem(    indent(tr("Judder Correction")),
+                                  &tvtime.judder_correction));
+    Add(new cMenuEditBoolItem(    indent(tr("Use progressive frame flag")),
+                                  &tvtime.use_progressive_frame_flag));
+    Add(new cMenuEditBoolItem(    indent(tr("Chroma Filter")),
+                                  &tvtime.chroma_filter));
   }
 
   Add(ctrl_unsharp = new cMenuEditBoolItem(tr("Sharpen / Blur"),
                                       &newconfig.unsharp));
   if(newconfig.unsharp) {
-    Add(new cMenuEditOddIntItem( tr("  Width of the luma matrix"),
+    Add(new cMenuEditOddIntItem( indent(tr("Width of the luma matrix")),
                                  &newconfig.unsharp_luma_matrix_width, 3, 11));
-    Add(new cMenuEditOddIntItem( tr("  Height of the luma matrix"),
+    Add(new cMenuEditOddIntItem( indent(tr("Height of the luma matrix")),
                                  &newconfig.unsharp_luma_matrix_height, 3, 11));
-    Add(new cMenuEditFpIntItem( tr("  Amount of luma sharpness/blur"),
-                                &newconfig.unsharp_luma_amount, -20, 20, 1,
-                                tr("Off")));
-    Add(new cMenuEditOddIntItem( tr("  Width of the chroma matrix"),
-                              &newconfig.unsharp_chroma_matrix_width, 3, 11));
-    Add(new cMenuEditOddIntItem( tr("  Height of the chroma matrix"),
-                              &newconfig.unsharp_chroma_matrix_height, 3, 11));
-    Add(new cMenuEditFpIntItem( tr("  Amount of chroma sharpness/blur"),
-                                &newconfig.unsharp_chroma_amount, -20, 20, 1,
-                                tr("Off")));
+    Add(new cMenuEditFpIntItem(  indent(tr("Amount of luma sharpness/blur")),
+                                 &newconfig.unsharp_luma_amount, -20, 20, 1,
+                                 tr("Off")));
+    Add(new cMenuEditOddIntItem( indent(tr("Width of the chroma matrix")),
+                                 &newconfig.unsharp_chroma_matrix_width, 3, 11));
+    Add(new cMenuEditOddIntItem( indent(tr("Height of the chroma matrix")),
+                                 &newconfig.unsharp_chroma_matrix_height, 3, 11));
+    Add(new cMenuEditFpIntItem(  indent(tr("Amount of chroma sharpness/blur")),
+                                 &newconfig.unsharp_chroma_amount, -20, 20, 1,
+                                 tr("Off")));
   }
 
   Add(ctrl_denoise3d = new cMenuEditBoolItem(tr("3D Denoiser"),
                                       &newconfig.denoise3d));
   if(newconfig.denoise3d) {
-    Add(new cMenuEditFpIntItem( tr("  Spatial luma strength"),
+    Add(new cMenuEditFpIntItem( indent(tr("Spatial luma strength")),
                                 &newconfig.denoise3d_luma, 0, 100, 1));
-    Add(new cMenuEditFpIntItem( tr("  Spatial chroma strength"),
+    Add(new cMenuEditFpIntItem( indent(tr("Spatial chroma strength")),
                                 &newconfig.denoise3d_chroma, 0, 100, 1));
-    Add(new cMenuEditFpIntItem( tr("  Temporal strength"),
+    Add(new cMenuEditFpIntItem( indent(tr("Temporal strength")),
                                 &newconfig.denoise3d_time, 0, 100, 1));
   }
 
@@ -692,8 +741,10 @@ void cMenuSetupVideo::Set(void)
   Add(new cMenuEditIntItem(tr("Saturation"), &newconfig.saturation,-1,0xffff));
   Add(new cMenuEditIntItem(tr("Contrast"),   &newconfig.contrast, -1, 0xffff));
   Add(new cMenuEditIntItem(tr("Brightness"), &newconfig.brightness,-1,0xffff));
+#ifdef HAVE_VDPAU
   Add(new cMenuEditIntItem(tr("Sharpness"),  &newconfig.sharpness, -1,0xffff));
   Add(new cMenuEditIntItem(tr("Noise Reduction"), &newconfig.noise_reduction, -1,0xffff));
+#endif
 #else
   Add(ctrl_hue = new cMenuEditStraItem(tr("HUE"), &newconfig.hue, 33, 
 				       controls));
@@ -706,6 +757,7 @@ void cMenuSetupVideo::Set(void)
   Add(ctrl_brightness = 
       new cMenuEditStraItem(tr("Brightness"), &newconfig.brightness, 33, 
 			    controls));
+#ifdef HAVE_VDPAU
   Add(ctrl_sharpness = 
       new cMenuEditStraItem(tr("Sharpness"), &newconfig.sharpness, 33, 
 			    controls));
@@ -713,6 +765,7 @@ void cMenuSetupVideo::Set(void)
       new cMenuEditStraItem(tr("Noise Reduction"), &newconfig.noise_reduction, 33, 
 			    controls));
 #endif
+#endif
 
 #ifdef DEVICE_SUPPORTS_IBP_TRICKSPEED
   Add(new cMenuEditBoolItem(tr("Smooth fast forward"),
@@ -761,7 +814,21 @@ eOSState cMenuSetupVideo::ProcessKey(eKeys Key)
        INDEX_TO_CONTROL(newconfig.contrast),
        newconfig.overscan, newconfig.vo_aspect_ratio);
 #endif
-  else if(item == ctrl_autocrop) {
+  else if(item == ctrl_autocrop
+                  || item == ctrl_autocrop_autodetect
+                  || item == ctrl_autocrop_autodetect_rate
+                  || item == ctrl_autocrop_soft
+                  || item == ctrl_autocrop_soft_start_step
+                  || item == ctrl_autocrop_fixedsize
+                  || item == ctrl_autocrop_stabilize_time
+                  || item == ctrl_autocrop_subs
+                  || item == ctrl_autocrop_subs_detect_lifetime
+                  || item == ctrl_autocrop_subs_detect_stabilize_time
+                  || item == ctrl_autocrop_logo_width
+                  || item == ctrl_autocrop_use_driver_crop
+                  || item == ctrl_autocrop_use_avards_analysis
+                  || item == ctrl_autocrop_overscan_compensate
+                  || item == ctrl_autocrop_bar_tone_tolerance) {
     cXinelibDevice::Instance().ConfigurePostprocessing(
 	 "autocrop", newconfig.autocrop ? true : false, 
 	 newconfig.AutocropOptions());
@@ -825,11 +892,22 @@ void cMenuSetupVideo::Store(void)
   SetupStore("Video.Deinterlace", xc.deinterlace_method);
   SetupStore("Video.DeinterlaceOptions", xc.deinterlace_opts);
 
-  SetupStore("Video.AutoCrop",   xc.autocrop); 
+  SetupStore("Video.AutoCrop",   xc.autocrop);
   SetupStore("Video.AutoCrop.AutoDetect", xc.autocrop_autodetect);
+  SetupStore("Video.AutoCrop.AutoDetectRate", xc.autocrop_autodetect_rate);
   SetupStore("Video.AutoCrop.SoftStart",  xc.autocrop_soft);
+  SetupStore("Video.AutoCrop.SoftStartStep",  xc.autocrop_soft_start_step);
   SetupStore("Video.AutoCrop.FixedSize",  xc.autocrop_fixedsize);
+  SetupStore("Video.AutoCrop.StabilizeTime",  xc.autocrop_stabilize_time);
   SetupStore("Video.AutoCrop.DetectSubs", xc.autocrop_subs);
+  SetupStore("Video.AutoCrop.SubsDetectLifetime", xc.autocrop_subs_detect_lifetime);
+  SetupStore("Video.AutoCrop.SubsDetectStabilizeTime", xc.autocrop_subs_detect_stabilize_time);
+  SetupStore("Video.AutoCrop.LogoWidth", xc.autocrop_logo_width);
+  SetupStore("Video.AutoCrop.UseDriverCrop", xc.autocrop_use_driver_crop);
+  SetupStore("Video.AutoCrop.UseAvardsAnalysis", xc.autocrop_use_avards_analysis);
+  SetupStore("Video.AutoCrop.OverscanCompensate", xc.autocrop_overscan_compensate);
+  SetupStore("Video.AutoCrop.BarToneTolerance", xc.autocrop_bar_tone_tolerance);
+
   SetupStore("Video.SwScale",           xc.swscale);
   SetupStore("Video.SwScale.Aspect",    xc.swscale_change_aspect);
   SetupStore("Video.SwScale.Resize",    xc.swscale_resize);
@@ -880,19 +958,18 @@ class cMenuSetupOSD : public cMenuSetupPage
     int orig_alpha_correction;
     int orig_alpha_correction_abs;
 
+    cOsdItem *ctrl_size;
+    cOsdItem *ctrl_width;
     cOsdItem *ctrl_scaling;
     cOsdItem *ctrl_alpha;
     cOsdItem *ctrl_alpha_abs;
     cOsdItem *ctrl_blending;
     cOsdItem *ctrl_lowres;
-#if VDRVERSNUM < 10515
-    cOsdItem *ctrl_spulang0;
-#endif
 
   protected:
     virtual void Store(void);
     void Set(void);
-  
+
   public:
     cMenuSetupOSD(void);
     ~cMenuSetupOSD();
@@ -917,24 +994,39 @@ cMenuSetupOSD::~cMenuSetupOSD()
 }
 
 void cMenuSetupOSD::Set(void)
-{ 
+{
   SetPlugin(cPluginManager::GetPlugin(PLUGIN_NAME_I18N));
   int current = Current();
   Clear();
 
+  ctrl_size = NULL;
+  ctrl_width = NULL;
   ctrl_scaling = NULL;
   ctrl_blending = NULL;
   ctrl_lowres = NULL;
   ctrl_alpha = NULL;
   ctrl_alpha_abs = NULL;
-#if VDRVERSNUM < 10515
-  ctrl_spulang0 = NULL;
-#endif
 
-  Add(NewTitle(tr("On-Screen Display")));
-  Add(new cMenuEditBoolItem(tr("Hide main menu"), 
+  Add(SeparatorItem(tr("On-Screen Display")));
+  Add(new cMenuEditBoolItem(tr("Hide main menu"),
 			    &newconfig.hide_main_menu));
 
+  Add(ctrl_size =
+      new cMenuEditStraI18nItem(tr("Resolution"), &newconfig.osd_size,
+	                        OSD_SIZE_count, xc.s_osdSizes));
+  if (newconfig.osd_size == OSD_SIZE_custom) {
+    Add(ctrl_width =
+        new cMenuEditTypedIntItem(indent(tr("Width")), "px",
+                                  &newconfig.osd_width, 480, 2048));
+    Add(new cMenuEditTypedIntItem(indent(tr("Height")), "px",
+                                  &newconfig.osd_height, 576, 1200));
+  }
+
+#if VDRVERSNUM >= 10717
+  Add(new cMenuEditStraI18nItem(tr("Color depth"), &newconfig.osd_color_depth,
+	                        OSD_DEPTH_count, xc.s_osdColorDepths));
+#endif
+
   Add(ctrl_blending =
       new cMenuEditBoolItem(tr("Blending method"),
 			    &newconfig.osd_blending,
@@ -942,7 +1034,7 @@ void cMenuSetupOSD::Set(void)
 			    tr(xc.s_osdBlendingMethods[OSD_BLENDING_HARDWARE])));
   if(newconfig.osd_blending == OSD_BLENDING_SOFTWARE) {
     Add(ctrl_lowres =
-	new cMenuEditBoolItem(tr("  Use hardware for low-res video"),
+	new cMenuEditBoolItem(indent(tr("Use hardware for low-res video")),
 			      &newconfig.osd_blending_lowresvideo));
   }
 
@@ -950,12 +1042,10 @@ void cMenuSetupOSD::Set(void)
       new cMenuEditStraI18nItem(tr("Scaling method"), &newconfig.osd_scaling,
 	                        OSD_SCALING_count, xc.s_osdScalings));
 
-#if VDRVERSNUM >= 10509
-# if 0
-  Add(new cMenuEditStraI18nItem(tr("Show all layers"), &newconfig.osd_mixer, 
-			OSD_MIXER_count, xc.s_osdMixers));
-# endif
-#endif
+  Add(new cMenuEditBoolItem(tr("Scale subtitles"), &newconfig.osd_spu_scaling));
+
+  Add(new cMenuEditStraI18nItem(tr("Show all layers"), &newconfig.osd_mixer,
+                                OSD_MIXER_count, xc.s_osdMixers));
 
   Add(ctrl_alpha =
       new cMenuEditTypedIntItem(tr("Dynamic transparency correction"), "%", 
@@ -966,32 +1056,13 @@ void cMenuSetupOSD::Set(void)
 				&newconfig.alpha_correction_abs, -0xff, 0xff,
 				tr("Off")));
   
-#if VDRVERSNUM < 10515
-  Add(NewTitle(tr("Subtitles")));
-  Add(new cMenuEditBoolItem(trVDR("Setup.EPG$Preferred languages"), 
-			    &newconfig.spu_autoshow));
-  if(newconfig.spu_autoshow) {
-    Add(ctrl_spulang0 =
-	new cMenuEditStrItem(trVDR("Setup.EPG$Preferred language"),
-			     newconfig.spu_lang[0], 4, LangNameChars));
-    Add(new cMenuEditStrItem(trVDR("Setup.EPG$Preferred language"),
-			     newconfig.spu_lang[1], 4, LangNameChars));
-    Add(new cMenuEditStrItem(trVDR("Setup.EPG$Preferred language"),
-			     newconfig.spu_lang[2], 4, LangNameChars));
-    Add(new cMenuEditStrItem(trVDR("Setup.EPG$Preferred language"),
-			     newconfig.spu_lang[3], 4, LangNameChars));
-  }
-#endif
-
   Add(new cMenuEditStraI18nItem(tr("External subtitle size"),
 				&newconfig.extsub_size, SUBTITLESIZE_count, xc.s_subtitleSizes));
 
-#if VDRVERSNUM >= 10510
   Add(new cMenuEditBoolItem(tr("DVB subtitle decoder"),
 			    &newconfig.dvb_subtitles,
 			    "VDR",
 			    "frontend"));
-#endif
 
   if(current<1) current=1; /* first item is not selectable */
   SetCurrent(Get(current));
@@ -1017,16 +1088,15 @@ eOSState cMenuSetupOSD::ProcessKey(eKeys Key)
   else if(item == ctrl_alpha_abs)
     xc.alpha_correction_abs = newconfig.alpha_correction_abs;
 
+  if (newconfig.osd_size == OSD_SIZE_custom && !ctrl_width)
+    Set();
+  if (newconfig.osd_size != OSD_SIZE_custom && ctrl_width)
+    Set();
+
   if(newconfig.osd_blending==OSD_BLENDING_SOFTWARE && !ctrl_lowres)
     Set();
   if(newconfig.osd_blending!=OSD_BLENDING_SOFTWARE && ctrl_lowres)
     Set();
-#if VDRVERSNUM < 10515
-  if(newconfig.spu_autoshow && !ctrl_spulang0)
-    Set();
-  if(!newconfig.spu_autoshow && ctrl_spulang0)
-    Set();
-#endif
   return state;
 }
 
@@ -1042,7 +1112,12 @@ void cMenuSetupOSD::Store(void)
   orig_alpha_correction = xc.alpha_correction;
   orig_alpha_correction_abs = xc.alpha_correction_abs;
 
+  SetupStore("OSD.Size",            xc.s_osdSizes[xc.osd_size]);
+  SetupStore("OSD.Width",           xc.osd_width);
+  SetupStore("OSD.Height",          xc.osd_height);
+  SetupStore("OSD.ColorDepth",      xc.s_osdColorDepths[xc.osd_color_depth]);
   SetupStore("OSD.Scaling",         xc.osd_scaling);
+  SetupStore("OSD.ScalingSPU",      xc.osd_spu_scaling);
   SetupStore("OSD.HideMainMenu",    xc.hide_main_menu);
   SetupStore("OSD.LayersVisible",   xc.osd_mixer);
   SetupStore("OSD.Blending",        xc.osd_blending);
@@ -1059,13 +1134,6 @@ void cMenuSetupOSD::Store(void)
   SetupStore("OSD.AlphaCorrectionAbs", xc.alpha_correction_abs);
 
   SetupStore("OSD.ExtSubSize", xc.extsub_size);
-#if VDRVERSNUM < 10515
-  SetupStore("OSD.SpuAutoSelect", xc.spu_autoshow);
-  SetupStore("OSD.SpuLang0", xc.spu_lang[0]);
-  SetupStore("OSD.SpuLang1", xc.spu_lang[1]);
-  SetupStore("OSD.SpuLang2", xc.spu_lang[2]);
-  SetupStore("OSD.SpuLang3", xc.spu_lang[3]);
-#endif
   SetupStore("OSD.DvbSubtitles", xc.dvb_subtitles);
 
   Setup.Save();
@@ -1113,14 +1181,14 @@ void cMenuSetupDecoder::Set(void)
   int current = Current();
   Clear();
 
-  Add(NewTitle(tr("Decoder")));
+  Add(SeparatorItem(tr("Decoder")));
   Add(ctrl_pes_buffers_ind = 
       new cMenuEditStraI18nItem(tr("Buffer size"), &pes_buffers_ind, 
 				PES_BUFFERS_count, xc.s_bufferSize));
   if(pes_buffers_ind == PES_BUFFERS_CUSTOM)
     Add(ctrl_pes_buffers = 
-	new cMenuEditIntItem(tr("  Number of PES packets"), &newconfig.pes_buffers, 
-			     10, 10000));
+	new cMenuEditIntItem(indent(tr("Number of PES packets")),
+                             &newconfig.pes_buffers, 10, 10000));
   else
     ctrl_pes_buffers = NULL;
 
@@ -1254,7 +1322,7 @@ void cMenuSetupLocal::Set(void)
   ctrl_audio_driver = NULL;
   ctrl_audio_port = NULL;
 
-  Add(NewTitle(tr("Local Frontend")));
+  Add(SeparatorItem(tr("Local Frontend")));
 
   Add(ctrl_local_fe = 
       new cMenuEditStraI18nItem(tr("Local Display Frontend"), &local_frontend,
@@ -1266,9 +1334,8 @@ void cMenuSetupLocal::Set(void)
   }
 
   if(local_frontend != FRONTEND_NONE) {
-    cString tmp = cString::sprintf("%s >>", tr("Decoder"));
-    Add(new cOsdItem(tmp, osUser1));
-    Add(NewTitle(tr("Video")));
+    Add(SubMenuItem(tr("Decoder"), osUser1));
+    Add(SeparatorItem(tr("Video")));
   }
 
   if(local_frontend == FRONTEND_X11) {
@@ -1300,10 +1367,10 @@ void cMenuSetupLocal::Set(void)
 						&newconfig.fullscreen));
     if(!newconfig.fullscreen) {
       Add(ctrl_window_width = 
-	  new cMenuEditTypedIntItem( tr("  Window width"), tr("px"), 
+	  new cMenuEditTypedIntItem( indent(tr("Window width")), tr("px"), 
 				     &newconfig.width, 1, 2048));
       Add(ctrl_window_height = 
-	  new cMenuEditTypedIntItem( tr("  Window height"), tr("px"), 
+	  new cMenuEditTypedIntItem( indent(tr("Window height")), tr("px"), 
 				     &newconfig.height, 1, 2048));
     }
   }
@@ -1322,7 +1389,7 @@ void cMenuSetupLocal::Set(void)
 				  xc.s_fieldOrder));
 #endif
 
-    Add(NewTitle(tr("Audio")));
+    Add(SeparatorItem(tr("Audio")));
 
     Add(ctrl_audio_driver = 
 	new cMenuEditStraI18nItem(tr("Driver"), &audio_driver, 
@@ -1331,6 +1398,10 @@ void cMenuSetupLocal::Set(void)
       Add(ctrl_audio_port = 
 	  new cMenuEditStrItem(tr("Port"), newconfig.audio_port, 31, 
 			       DriverNameChars));
+
+    if(audio_driver != AUDIO_DRIVER_NONE)
+      Add(new cMenuEditStraI18nItem(tr("Speakers"), &newconfig.speaker_type,
+                                    SPEAKERS_count, xc.s_speakerArrangements));
   }
   
   if(current<1) current=1; /* first item is not selectable */
@@ -1424,6 +1495,7 @@ void cMenuSetupLocal::Store(void)
   SetupStore("Frontend",      xc.local_frontend);
   SetupStore("Audio.Driver",  xc.audio_driver);
   SetupStore("Audio.Port",    xc.audio_port);
+  SetupStore("Audio.Speakers", xc.s_speakerArrangements[xc.speaker_type]);
   SetupStore("Video.Driver",  xc.video_driver);
   SetupStore("Video.Port",    xc.video_port);
 #if 0
@@ -1480,7 +1552,7 @@ void cMenuSetupRemote::Set(void)
   SetPlugin(cPluginManager::GetPlugin(PLUGIN_NAME_I18N));
   Clear();
 
-  Add(NewTitle(tr("Remote Clients")));
+  Add(SeparatorItem(tr("Remote Clients")));
   Add(ctrl_remote_mode = new cMenuEditBoolItem(tr("Allow remote clients"), 
 					       &newconfig.remote_mode));
   ctrl_usertp = NULL;
@@ -1490,46 +1562,46 @@ void cMenuSetupRemote::Set(void)
   ctrl_http_ctrl = NULL;
   ctrl_rtsp_ctrl = NULL;
   if(newconfig.remote_mode) {
-    Add(new cMenuEditIntItem( tr("  Listen port (TCP and broadcast)"), 
+    Add(new cMenuEditIntItem( indent(tr("Listen port (TCP and broadcast)")),
 			      &newconfig.listen_port,
 			      0, 0xffff));
-    Add(new cMenuEditStrItem( tr("  Listen address"), 
+    Add(new cMenuEditStrItem( indent(tr("Listen address")),
 			      &newconfig.remote_local_ip[0], 16, "0123456789."));
-    Add(new cMenuEditBoolItem(tr("  Remote keyboard"), 
+    Add(new cMenuEditBoolItem(indent(tr("Remote keyboard")),
 			      &newconfig.remote_keyboard));
-    Add(new cMenuEditIntItem( tr("  Max number of clients"), 
+    Add(new cMenuEditIntItem( indent(tr("Max number of clients")),
 			      &newconfig.remote_max_clients,
 			      1, MAXCLIENTS));
 
-    Add(new cMenuEditBoolItem(tr("  PIPE transport"), 
+    Add(new cMenuEditBoolItem(indent(tr("PIPE transport")),
 			      &newconfig.remote_usepipe));
-    Add(new cMenuEditBoolItem(tr("  TCP transport"), 
+    Add(new cMenuEditBoolItem(indent(tr("TCP transport")),
 			      &newconfig.remote_usetcp));
-    Add(new cMenuEditBoolItem(tr("  UDP transport"), 
+    Add(new cMenuEditBoolItem(indent(tr("UDP transport")),
 			      &newconfig.remote_useudp));
-    Add(ctrl_usertp = 
-	new cMenuEditBoolItem(tr("  RTP (multicast) transport"), 
+    Add(ctrl_usertp =
+	new cMenuEditBoolItem(indent(tr("RTP (multicast) transport")),
 			      &newconfig.remote_usertp));
     if(newconfig.remote_usertp) {
       Add(ctrl_rtp_addr =
-	  new cMenuEditStrItem( tr("    Address"), 
-			        &newconfig.remote_rtp_addr[0], 16, "0123456789."));
-      Add(new cMenuEditOddIntItem( tr("    Port"), 
-				&newconfig.remote_rtp_port, 1000, 0xfffe));
-      Add(new cMenuEditIntItem( tr("    TTL"), 
-				&newconfig.remote_rtp_ttl, 1, 10));
-      Add(new cMenuEditBoolItem(tr("    Transmit always on"), 
-				&newconfig.remote_rtp_always_on));
-      Add(new cMenuEditBoolItem(tr("    SAP announcements"), 
-				&newconfig.remote_rtp_sap));
+	  new cMenuEditStrItem(    inden2(tr("Address")),
+                                   &newconfig.remote_rtp_addr[0], 16, "0123456789."));
+      Add(new cMenuEditOddIntItem( inden2(tr("Port")),
+                                   &newconfig.remote_rtp_port, 1000, 0xfffe));
+      Add(new cMenuEditIntItem(    inden2(tr("TTL")),
+                                   &newconfig.remote_rtp_ttl, 1, 10));
+      Add(new cMenuEditBoolItem(   inden2(tr("Transmit always on")),
+                                   &newconfig.remote_rtp_always_on));
+      Add(new cMenuEditBoolItem(   inden2(tr("SAP announcements")),
+                                   &newconfig.remote_rtp_sap));
     }
-    Add(new cMenuEditBoolItem(tr("  Server announce broadcasts"), 
+    Add(new cMenuEditBoolItem(indent(tr("Server announce broadcasts")),
 			      &newconfig.remote_usebcast));
 
-    Add(new cMenuEditBoolItem(tr("  HTTP transport for media files"), 
+    Add(new cMenuEditBoolItem(indent(tr("HTTP transport for media files")),
 			      &newconfig.remote_http_files));
 
-    Add(NewTitle(tr("Additional network services")));
+    Add(SeparatorItem(tr("Additional network services")));
     Add(ctrl_use_http =
 	new cMenuEditBoolItem(tr("HTTP server"),
 			      &newconfig.remote_use_http));
@@ -1663,7 +1735,7 @@ void cMenuSetupMediaPlayer::Set(void)
   int current = Current();
   Clear();
 
-  Add(NewTitle(tr("Playlist settings")));
+  Add(SeparatorItem(tr("Playlist settings")));
 
   Add(media_ctrl_playlist_tracknumber =
       new cMenuEditBoolItem(tr("Show the track number"),
@@ -1687,16 +1759,20 @@ void cMenuSetupMediaPlayer::Set(void)
 
   Add(new cMenuEditBoolItem(tr("Arrow keys control DVD playback"),
                             &newconfig.dvd_arrow_keys_control_playback));
+  Add(new cMenuEditBoolItem(tr("Show hidden files"),
+                            &newconfig.show_hidden_files));
+  Add(new cMenuEditBoolItem(tr("Allow removing files"),
+                            &newconfig.media_enable_delete));
 
-  Add(NewTitle(tr("Media Player")));
-  Add(new cMenuEditBitItem(tr("Play file >>"),        &newconfig.media_menu_items, MEDIA_MENU_FILES));
-  Add(new cMenuEditBitItem(tr("Play music >>"),       &newconfig.media_menu_items, MEDIA_MENU_MUSIC));
-  Add(new cMenuEditBitItem(tr("View images >>"),      &newconfig.media_menu_items, MEDIA_MENU_IMAGES));
-  Add(new cMenuEditBitItem(tr("Play DVD disc >>"),    &newconfig.media_menu_items, MEDIA_MENU_DVD));
-  Add(new cMenuEditBitItem(tr("Play audio CD >>"),    &newconfig.media_menu_items, MEDIA_MENU_CD));
-  Add(new cMenuEditBitItem(tr("Play BluRay disc >>"), &newconfig.media_menu_items, MEDIA_MENU_BLURAY));
-  Add(new cMenuEditBitItem(tr("Video settings"),      &newconfig.media_menu_items, MEDIA_MENU_VIDEO_SETUP));
-  Add(new cMenuEditBitItem(tr("Audio settings"),      &newconfig.media_menu_items, MEDIA_MENU_AUDIO_SETUP));
+  Add(SeparatorItem(tr("Media Player")));
+  Add(new cMenuEditBitItem(tr("Play file"),        &newconfig.media_menu_items, MEDIA_MENU_FILES));
+  Add(new cMenuEditBitItem(tr("Play music"),       &newconfig.media_menu_items, MEDIA_MENU_MUSIC));
+  Add(new cMenuEditBitItem(tr("View images"),      &newconfig.media_menu_items, MEDIA_MENU_IMAGES));
+  Add(new cMenuEditBitItem(tr("Play DVD disc"),    &newconfig.media_menu_items, MEDIA_MENU_DVD));
+  Add(new cMenuEditBitItem(tr("Play audio CD"),    &newconfig.media_menu_items, MEDIA_MENU_CD));
+  Add(new cMenuEditBitItem(tr("Play BluRay disc"), &newconfig.media_menu_items, MEDIA_MENU_BLURAY));
+  Add(new cMenuEditBitItem(tr("Video settings"),   &newconfig.media_menu_items, MEDIA_MENU_VIDEO_SETUP));
+  Add(new cMenuEditBitItem(tr("Audio settings"),   &newconfig.media_menu_items, MEDIA_MENU_AUDIO_SETUP));
 
   if(current<1) current=1; /* first item is not selectable */
   SetCurrent(Get(current));
@@ -1720,6 +1796,8 @@ void cMenuSetupMediaPlayer::Store(void)
   SetupStore("Media.EnableID3Scanner", xc.enable_id3_scanner);
   SetupStore("Media.DVD.ArrowKeysControlPlayback", xc.dvd_arrow_keys_control_playback);
   SetupStore("Media.MenuItems", xc.media_menu_items);
+  SetupStore("Media.ShowHiddenFiles", xc.show_hidden_files);
+  SetupStore("Media.EnableDelete", xc.media_enable_delete);
 
   Setup.Save();
 }
@@ -1760,11 +1838,7 @@ void cTestGrayscale::Show()
   int i;
 
   if(!m_Osd)
-#if VDRVERSNUM >= 10509
     m_Osd = cOsdProvider::NewOsd(OSD_X, OSD_Y, 0);
-#else
-    m_Osd = cOsdProvider::NewOsd(OSD_X, OSD_Y);
-#endif
 
   if(m_Osd) {
     if (m_Osd->CanHandleAreas(areas, sizeof(areas) / sizeof(tArea) ) == oeOk) {
@@ -1828,6 +1902,7 @@ eOSState cTestGrayscale::ProcessKey(eKeys key)
 	cXinelibDevice::Instance().ConfigureVideo(xc.hue, xc.saturation, br, xc.sharpness, xc.noise_reduction, co, xc.overscan, xc.vo_aspect_ratio);
 	m_Osd->Flush();
 	return osContinue;
+      default:; // all other keys - do nothing.
     }
   }
   return state;
@@ -1863,11 +1938,7 @@ void cTestBitmap::Show()
   int x, y, bit = 0;
 
   if(!m_Osd) {
-#if VDRVERSNUM >= 10509
     m_Osd = cOsdProvider::NewOsd(OSD_X, OSD_Y, 0);
-#else
-    m_Osd = cOsdProvider::NewOsd(OSD_X, OSD_Y);
-#endif
 
     if(m_Osd) {
       if (m_Osd->CanHandleAreas(areas, sizeof(areas) / sizeof(tArea) ) == oeOk) {
