@@ -4,7 +4,7 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: ts.c,v 1.20 2010/02/03 10:09:35 phintuka Exp $
+ * $Id: ts.c,v 1.29 2011/11/13 09:45:01 phintuka Exp $
  *
  */
 
@@ -14,6 +14,9 @@
 #include <inttypes.h>
 #include <stdlib.h>
 #include <string.h>
+#ifdef __FreeBSD__
+#include <sys/types.h>
+#endif
 
 #ifndef LOG_MODULENAME
 #  define LOG_MODULENAME "[mpeg-ts  ] "
@@ -28,13 +31,13 @@
 #ifdef LOG_PMT
 #  define LOGPMT LOGMSG
 #else
-#  define LOGPMT(x...)
+#  define LOGPMT(x...) do {} while (0)
 #endif
 
 #ifdef LOG_PCR
 #  define LOGPCR LOGMSG
 #else
-#  define LOGPCR(x...)
+#  define LOGPCR(x...) do {} while (0)
 #endif
 
 
@@ -76,6 +79,7 @@ static uint32_t ts_compute_crc32(const uint8_t *data, uint32_t length, uint32_t
 int ts_parse_pat(pat_data_t *pat, const uint8_t *pkt)
 {
   const uint8_t *original_pkt = pkt;
+  uint           pat_changed  = 0;
 
   if (! ts_PAYLOAD_START(pkt)) {
     LOGMSG ("parse_pat: PAT without payload unit start indicator");
@@ -92,7 +96,7 @@ int ts_parse_pat(pat_data_t *pat, const uint8_t *pkt)
   uint     section_syntax_indicator = ((pkt[6] >> 7) & 0x01) ;
   uint     section_length           = ((pkt[6] & 0x03) << 8) | pkt[7];
 /*uint     transport_stream_id      = (pkt[8]  << 8) | pkt[9];*/
-/*uint     version_number           = (pkt[10] >> 1) & 0x1f;*/
+  uint     version_number           = (pkt[10] >> 1) & 0x1f;
   uint     current_next_indicator   = pkt[10] & 0x01;
   uint     section_number           = pkt[11];
   uint     last_section_number      = pkt[12];
@@ -125,6 +129,15 @@ int ts_parse_pat(pat_data_t *pat, const uint8_t *pkt)
     return 0;
   }
 
+  /* check if version or crc changed */
+  if (pat->crc32 != crc32 || pat->version != version_number) {
+    LOGPMT("PAT changed: version %d->%d, crc32 0x%x->0x%x",
+           pat->version, version_number, pat->crc32, crc32);
+    pat->crc32   = crc32;
+    pat->version = version_number;
+    pat_changed++;
+  }
+
   /*
    * Process all programs in the program loop
    */
@@ -143,8 +156,19 @@ int ts_parse_pat(pat_data_t *pat, const uint8_t *pkt)
     if (program_number == 0x0000)
       continue;
 
-    pat->program_number[program_count] = program_number;
-    pat->pmt_pid[program_count] = pmt_pid;
+    if (pat->program_number[program_count] != program_number ||
+        pat->pmt_pid[program_count]        != pmt_pid) {
+
+      LOGPMT("PAT: program %d changed: number %d->%d, pmt pid 0x%x->0x%x",
+             program_count,
+             pat->program_number[program_count], program_number,
+             pat->pmt_pid[program_count], pmt_pid);
+
+      pat->program_number[program_count] = program_number;
+      pat->pmt_pid[program_count] = pmt_pid;
+
+      pat_changed++;
+    }
 
     LOGPMT("PAT acquired count=%d programNumber=0x%04x pmtPid=0x%04x",
            program_count,
@@ -156,6 +180,8 @@ int ts_parse_pat(pat_data_t *pat, const uint8_t *pkt)
 
   pat->program_number[program_count] = 0;
 
+  pat->pat_changed_flag = !!pat_changed;
+
   return program_count;
 }
 
@@ -193,6 +219,32 @@ static int find_audio_track(pmt_data_t *pmt, uint pid)
   return -1;
 }
 
+static inline int is_audio_descriptor(const uint8_t descriptor_tag)
+{
+  switch (descriptor_tag) {
+    case STREAM_DESCR_AC3:
+    case STREAM_DESCR_EAC3:
+    case STREAM_DESCR_DTS:
+    case STREAM_DESCR_AAC:
+      return 1;
+    default:;
+  }
+  return 0;
+}
+
+static ts_stream_type descriptor_to_stream_type(const uint8_t descriptor_tag)
+{
+  switch (descriptor_tag) {
+    case STREAM_DESCR_DVBSUB: return STREAM_DVBSUB;
+    case STREAM_DESCR_AC3:    return STREAM_AUDIO_AC3;
+    case STREAM_DESCR_EAC3:   return STREAM_AUDIO_EAC3;
+    case STREAM_DESCR_DTS:    return STREAM_AUDIO_DTS;
+    case STREAM_DESCR_AAC:    return STREAM_AUDIO_AAC;
+    default:;
+  }
+  return (ts_stream_type)0;
+}
+
 /*
  * ts_parse_pmt()
  *
@@ -380,11 +432,12 @@ int ts_parse_pmt (pmt_data_t *pmt, uint program_no, const uint8_t *pkt)
         break;
       case ISO_13818_PES_PRIVATE:
         for (i = 5; i < coded_length; i += stream[i+1] + 2) {
-          if ((stream[i] == STREAM_DESCR_AC3) && (pmt->audio_tracks_count < TS_MAX_AUDIO_TRACKS)) {
-            if (find_audio_track(pmt, pid) < 0) {
-              LOGPMT("parse_pmt: AC3 audio pid 0x%.4x type %2.2x", pid, stream[0]);
+          if (is_audio_descriptor(stream[i])) {
+            if (pmt->audio_tracks_count < TS_MAX_AUDIO_TRACKS &&
+                find_audio_track(pmt, pid) < 0) {
+              LOGPMT("parse_pmt: PS1 audio pid 0x%.4x type %2.2x descriptor %2.2x", pid, stream[0], stream[i]);
               pmt->audio_tracks[pmt->audio_tracks_count].pid  = pid;
-              pmt->audio_tracks[pmt->audio_tracks_count].type = STREAM_AUDIO_AC3;
+              pmt->audio_tracks[pmt->audio_tracks_count].type = descriptor_to_stream_type(stream[i]);
               /* demux_ts_get_lang_desc(pmt->audio_tracks[pmt->audio_tracks_count].lang, */
               /*                        stream + 5, stream_info_length); */
               pmt->audio_tracks_count++;
@@ -410,7 +463,7 @@ int ts_parse_pmt (pmt_data_t *pmt, uint program_no, const uint8_t *pkt)
 
               LOGPMT("parse_pmt: DVBSUB pid 0x%.4x: %s  page %d %d type %2.2x", pid,
                      pmt->spu_tracks[no].lang, pmt->spu_tracks[no].comp_page_id,
-                     pmt->spu_tracks[no].aux_page_id, stream[0]);
+                     pmt->spu_tracks[no].aux_page_id, STREAM_DVBSUB);
             }
           }
         }
@@ -454,11 +507,11 @@ int ts_parse_pmt (pmt_data_t *pmt, uint program_no, const uint8_t *pkt)
   pid = ((pmt->pmt[8] << 8) | pmt->pmt[9]) & 0x1fff;
   if (pmt->pcr_pid != pid) {
 
-    if (pmt->pcr_pid == INVALID_PID)
+    if (pmt->pcr_pid == INVALID_PID) {
       LOGPMT("parse_pmt: pcr pid 0x%.4x", pid);
-    else
+    } else {
       LOGPMT("parse_pmt: pcr pid changed 0x%.4x", pid);
-
+    }
     pmt->pcr_pid = pid;
   }
 
@@ -484,17 +537,17 @@ static int ts_get_pcr_1(const uint8_t *pkt, int64_t *ppcr)
     return 0;
 
   int64_t pcr;
-  uint    epcr;
-
   pcr  = ((int64_t) pkt[6]) << 25;
   pcr += (int64_t) (pkt[7]  << 17);
   pcr += (int64_t) (pkt[8]  << 9);
   pcr += (int64_t) (pkt[9]  << 1);
   pcr += (int64_t) ((pkt[10] & 0x80) >> 7);
 
-  epcr = ((pkt[10] & 0x1) << 8) | pkt[11];
-
+#ifdef LOG_PCR
+  uint epcr = ((pkt[10] & 0x1) << 8) | pkt[11];
   LOGPCR("ts_get_pcr: PCR: %"PRId64", EPCR: %u", pcr, epcr);
+#endif
+
   *ppcr = pcr;
   return 1;
 }
@@ -588,7 +641,7 @@ static size_t ts_add_payload(ts_state_t *ts, const uint8_t *data)
     memmove(ts->buf, ts->buf+TS_SIZE, ts->buf_len);
   }
 
-  size_t len = ts_PAYLOAD_SIZE(data);
+  ssize_t len = ts_PAYLOAD_SIZE(data);
   if (len > 0) {
     memcpy(ts->buf + ts->buf_len, ts_GET_PAYLOAD(data), len);
     ts->buf_len += len;
