@@ -4,7 +4,7 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: ts.h,v 1.13 2010/02/02 23:08:22 phintuka Exp $
+ * $Id: ts.h,v 1.17 2010/07/19 13:12:07 phintuka Exp $
  *
  */
 
@@ -76,12 +76,22 @@ typedef enum {
   ISO_14496_PART10_VIDEO = 0x1b,  /* ISO/IEC 14496-10 Video (MPEG-4 part 10/AVC, aka H.264) */
   STREAM_VIDEO_MPEG      = 0x80,
   STREAM_AUDIO_AC3       = 0x81,
-  STREAM_DVBSUB          = 0x100
+
+  /* private streams, payload type detected from PMT stream descriptor tag */
+
+  STREAM_AUDIO_EAC3      = 0x7a06, // ISO_13818_PES_PRIVATE | (STREAM_DESCR_EAC3   << 8)
+  STREAM_AUDIO_DTS       = 0x7b06, // ISO_13818_PES_PRIVATE | (STREAM_DESCR_DTS    << 8)
+  STREAM_AUDIO_AAC       = 0x7c06, // ISO_13818_PES_PRIVATE | (STREAM_DESCR_AAC    << 8)
+  STREAM_DVBSUB          = 0x5906, // ISO_13818_PES_PRIVATE | (STREAM_DESCR_DVBSUB << 8)
+
 } ts_stream_type;
 
 /* stream info descriptors */
-#define STREAM_DESCR_AC3    0x6a
 #define STREAM_DESCR_DVBSUB 0x59
+#define STREAM_DESCR_AC3    0x6a
+#define STREAM_DESCR_EAC3   0x7a
+#define STREAM_DESCR_DTS    0x7b
+#define STREAM_DESCR_AAC    0x7c
 
 /*
  * PAT
@@ -93,10 +103,21 @@ typedef enum {
 #define TS_MAX_SPU_TRACKS   32
 
 typedef struct {
-  int      program_number[TS_MAX_PROGRAMS];
+  uint16_t program_number[TS_MAX_PROGRAMS];
   uint16_t pmt_pid[TS_MAX_PROGRAMS];
+
+  uint8_t  version;
+  uint32_t crc32;
+  uint8_t  pat_changed_flag;
 } pat_data_t;
 
+/*
+ * ts_parse_pat()
+ *
+ * returns n : number of programs in PAT
+ *         0 : error or unchanged PAT
+ */
+
 int ts_parse_pat(pat_data_t *pat_data, const uint8_t *ts_data);
 
 
@@ -136,10 +157,10 @@ typedef struct {
 } pmt_data_t;
 
 /*
- * parse_pmt()
+ * ts_parse_pmt()
  *
  * returns 1 : PMT parsed and changed
- *         0 : error or unchanged pmt
+ *         0 : error or unchanged PMT
  */
 int ts_parse_pmt(pmt_data_t *pmt, uint program_no, const uint8_t *ts_data);
 
