@@ -58,6 +58,10 @@
 #  define BUF_SPU_HDMV            0x04080000
 #endif
 
+#ifndef MAX
+#  define MAX(a,b) (a>b)?(a):(b)
+#endif
+
 enum {
   SEGTYPE_PALETTE              = 0x14,
   SEGTYPE_OBJECT               = 0x15,
@@ -89,15 +93,16 @@ struct subtitle_object_s {
   uint16_t    xpos, ypos;
   uint16_t    width, height;
 
+  /* xine format */
   rle_elem_t *rle;
   unsigned int num_rle;
   size_t      data_size;
 
-#if 0
-  uint8_t    *raw_data; /* partial RLE data in HDMV format */
-  size_t      raw_data_len;
-  size_t      raw_data_size;
-#endif
+  /* HDMV format (used when object does not fit to single segment) */
+  uint32_t    data_len;      /* size of complete object */
+  uint8_t    *raw_data;      /* partial RLE data in HDMV format */
+  size_t      raw_data_len;  /* bytes buffered */
+  size_t      raw_data_size; /* allocated size */
 
   subtitle_object_t *next;
 
@@ -125,7 +130,7 @@ struct window_def_s {
 typedef struct composition_object_s composition_object_t;
 struct composition_object_s {
   uint8_t     window_id_ref;
-  uint8_t     object_id_ref;
+  uint16_t    object_id_ref;
 
   uint16_t    xpos, ypos;
 
@@ -166,27 +171,27 @@ struct presentation_segment_s {
  */
 
 #define LIST_REPLACE(list, obj, FREE_FUNC)      \
-  do {						\
-    unsigned int id = obj->id;			\
-						\
-    /* insert to list */			\
-    obj->next = list;				\
-    list = obj;					\
-						\
-    /* remove old */				\
-    while (obj->next && obj->next->id != id)	\
-      obj = obj->next;				\
-    if (obj->next) {				\
-      void *tmp = (void*)obj->next;		\
-      obj->next = obj->next->next;		\
-      FREE_FUNC(tmp);				\
-    }						\
+  do {                                          \
+    unsigned int id = obj->id;                  \
+                                                \
+    /* insert to list */                        \
+    obj->next = list;                           \
+    list = obj;                                 \
+                                                \
+    /* remove old */                            \
+    while (obj->next && obj->next->id != id)    \
+      obj = obj->next;                          \
+    if (obj->next) {                            \
+      void *tmp = (void*)obj->next;             \
+      obj->next = obj->next->next;              \
+      FREE_FUNC(tmp);                           \
+    }                                           \
   } while (0);
 
 #define LIST_DESTROY(list, FREE_FUNC) \
-  while (list) {	       \
+  while (list) {               \
     void *tmp = (void*)list;   \
-    list = list->next;	       \
+    list = list->next;         \
     FREE_FUNC(tmp);            \
   }
 
@@ -194,6 +199,7 @@ static void free_subtitle_object(void *ptr)
 {
   if (ptr) {
     free(((subtitle_object_t*)ptr)->rle);
+    free(((subtitle_object_t*)ptr)->raw_data);
     free(ptr);
   }
 }
@@ -267,8 +273,8 @@ static void segbuf_parse_segment_header(segment_buffer_t *buf)
     buf->error        = 0;
 
     if ( buf->segment_type < 0x14 ||
-	 ( buf->segment_type > 0x18 &&
-	   buf->segment_type != 0x80)) {
+         ( buf->segment_type > 0x18 &&
+           buf->segment_type != 0x80)) {
       XINE_HDMV_ERROR("unknown segment type, resetting\n");
       segbuf_reset(buf);
     }
@@ -296,7 +302,7 @@ static void segbuf_fill(segment_buffer_t *buf, uint8_t *data, size_t len)
 
 static int segbuf_segment_complete(segment_buffer_t *buf)
 {
-  return (buf->segment_len >= 0) && (buf->len >= buf->segment_len + 3);
+  return (buf->segment_len >= 0) && (buf->len >= (unsigned)buf->segment_len + 3);
 }
 
 static void segbuf_skip_segment(segment_buffer_t *buf)
@@ -336,7 +342,7 @@ static uint8_t segbuf_get_u8(segment_buffer_t *buf)
 {
   if (!(buf->error = ++buf->segment_data > buf->segment_end))
     return buf->segment_data[-1];
-  XINE_HDMV_ERROR("segbuf_get_u8: read failed (end of segment reached) !");
+  XINE_HDMV_ERROR("segbuf_get_u8: read failed (end of segment reached) !\n");
   return 0;
 }
 
@@ -361,7 +367,7 @@ static subtitle_clut_t *segbuf_decode_palette(segment_buffer_t *buf)
 
   size_t  len     = segbuf_data_length(buf);
   size_t  entries = len / 5;
-  int     i;
+  size_t  i;
 
   if (buf->error)
     return NULL;
@@ -413,17 +419,17 @@ static int segbuf_decode_rle(segment_buffer_t *buf, subtitle_object_t *obj)
     } else {
       byte = segbuf_get_u8 (buf);
       if (!(byte & 0x80)) {
-	rlep->color = 0;
-	if (!(byte & 0x40))
-	  rlep->len   = byte & 0x3f;
-	else
-	  rlep->len   = ((byte & 0x3f) << 8) | segbuf_get_u8 (buf);
+        rlep->color = 0;
+        if (!(byte & 0x40))
+          rlep->len   = byte & 0x3f;
+        else
+          rlep->len   = ((byte & 0x3f) << 8) | segbuf_get_u8 (buf);
       } else {
-	if (!(byte & 0x40))
-	  rlep->len   = byte & 0x3f;
-	else
-	  rlep->len   = ((byte & 0x3f) << 8) | segbuf_get_u8 (buf);
-	rlep->color = segbuf_get_u8 (buf);
+        if (!(byte & 0x40))
+          rlep->len   = byte & 0x3f;
+        else
+          rlep->len   = ((byte & 0x3f) << 8) | segbuf_get_u8 (buf);
+        rlep->color = segbuf_get_u8 (buf);
       }
     }
 
@@ -435,10 +441,10 @@ static int segbuf_decode_rle(segment_buffer_t *buf, subtitle_object_t *obj)
     } else {
       /* end of line marker (00 00) */
       if (x < obj->width) {
-	rlep->len = obj->width - x;
-	rlep->color = 0xff;
-	rlep++;
-	obj->num_rle ++;
+        rlep->len = obj->width - x;
+        rlep->color = 0xff;
+        rlep++;
+        obj->num_rle ++;
       }
       x = 0;
       y++;
@@ -455,42 +461,104 @@ static int segbuf_decode_rle(segment_buffer_t *buf, subtitle_object_t *obj)
   return buf->error;
 }
 
-static subtitle_object_t *segbuf_decode_object(segment_buffer_t *buf)
+static subtitle_object_t *segbuf_decode_object(segment_buffer_t *buf, subtitle_object_t *objects)
 {
-  uint8_t  object_id = segbuf_get_u16(buf);
+  uint16_t object_id = segbuf_get_u16(buf);
   uint8_t  version   = segbuf_get_u8 (buf);
   uint8_t  seq_desc  = segbuf_get_u8 (buf);
 
   XINE_HDMV_TRACE("  decode_object: object_id %d, version %d, seq 0x%x\n",
                   object_id, version, seq_desc);
 
-  //LIST_FIND();
-  subtitle_object_t *obj = calloc(1, sizeof(subtitle_object_t));
-  obj->id = object_id;
-
   if (seq_desc & 0x80) {
+    /* new object (first-in-sequence flag set) */
 
-    uint32_t data_len = segbuf_get_u24(buf);
+    subtitle_object_t *obj = calloc(1, sizeof(subtitle_object_t));
+
+    obj->id       = object_id;
+    obj->data_len = segbuf_get_u24(buf);
     obj->width    = segbuf_get_u16(buf);
     obj->height   = segbuf_get_u16(buf);
 
-    XINE_HDMV_TRACE("    object length %d bytes, size %dx%d\n", data_len, obj->width, obj->height);
-
-    segbuf_decode_rle (buf, obj);
-
     if (buf->error) {
+      XINE_HDMV_TRACE("    decode error at object header\n");
       free_subtitle_object(obj);
       return NULL;
     }
 
-  } else {
-    XINE_HDMV_ERROR("    TODO: APPEND RLE, length %d bytes\n", buf->segment_len - 4);
-    /* TODO */
-    free_subtitle_object(obj);
+    obj->data_len -= 4; /* width, height parsed */
+
+    XINE_HDMV_TRACE("    object length %d bytes, size %dx%d\n", obj->data_len, obj->width, obj->height);
+
+    if (obj->data_len > segbuf_data_length(buf)) {
+      XINE_HDMV_TRACE("    object length %d bytes, have only %zd bytes -> missing %d bytes\n",
+                      obj->data_len, segbuf_data_length(buf), obj->data_len - (int)segbuf_data_length(buf));
+
+      if (obj->raw_data)
+        free(obj->raw_data);
+
+      /* store partial RLE data in HDMV format */
+      obj->raw_data_len  = segbuf_data_length(buf);
+      obj->raw_data_size = MAX(obj->data_len, obj->raw_data_len);
+      obj->raw_data      = malloc(obj->raw_data_size);
+      memcpy(obj->raw_data, buf->segment_data, obj->raw_data_len);
+
+      return obj;
+    }
+
+    segbuf_decode_rle (buf, obj);
+
+    if (buf->error) {
+      XINE_HDMV_TRACE("    decode error at RLE data\n");
+      free_subtitle_object(obj);
+      return NULL;
+    }
+
+    return obj;
+  }
+
+  /* not first-of-sequence --> append data to already existing objct */
+
+  /* search for object */
+  while (objects && objects->id != object_id)
+    objects = objects->next;
+
+  if (!objects) {
+    XINE_HDMV_TRACE("    object not found from list, discarding segment\n");
     return NULL;
   }
 
-  return obj;
+  /* store partial RLE data in HDMV format */
+  if (objects->raw_data_size < objects->raw_data_len + segbuf_data_length(buf)) {
+    XINE_HDMV_ERROR("object larger than object size !\n");
+    return NULL;
+  }
+  memcpy(objects->raw_data + objects->raw_data_len, buf->segment_data, segbuf_data_length(buf));
+  objects->raw_data_len  += segbuf_data_length(buf);
+
+  /* if complete, decode RLE data */
+  if (objects->raw_data_len >= objects->data_len) {
+    /* create dummy buffer for segbuf_decode_rle */
+    segment_buffer_t tmpbuf = {
+      .segment_data = objects->raw_data,
+      .segment_end  = objects->raw_data + objects->raw_data_len,
+    };
+
+    /* decode RLE data */
+    segbuf_decode_rle (&tmpbuf, objects);
+
+    if (tmpbuf.error) {
+      XINE_HDMV_TRACE("    error decoding multi-segment object\n");
+    }
+
+    /* free decode buffer */
+    free(objects->raw_data);
+    objects->raw_data      = NULL;
+    objects->raw_data_len  = 0;
+    objects->raw_data_size = 0;
+  }
+
+  return NULL;
 }
 
 static window_def_t *segbuf_decode_window_definition(segment_buffer_t *buf)
@@ -522,13 +590,14 @@ static int segbuf_decode_video_descriptor(segment_buffer_t *buf)
   uint8_t  frame_rate = segbuf_get_u8 (buf);
 
   XINE_HDMV_TRACE("  video_descriptor: %dx%d fps %d\n", width, height, frame_rate);
+
   return buf->error;
 }
 
 static int segbuf_decode_composition_descriptor(segment_buffer_t *buf, composition_descriptor_t *descr)
 {
   descr->number = segbuf_get_u16(buf);
-  descr->state  = segbuf_get_u8 (buf);
+  descr->state  = segbuf_get_u8 (buf) & 0xc0;
 
   XINE_HDMV_TRACE("  composition_descriptor: number %d, state %d\n", descr->number, descr->state);
   return buf->error;
@@ -632,6 +701,14 @@ typedef struct spuhdmv_decoder_s {
 
 } spuhdmv_decoder_t;
 
+static void free_objs(spuhdmv_decoder_t *this)
+{
+  LIST_DESTROY (this->cluts,    free);
+  LIST_DESTROY (this->objects,  free_subtitle_object);
+  LIST_DESTROY (this->windows,  free);
+  LIST_DESTROY (this->segments, free_presentation_segment);
+}
+
 static int decode_palette(spuhdmv_decoder_t *this)
 {
   /* decode */
@@ -647,7 +724,7 @@ static int decode_palette(spuhdmv_decoder_t *this)
 static int decode_object(spuhdmv_decoder_t *this)
 {
   /* decode */
-  subtitle_object_t *obj = segbuf_decode_object(this->buf);
+  subtitle_object_t *obj = segbuf_decode_object(this->buf, this->objects);
   if (!obj)
     return 1;
 
@@ -677,6 +754,11 @@ static int decode_presentation_segment(spuhdmv_decoder_t *this)
 
   seg->pts = this->pts;
 
+  /* epoch start or acquistion point -> drop cached objects */
+  if (seg->comp_descr.state) {
+    free_objs(this);
+  }
+
   /* replace */
   if (this->segments)
     LIST_DESTROY(this->segments, free_presentation_segment);
@@ -686,12 +768,12 @@ static int decode_presentation_segment(spuhdmv_decoder_t *this)
 }
 
 static int show_overlay(spuhdmv_decoder_t *this, composition_object_t *cobj, unsigned int palette_id_ref,
-			int overlay_index, int64_t pts, int force_update)
+                        int overlay_index, int64_t pts, int force_update)
 {
   video_overlay_manager_t *ovl_manager = this->stream->video_out->get_overlay_manager(this->stream->video_out);
   metronom_t              *metronom    = this->stream->metronom;
-  video_overlay_event_t    event       = {0};
-  vo_overlay_t             overlay     = {0};
+  video_overlay_event_t    event       = {.vpts = 0};
+  vo_overlay_t             overlay     = {.data_size = 0};
 
   /* find palette */
   subtitle_clut_t *clut = this->cluts;
@@ -710,6 +792,10 @@ static int show_overlay(spuhdmv_decoder_t *this, composition_object_t *cobj, uns
     XINE_HDMV_TRACE("  show_overlay: object %d not found !\n", cobj->object_id_ref);
     return -1;
   }
+  if (!obj->rle) {
+    XINE_HDMV_TRACE("  show_overlay: object %d RLE data not decoded !\n", cobj->object_id_ref);
+    return -1;
+  }
 
   /* find window */
   window_def_t *wnd = this->windows;
@@ -780,7 +866,7 @@ static int show_overlay(spuhdmv_decoder_t *this, composition_object_t *cobj, uns
 
 static void hide_overlays(spuhdmv_decoder_t *this, int64_t pts)
 {
-  video_overlay_event_t event = {0};
+  video_overlay_event_t event = {.vpts = 0};
   int i = 0;
 
   while (this->overlay_handles[i] >= 0) {
@@ -809,7 +895,7 @@ static void update_overlays(spuhdmv_decoder_t *this)
 
   while (pseg) {
 
-    if (!pseg->comp_descr.state) {
+    if (!pseg->object_number) {
 
       /* HIDE */
       if (!pseg->shown)
@@ -837,17 +923,9 @@ static void update_overlays(spuhdmv_decoder_t *this)
   }
 }
 
-static void free_objs(spuhdmv_decoder_t *this)
-{
-  LIST_DESTROY (this->cluts,    free);
-  LIST_DESTROY (this->objects,  free_subtitle_object);
-  LIST_DESTROY (this->windows,  free);
-  LIST_DESTROY (this->segments, free_presentation_segment);
-}
-
 static void decode_segment(spuhdmv_decoder_t *this)
 {
-  XINE_HDMV_TRACE("*** new segment, pts %010"PRId64": 0x%02x (%8d bytes)",
+  XINE_HDMV_TRACE("*** new segment, pts %010"PRId64": 0x%02x (%8d bytes)\n",
                   this->pts, this->buf->segment_type, this->buf->segment_len);
 
   switch (segbuf_segment_type(this->buf)) {
@@ -872,8 +950,10 @@ static void decode_segment(spuhdmv_decoder_t *this)
     break;
   case SEGTYPE_END_OF_DISPLAY:
     XINE_HDMV_TRACE("  segment: END OF DISPLAY\n");
+#if 0
     /* drop all cached objects */
     free_objs(this);
+#endif
     break;
   default:
     XINE_HDMV_ERROR("  segment type 0x%x unknown, skipping\n", segbuf_segment_type(this->buf));
