@@ -168,6 +168,10 @@
 #  define BUF_SPU_HDMV            0x04080000
 #endif
 
+#ifndef DEMUX_OPTIONAL_DATA_FLUSH
+#  define DEMUX_OPTIONAL_DATA_FLUSH 0x10000
+#endif
+
 /*
   #define TS_LOG
   #define TS_PMT_LOG
@@ -216,6 +220,16 @@
 #define ISO13522_STREAM  0xF3
 #define PROG_STREAM_DIR  0xFF
 
+/* descriptors in PMT stream info */
+#define DESCRIPTOR_REG_FORMAT  0x05
+#define DESCRIPTOR_LANG        0x0a
+#define DESCRIPTOR_TELETEXT    0x56
+#define DESCRIPTOR_DVBSUB      0x59
+#define DESCRIPTOR_AC3         0x6a
+#define DESCRIPTOR_EAC3        0x7a
+#define DESCRIPTOR_DTS         0x7b
+#define DESCRIPTOR_AAC         0x7c
+
   typedef enum
     {
       ISO_11172_VIDEO = 0x01,           /* ISO/IEC 11172 Video */
@@ -283,7 +297,6 @@ typedef struct {
   buf_element_t   *buf;
   unsigned int     counter;
   uint16_t         descriptor_tag; /* +0x100 for PES stream IDs (no available TS descriptor tag?) */
-  int64_t          packet_count;
   int              corrupted_pes;
   uint32_t         buffered_bytes;
   int              autodetected;
@@ -295,7 +308,7 @@ typedef struct {
 
 typedef struct {
   spu_dvb_descriptor_t desc;
-  int pid;
+  unsigned int pid;
   int media_index;
 } demux_ts_spu_lang;
 
@@ -303,7 +316,7 @@ typedef struct {
 #define MAX_AUDIO_TRACKS 32
 
 typedef struct {
-    int pid;
+    unsigned int pid;
     int media_index;
     char lang[4];
 } demux_ts_audio_track;
@@ -329,7 +342,6 @@ typedef struct {
   int              pkt_size;   /* TS packet size */
   int              pkt_offset; /* TS packet offset */
 
-  int              blockSize;
   int              rate;
   int              media_num;
   demux_ts_media   media[MAX_PIDS];
@@ -344,17 +356,12 @@ typedef struct {
    * and audio PIDs, we keep the index of the corresponding entry
    * inthe media[] array.
    */
-  unsigned int     programNumber;
-  unsigned int     pcrPid;
-  unsigned int     pid;
-  unsigned int     pid_count;
   unsigned int     videoPid;
   unsigned int     videoMedia;
 
   demux_ts_audio_track audio_tracks[MAX_AUDIO_TRACKS];
   int              audio_tracks_count;
 
-  int              send_end_buffers;
   int64_t          last_pts[2];
   int              send_newpts;
   int              buf_flag_seek;
@@ -727,15 +734,12 @@ static void demux_ts_parse_pat (demux_ts_t*this, unsigned char *original_pkt,
       this->videoPid = INVALID_PID;
       this->spu_pid = INVALID_PID;
 
-      this->pmt_pid[program_count] = pmt_pid;
-
       if (this->pmt[program_count] != NULL) {
         free(this->pmt[program_count]);
         this->pmt[program_count] = NULL;
         this->pmt_write_ptr[program_count] = NULL;
       }
     }
-
 #ifdef TS_PAT_LOG
     if (this->program_number[program_count] != INVALID_PROGRAM)
       printf ("demux_ts: PAT acquired count=%d programNumber=0x%04x "
@@ -748,34 +752,37 @@ static void demux_ts_parse_pat (demux_ts_t*this, unsigned char *original_pkt,
 }
 
 static int demux_ts_parse_pes_header (xine_t *xine, demux_ts_media *m,
-				      uint8_t *buf, int packet_len,
-                                      xine_stream_t *stream) {
+                                      uint8_t *buf, int packet_len) {
 
   unsigned char *p;
-  uint32_t       header_len;
+  int            header_len;
   int64_t        pts;
   uint32_t       stream_id;
-  int            pkt_len;
+
+  if (packet_len < 9) {
+    xprintf (xine, XINE_VERBOSITY_DEBUG,
+             "demux_ts: too short PES packet header (%d bytes)\n", packet_len);
+    return 0;
+  }
 
   p = buf;
-  pkt_len = packet_len;
 
   /* we should have a PES packet here */
 
   if (p[0] || p[1] || (p[2] != 1)) {
     xprintf (xine, XINE_VERBOSITY_DEBUG,
-	     "demux_ts: error %02x %02x %02x (should be 0x000001) \n", p[0], p[1], p[2]);
+             "demux_ts: error %02x %02x %02x (should be 0x000001) \n", p[0], p[1], p[2]);
     return 0 ;
   }
 
-  packet_len -= 6;
   /* packet_len = p[4] << 8 | p[5]; */
   stream_id  = p[3];
+  header_len = p[8];
 
-  if (packet_len==0)
-  {
+  /* sometimes corruption on header_len causes segfault in memcpy below */
+  if (header_len + 9 > packet_len) {
     xprintf (xine, XINE_VERBOSITY_DEBUG,
-             "demux_ts: error pes length 0\n");
+             "demux_ts: illegal value for PES_header_data_length (0x%x)\n", header_len);
     return 0;
   }
 
@@ -786,6 +793,10 @@ static int demux_ts_parse_pes_header (xine_t *xine, demux_ts_media *m,
 
   if (p[7] & 0x80) { /* pts avail */
 
+    if (header_len < 5) {
+      return 0;
+    }
+
     pts  = (int64_t)(p[ 9] & 0x0E) << 29 ;
     pts |=  p[10]         << 22 ;
     pts |= (p[11] & 0xFE) << 14 ;
@@ -810,17 +821,8 @@ static int demux_ts_parse_pes_header (xine_t *xine, demux_ts_media *m,
 
   m->pts       = pts;
 
-  header_len = p[8];
-
-  /* sometimes corruption on header_len causes segfault in memcpy below */
-  if (header_len + 9 > pkt_len) {
-    xprintf (xine, XINE_VERBOSITY_DEBUG,
-	     "demux_ts: illegal value for PES_header_data_length (0x%x)\n", header_len);
-    return 0;
-  }
-
   p += header_len + 9;
-  packet_len -= header_len + 3;
+  packet_len -= header_len + 9;
 
   if (m->descriptor_tag == STREAM_VIDEO_VC1) {
     m->content   = p;
@@ -853,10 +855,15 @@ static int demux_ts_parse_pes_header (xine_t *xine, demux_ts_media *m,
      * these "raw" streams may begin with a byte that looks like a stream type.
      * For audio streams, m->type already contains the stream no.
      */
-    if((m->descriptor_tag == STREAM_AUDIO_AC3) ||    /* ac3 - raw */
-       (p[0] == 0x0B && p[1] == 0x77)) { /* ac3 - syncword */
+    if(m->descriptor_tag == HDMV_AUDIO_84_EAC3) {
       m->content   = p;
       m->size = packet_len;
+      m->type |= BUF_AUDIO_EAC3;
+      return 1;
+
+    } else if (m->descriptor_tag == STREAM_AUDIO_AC3) {    /* ac3 - raw */
+      m->content = p;
+      m->size = packet_len;
       m->type |= BUF_AUDIO_A52;
       return 1;
 
@@ -875,8 +882,15 @@ static int demux_ts_parse_pes_header (xine_t *xine, demux_ts_media *m,
       m->type |= BUF_AUDIO_DTS;
       return 1;
 
+    } else if (packet_len < 2) {
+      return 0;
+
     } else if (m->descriptor_tag == HDMV_AUDIO_80_PCM) {
 
+      if (packet_len < 4) {
+        return 0;
+      }
+
       m->content = p + 4;
       m->size    = packet_len - 4;
       m->type   |= BUF_AUDIO_LPCM_BE;
@@ -888,7 +902,7 @@ static int demux_ts_parse_pes_header (xine_t *xine, demux_ts_media *m,
       return 1;
 
     } else if (m->descriptor_tag == ISO_13818_PES_PRIVATE
-	     && p[0] == 0x20 && p[1] == 0x00) {
+               && p[0] == 0x20 && p[1] == 0x00) {
       /* DVBSUB */
       long payload_len = ((buf[4] << 8) | buf[5]) - header_len - 3;
 
@@ -897,6 +911,13 @@ static int demux_ts_parse_pes_header (xine_t *xine, demux_ts_media *m,
       m->type |= BUF_SPU_DVB;
       m->buf->decoder_info[2] = payload_len;
       return 1;
+
+    } else if (p[0] == 0x0B && p[1] == 0x77) { /* ac3 - syncword */
+      m->content = p;
+      m->size = packet_len;
+      m->type |= BUF_AUDIO_A52;
+      return 1;
+
     } else if ((p[0] & 0xE0) == 0x20) {
       spu_id = (p[0] & 0x1f);
 
@@ -906,14 +927,20 @@ static int demux_ts_parse_pes_header (xine_t *xine, demux_ts_media *m,
       return 1;
     } else if ((p[0] & 0xF0) == 0x80) {
 
+      if (packet_len < 4) {
+        return 0;
+      }
+
       m->content   = p+4;
       m->size      = packet_len - 4;
       m->type      |= BUF_AUDIO_A52;
       return 1;
 
+#if 0
+    /* commented out: does not set PCM type. Decoder can't handle raw PCM stream without configuration. */
     } else if ((p[0]&0xf0) == 0xa0) {
 
-      int pcm_offset;
+      unsigned int pcm_offset;
 
       for (pcm_offset=0; ++pcm_offset < packet_len-1 ; ){
         if (p[pcm_offset] == 0x01 && p[pcm_offset+1] == 0x80 ) { /* START */
@@ -922,13 +949,18 @@ static int demux_ts_parse_pes_header (xine_t *xine, demux_ts_media *m,
         }
       }
 
+      if (packet_len < pcm_offset) {
+        return 0;
+      }
+
       m->content   = p+pcm_offset;
       m->size      = packet_len-pcm_offset;
       m->type      |= BUF_AUDIO_LPCM_BE;
       return 1;
+#endif
     }
 
-  } else if ((stream_id >= 0xbc) && ((stream_id & 0xf0) == 0xe0)) {
+  } else if ((stream_id & 0xf0) == 0xe0) {
 
     m->content   = p;
     m->size      = packet_len;
@@ -986,9 +1018,48 @@ static int demux_ts_parse_pes_header (xine_t *xine, demux_ts_media *m,
 }
 
 
+static void fill_extra_info(demux_ts_t *this, buf_element_t *buf)
+{
+  off_t len = this->input->get_length (this->input);
+  off_t pos = this->input->get_current_pos (this->input);
+
+  if (len) {
+    buf->extra_info->input_normpos = (int)( (double) pos * 65535 / len );
+  }
+
+  if (this->input->get_current_time)
+    buf->extra_info->input_time = this->input->get_current_time(this->input);
+
+  if (this->rate) {
+    if (buf->extra_info->input_time <= 0)
+      buf->extra_info->input_time = (int)((int64_t)pos * 1000 / this->rate);
+
+    buf->extra_info->total_time = (int)((int64_t)len * 1000 / this->rate);
+  }
+}
+
 /*
  *  buffer arriving pes data
  */
+static void flush_demux_ts_media(demux_ts_media *m)
+{
+  if (m && m->buf) {
+    m->buf->content = m->buf->mem;
+    m->buf->size = m->buffered_bytes;
+    m->buf->type = m->type;
+    m->buf->pts = m->pts;
+    m->buf->decoder_info[0] = 1;
+
+    m->fifo->put(m->fifo, m->buf);
+    m->buffered_bytes = 0;
+    m->buf = NULL; /* forget about buf -- not our responsibility anymore */
+
+#ifdef TS_LOG
+      printf ("demux_ts: produced buffer, pts=%lld\n", m->pts);
+#endif
+  }
+}
+
 static void demux_ts_buffer_pes(demux_ts_t*this, unsigned char *ts,
                                 unsigned int mediaIndex,
                                 unsigned int pus,
@@ -1018,9 +1089,6 @@ static void demux_ts_buffer_pes(demux_ts_t*this, unsigned char *ts,
   if (pus) { /* new PES packet */
 
     if (m->buffered_bytes) {
-      m->buf->content = m->buf->mem;
-      m->buf->size = m->buffered_bytes;
-      m->buf->type = m->type;
       if( (m->buf->type & 0xffff0000) == BUF_SPU_DVD ) {
         m->buf->decoder_flags |= BUF_FLAG_SPECIAL;
         m->buf->decoder_info[1] = BUF_SPECIAL_SPU_DVD_SUBTYPE;
@@ -1029,26 +1097,13 @@ static void demux_ts_buffer_pes(demux_ts_t*this, unsigned char *ts,
       else {
         m->buf->decoder_flags |= BUF_FLAG_FRAME_END;
       }
-      m->buf->pts = m->pts;
-      m->buf->decoder_info[0] = 1;
 
-      if( this->input->get_length (this->input) )
-        m->buf->extra_info->input_normpos = (int)( (double) this->input->get_current_pos (this->input) *
-                                         65535 / this->input->get_length (this->input) );
-      if (this->rate)
-        m->buf->extra_info->input_time = (int)((int64_t)this->input->get_current_pos (this->input)
-                                         * 1000 / (this->rate * 50));
-      m->fifo->put(m->fifo, m->buf);
-      m->buffered_bytes = 0;
-      m->buf = NULL; /* forget about buf -- not our responsibility anymore */
-#ifdef TS_LOG
-      printf ("demux_ts: produced buffer, pts=%lld\n", m->pts);
-#endif
+      flush_demux_ts_media(m);
     }
     /* allocate the buffer here, as pes_header needs a valid buf for dvbsubs */
      m->buf = m->fifo->buffer_pool_alloc(m->fifo);
 
-    if (!demux_ts_parse_pes_header(this->stream->xine, m, ts, len, this->stream)) {
+    if (!demux_ts_parse_pes_header(this->stream->xine, m, ts, len)) {
       m->buf->free_buffer(m->buf);
       m->buf = NULL;
 
@@ -1067,31 +1122,15 @@ static void demux_ts_buffer_pes(demux_ts_t*this, unsigned char *ts,
       m->corrupted_pes = 0;
       memcpy(m->buf->mem, ts+len-m->size, m->size);
       m->buffered_bytes = m->size;
+
+      fill_extra_info(this, m->buf);
     }
 
   } else if (!m->corrupted_pes) { /* no pus -- PES packet continuation */
 
     if ((m->buffered_bytes + len) > MAX_PES_BUF_SIZE) {
-      m->buf->content = m->buf->mem;
-      m->buf->size = m->buffered_bytes;
-      m->buf->type = m->type;
-      m->buf->pts = m->pts;
-      m->buf->decoder_info[0] = 1;
-      if( this->input->get_length (this->input) )
-        m->buf->extra_info->input_normpos = (int)( (double) this->input->get_current_pos (this->input) *
-                                         65535 / this->input->get_length (this->input) );
-      if (this->rate)
-        m->buf->extra_info->input_time = (int)((int64_t)this->input->get_current_pos (this->input)
-                                         * 1000 / (this->rate * 50));
-
-      m->fifo->put(m->fifo, m->buf);
-      m->buffered_bytes = 0;
+      flush_demux_ts_media(m);
       m->buf = m->fifo->buffer_pool_alloc(m->fifo);
-
-#ifdef TS_LOG
-      printf ("demux_ts: produced buffer, pts=%lld\n", m->pts);
-#endif
-
     }
     memcpy(m->buf->mem + m->buffered_bytes, ts, len);
     m->buffered_bytes += len;
@@ -1134,7 +1173,7 @@ static void demux_ts_get_lang_desc(demux_ts_t *this, char *dest,
   while (d < (data + length))
 
     {
-      if (d[0] == 10 && d[1] >= 4)
+      if (d[0] == DESCRIPTOR_LANG && d[1] >= 4)
 
 	{
       memcpy(dest, d + 2, 3);
@@ -1159,7 +1198,7 @@ static void demux_ts_get_reg_desc(demux_ts_t *this, uint32_t *dest,
   while (d < (data + length))
 
     {
-      if (d[0] == 5 && d[1] >= 4)
+      if (d[0] == DESCRIPTOR_REG_FORMAT && d[1] >= 4)
 
 	{
           *dest = (d[2] << 24) | (d[3] << 16) | (d[4] << 8) | d[5];
@@ -1186,6 +1225,15 @@ static inline int ts_payloadsize(unsigned char * tsp)
   return 184;
 }
 
+/* check if an apid is in the list of known apids */
+static int apid_check(demux_ts_t*this, unsigned int pid) {
+  int i;
+  for (i = 0; i < this->audio_tracks_count; i++) {
+    if (this->audio_tracks[i].pid == pid)
+      return i;
+  }
+  return -1;
+}
 
 /*
  * NAME demux_ts_parse_pmt
@@ -1232,7 +1280,7 @@ static void demux_ts_parse_pmt (demux_ts_t     *this,
     pkt+=pkt[4]; /* pointer to start of section */
     offset=1;
 
-    if (this->pmt[program_count] != NULL) free(this->pmt[program_count]);
+    free(this->pmt[program_count]);
     this->pmt[program_count] = (uint8_t *) calloc(4096, sizeof(unsigned char));
     this->pmt_write_ptr[program_count] = this->pmt[program_count];
 
@@ -1297,7 +1345,7 @@ printf("Program Number is %i, looking for %i\n",program_number,this->program_num
 
   count=ts_payloadsize(originalPkt);
 
-  ptr = originalPkt+offset+(PKT_SIZE-count);
+  ptr = (char*)originalPkt+offset+(PKT_SIZE-count);
   len = count-offset;
   memcpy (this->pmt_write_ptr[program_count], ptr, len);
   this->pmt_write_ptr[program_count] +=len;
@@ -1435,14 +1483,7 @@ printf("Program Number is %i, looking for %i\n",program_number,this->program_num
     case ISO_13818_PART7_AUDIO:
     case ISO_14496_PART3_AUDIO:
       if (this->audio_tracks_count < MAX_AUDIO_TRACKS) {
-        int i, found = 0;
-        for(i = 0; i < this->audio_tracks_count; i++) {
-            if(this->audio_tracks[i].pid == pid) {
-                found = 1;
-                break;
-            }
-        }
-        if(!found) {
+        if (apid_check(this, pid) < 0) {
 #ifdef TS_PMT_LOG
             printf ("demux_ts: PMT audio pid 0x%.4x type %2.2x\n", pid, stream[0]);
 #endif
@@ -1473,33 +1514,30 @@ printf("Program Number is %i, looking for %i\n",program_number,this->program_num
       break;
     case ISO_13818_PES_PRIVATE:
       for (i = 5; i < coded_length; i += stream[i+1] + 2) {
-          if ((stream[i] == 0x6a) && (this->audio_tracks_count < MAX_AUDIO_TRACKS)) {
-          int i, found = 0;
-          for(i = 0; i < this->audio_tracks_count; i++) {
-            if(this->audio_tracks[i].pid == pid) {
-              found = 1;
-              break;
-             }
-	  }
-          if(!found) {
+        if (((stream[i] == 0x6a) || (stream[i] == 0x7a)) && (this->audio_tracks_count < MAX_AUDIO_TRACKS)) {
+          if (apid_check(this, pid) < 0) {
 #ifdef TS_PMT_LOG
             printf ("demux_ts: PMT AC3 audio pid 0x%.4x type %2.2x\n", pid, stream[0]);
 #endif
-          demux_ts_pes_new(this, this->media_num, pid,
-                           this->audio_fifo, STREAM_AUDIO_AC3);
+            if (stream[i] == 0x6a)
+              demux_ts_pes_new(this, this->media_num, pid,
+                               this->audio_fifo, STREAM_AUDIO_AC3);
+            else
+              demux_ts_pes_new(this, this->media_num, pid,
+                               this->audio_fifo, HDMV_AUDIO_84_EAC3);
 
-          this->audio_tracks[this->audio_tracks_count].pid = pid;
-          this->audio_tracks[this->audio_tracks_count].media_index = this->media_num;
-          this->media[this->media_num].type = this->audio_tracks_count;
-          demux_ts_get_lang_desc(this, this->audio_tracks[this->audio_tracks_count].lang,
-                                 stream + 5, stream_info_length);
+            this->audio_tracks[this->audio_tracks_count].pid = pid;
+            this->audio_tracks[this->audio_tracks_count].media_index = this->media_num;
+            this->media[this->media_num].type = this->audio_tracks_count;
+            demux_ts_get_lang_desc(this, this->audio_tracks[this->audio_tracks_count].lang,
+                                   stream + 5, stream_info_length);
           demux_send_special_audio_buf(this, STREAM_AUDIO_AC3, this->audio_tracks_count);
-          this->audio_tracks_count++;
-          break;
+            this->audio_tracks_count++;
+            break;
           }
         }
         /* Teletext */
-        else if (stream[i] == 0x56)
+        else if (stream[i] == DESCRIPTOR_TELETEXT)
           {
 #ifdef TS_PMT_LOG
             printf ("demux_ts: PMT Teletext, pid: 0x%.4x type %2.2x\n", pid, stream[0]);
@@ -1508,13 +1546,13 @@ printf("Program Number is %i, looking for %i\n",program_number,this->program_num
               printf ("%.2x ", stream[i]);
             printf ("\n");
 #endif
-          break;
+            break;
           }
 
 	/* DVBSUB */
-	else if (stream[i] == 0x59)
+	else if (stream[i] == DESCRIPTOR_DVBSUB)
 	  {
-	    int pos;
+	    unsigned int pos;
             for (pos = i + 2;
 		 pos + 8 <= i + 2 + stream[i + 1]
 		   && this->spu_langs_count < MAX_SPU_LANGS;
@@ -1597,14 +1635,7 @@ printf("Program Number is %i, looking for %i\n",program_number,this->program_num
  * FIXME: This will need expanding if we ever see a DTS or other media format here.
  */
         if ((this->audio_tracks_count < MAX_AUDIO_TRACKS) && (stream[0] >= 0x80) ) {
-        int i, found = 0;
-        for(i = 0; i < this->audio_tracks_count; i++) {
-            if(this->audio_tracks[i].pid == pid) {
-                found = 1;
-                break;
-            }
-        }
-        if(!found) {
+          if (apid_check(this,pid) < 0) {
             uint32_t format_identifier=0;
             demux_ts_get_reg_desc(this, &format_identifier,
                     stream + 5, stream_info_length);
@@ -1641,6 +1672,7 @@ printf("Program Number is %i, looking for %i\n",program_number,this->program_num
     section_length -= coded_length;
   }
 
+#if 0
   /*
    * Get the current PCR PID.
    */
@@ -1656,6 +1688,7 @@ printf("Program Number is %i, looking for %i\n",program_number,this->program_num
 #endif
     this->pcrPid = pid;
   }
+#endif
 
   if ( this->stream->spu_channel>=0 && this->spu_langs_count>0 )
     demux_ts_update_spu_channel( this );
@@ -1696,7 +1729,7 @@ static int sync_correct(demux_ts_t*this, uint8_t *buf, int32_t npkt_read) {
     memmove(&buf[0], &buf[n + p * this->pkt_size],
 	    ((this->pkt_size * (npkt_read - p)) - n));
     read_length = this->input->read(this->input,
-				    &buf[(this->pkt_size * (npkt_read - p)) - n],
+				    (char*)&buf[(this->pkt_size * (npkt_read - p)) - n],
 				    n + p * this->pkt_size);
     /* FIXME: when read_length is not as required... we now stop demuxing */
     if (read_length != (n + p * this->pkt_size)) {
@@ -1766,7 +1799,7 @@ static unsigned char * demux_synchronise(demux_ts_t* this) {
 
     /* NEW: handle read returning less packets than NPKT_PER_READ... */
     do {
-      read_length = this->input->read(this->input, this->buf,
+      read_length = this->input->read(this->input, (char*)this->buf,
 				      this->pkt_size * NPKT_PER_READ);
       if (read_length < 0 || read_length % this->pkt_size) {
 	xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
@@ -1808,6 +1841,7 @@ static unsigned char * demux_synchronise(demux_ts_t* this) {
 }
 
 
+#ifdef TS_LOG
 static int64_t demux_ts_adaptation_field_parse(uint8_t *data,
 					       uint32_t adaptation_field_length) {
 
@@ -1851,6 +1885,9 @@ static int64_t demux_ts_adaptation_field_parse(uint8_t *data,
   }
 #endif
   if(PCR_flag) {
+    if (adaptation_field_length < offset + 6)
+      return 0;
+
     PCR  = (((int64_t) data[offset]) & 0xFF) << 25;
     PCR += (int64_t) ((data[offset+1] & 0xFF) << 17);
     PCR += (int64_t) ((data[offset+2] & 0xFF) << 9);
@@ -1865,6 +1902,9 @@ static int64_t demux_ts_adaptation_field_parse(uint8_t *data,
     offset+=6;
   }
   if(OPCR_flag) {
+    if (adaptation_field_length < offset + 6)
+      return PCR;
+
     OPCR = data[offset] << 25;
     OPCR |= data[offset+1] << 17;
     OPCR |= data[offset+2] << 9;
@@ -1893,16 +1933,7 @@ static int64_t demux_ts_adaptation_field_parse(uint8_t *data,
 #endif
   return PCR;
 }
-
-/* check if an apid is in the list of known apids */
-static int apid_check(demux_ts_t*this, unsigned int pid) {
-  int i;
-  for(i=0; i<this->audio_tracks_count; i++) {
-    if(this->audio_tracks[i].pid == pid)
-      return i;
-  }
-  return -1;
-}
+#endif
 
 /* transport stream packet layer */
 static void demux_ts_parse_packet (demux_ts_t*this) {
@@ -1919,7 +1950,7 @@ static void demux_ts_parse_packet (demux_ts_t*this) {
   unsigned int   data_offset;
   unsigned int   data_len;
   uint32_t       program_count;
-  int i;
+  unsigned int i;
 
   /* get next synchronised packet, or NULL */
   originalPkt = demux_synchronise(this);
@@ -1972,8 +2003,10 @@ static void demux_ts_parse_packet (demux_ts_t*this) {
     for (i=0; i < this->scrambled_npids; i++) {
       if (this->scrambled_pids[i] == pid) return;
     }
-    this->scrambled_pids[this->scrambled_npids] = pid;
-    this->scrambled_npids++;
+    if (this->scrambled_npids < MAX_PIDS) {
+      this->scrambled_pids[this->scrambled_npids] = pid;
+      this->scrambled_npids++;
+    }
 
     xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "demux_ts: PID 0x%.4x is scrambled!\n", pid);
     return;
@@ -1983,9 +2016,11 @@ static void demux_ts_parse_packet (demux_ts_t*this) {
 
   if( adaptation_field_control & 0x2 ){
     uint32_t adaptation_field_length = originalPkt[4];
+#ifdef TS_LOG
     if (adaptation_field_length > 0) {
       demux_ts_adaptation_field_parse (originalPkt+5, adaptation_field_length);
     }
+#endif
     /*
      * Skip adaptation header.
      */
@@ -2061,14 +2096,8 @@ static void demux_ts_parse_packet (demux_ts_t*this) {
       }
     } else if ( (pes_stream_id >= AUDIO_STREAM_S) && (pes_stream_id <= AUDIO_STREAM_E) ) {
        if (this->audio_tracks_count < MAX_AUDIO_TRACKS) {
-           int i, found = 0;
-           for(i = 0; i < this->audio_tracks_count; i++) {
-               if(this->audio_tracks[i].pid == pid) {
-                   found = 1;
-                   break;
-               }
-           }
-           if(!found) {
+
+           if (apid_check(this, pid) < 0) {
 #ifdef TS_PMT_LOG
                xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
                         "demux_ts: auto-detected audio pid 0x%.4x\n", pid);
@@ -2246,7 +2275,6 @@ static void demux_ts_send_headers (demux_plugin_t *this_gen) {
 
   this->status = DEMUX_OK ;
 
-  this->send_end_buffers  = 1;
   this->scrambled_npids   = 0;
 
   /* DVBSUB */
@@ -2264,19 +2292,23 @@ static int demux_ts_seek (demux_plugin_t *this_gen,
 
   demux_ts_t *this = (demux_ts_t *) this_gen;
   int i;
-  start_time /= 1000;
   start_pos = (off_t) ( (double) start_pos / 65535 *
               this->input->get_length (this->input) );
 
   if (this->input->get_capabilities(this->input) & INPUT_CAP_SEEKABLE) {
 
     if ((!start_pos) && (start_time)) {
-      start_pos = start_time;
-      start_pos *= this->rate;
-      start_pos *= 50;
-    }
-    this->input->seek (this->input, start_pos, SEEK_SET);
 
+      if (this->input->seek_time) {
+        this->input->seek_time (this->input, start_time, SEEK_SET);
+      } else {
+        start_pos = start_time / 1000;
+        start_pos *= this->rate;
+        this->input->seek (this->input, start_pos, SEEK_SET);
+      }
+    } else {
+      this->input->seek (this->input, start_pos, SEEK_SET);
+    }
   }
 
   this->send_newpts = 1;
@@ -2312,8 +2344,7 @@ static int demux_ts_get_stream_length (demux_plugin_t *this_gen) {
   demux_ts_t*this = (demux_ts_t*)this_gen;
 
   if (this->rate)
-    return (int)((int64_t) this->input->get_length (this->input)
-                 * 1000 / (this->rate * 50));
+    return (int)((int64_t) this->input->get_length (this->input) * 1000 / this->rate);
   else
     return 0;
 }
@@ -2324,17 +2355,68 @@ static uint32_t demux_ts_get_capabilities(demux_plugin_t *this_gen)
   return DEMUX_CAP_AUDIOLANG | DEMUX_CAP_SPULANG;
 }
 
+static void flush_decoders(demux_ts_t *this)
+{
+  buf_element_t *buf;
+  xine_stream_t *stream = this->stream;
+  unsigned int i;
+
+  /* flush demuxer caches*/
+  for (i = 0; i < MAX_PIDS; i++) {
+    demux_ts_media *m = &this->media[i];
+    if (m->buf) {
+      m->buf->decoder_flags |= BUF_FLAG_FRAME_END;
+      flush_demux_ts_media(m);
+    }
+  }
+
+  /* flush decoders */
+  buf = stream->video_fifo->buffer_pool_alloc (stream->video_fifo);
+  buf->type = BUF_CONTROL_FLUSH_DECODER;
+  stream->video_fifo->put (stream->video_fifo, buf);
+#if 0
+  buf = stream->audio_fifo->buffer_pool_alloc (stream->audio_fifo);
+  buf->type = BUF_CONTROL_FLUSH_DECODER;
+  stream->audio_fifo->put (stream->audio_fifo, buf);
+#endif
+  /* reset decoders */
+  buf = stream->video_fifo->buffer_pool_alloc (stream->video_fifo);
+  buf->type = BUF_CONTROL_RESET_DECODER;
+  stream->video_fifo->put (stream->video_fifo, buf);
+
+  buf = stream->audio_fifo->buffer_pool_alloc (stream->audio_fifo);
+  buf->type = BUF_CONTROL_RESET_DECODER;
+  stream->audio_fifo->put (stream->audio_fifo, buf);
+
+  /* reset demuxer */
+  this->videoPid    = INVALID_PID;
+  this->audio_tracks_count = 0;
+  this->media_num   = 0;
+  this->send_newpts = 1;
+  this->spu_pid     = INVALID_PID;
+  this->spu_media   = 0;
+  this->spu_langs_count= 0;
+  this->last_pmt_crc = 0;
+
+  /* wait until processed */
+  _x_demux_control_headers_done (stream);
+
+  /* start */
+  _x_demux_control_start (stream);
+}
+
 static int demux_ts_get_optional_data(demux_plugin_t *this_gen,
 				      void *data, int data_type)
 {
   demux_ts_t *this = (demux_ts_t *) this_gen;
   char *str = data;
-  int channel = *((int *)data);
 
   /* be a bit paranoid */
-  if (this == NULL || this->stream == NULL)
+  if (this == NULL || this->stream == NULL || data == NULL)
     return DEMUX_OPTIONAL_UNSUPPORTED;
 
+  int channel = *((int *)data);
+
   switch (data_type)
     {
     case DEMUX_OPTIONAL_DATA_AUDIOLANG:
@@ -2376,22 +2458,32 @@ static int demux_ts_get_optional_data(demux_plugin_t *this_gen,
       }
       return DEMUX_OPTIONAL_SUCCESS;
 
+    case DEMUX_OPTIONAL_DATA_FLUSH:
+      // BluRay: stream reset
+      flush_decoders(this);
+      return DEMUX_OPTIONAL_SUCCESS;
+
     default:
       return DEMUX_OPTIONAL_UNSUPPORTED;
     }
 }
 
-static int detect_ts(uint8_t *buf, size_t len, int ts_size)
+static int detect_ts(uint8_t *buf, size_t len, unsigned int ts_size)
 {
-  int    i, j;
-  int    try_again, ts_detected = 0;
-  size_t packs = len / ts_size - 2;
+  unsigned int i, try_again;
+  size_t       j, packs;
+
+  if (len < ts_size * 4) {
+    return 0;
+  }
+
+  packs = len / ts_size - 2;
 
   for (i = 0; i < ts_size; i++) {
     try_again = 0;
     if (buf[i] == SYNC_BYTE) {
       for (j = 1; j < packs; j++) {
-	if (buf[i + j*ts_size] != SYNC_BYTE) {
+        if (buf[i + j*ts_size] != SYNC_BYTE) {
 	  try_again = 1;
 	  break;
 	}
@@ -2400,12 +2492,12 @@ static int detect_ts(uint8_t *buf, size_t len, int ts_size)
 #ifdef TS_LOG
 	printf ("demux_ts: found 0x47 pattern at offset %d\n", i);
 #endif
-	ts_detected = 1;
+	return 1;
       }
     }
   }
 
-  return ts_detected;
+  return 0;
 }
 
 static demux_plugin_t *open_plugin (demux_class_t *class_gen,
@@ -2415,25 +2507,32 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen,
   demux_ts_t *this;
   int         i;
   int         hdmv = -1;
+  int         size;
 
   switch (stream->content_detection_method) {
 
   case METHOD_BY_CONTENT: {
     uint8_t buf[2069];
 
-    if (!_x_demux_read_header(input, buf, sizeof(buf)))
+    size = _x_demux_read_header(input, buf, sizeof(buf));
+    if (size < 4 * PKT_SIZE)
       return NULL;
 
-    if (detect_ts(buf, sizeof(buf), PKT_SIZE))
+    if (detect_ts(buf, size, PKT_SIZE))
       hdmv = 0;
-    else if (detect_ts(buf, sizeof(buf), PKT_SIZE+4))
+    else if (detect_ts(buf, size, PKT_SIZE+4))
       hdmv = 1;
     else
       return NULL;
   }
     break;
 
-  case METHOD_BY_EXTENSION: {
+#if DEMUXER_PLUGIN_IFACE_VERSION < 27
+  case METHOD_BY_EXTENSION:
+#else
+  case METHOD_BY_MRL:
+#endif
+  {
     const char *const mrl = input->get_mrl (input);
 
     if (_x_demux_check_extension (mrl, "m2ts mts"))
@@ -2442,8 +2541,11 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen,
       hdmv = 0;
 
     /* check extension */
+#if DEMUXER_PLUGIN_IFACE_VERSION < 27
     const char *const extensions = class_gen->get_extensions (class_gen);
-
+#else
+    const char *const extensions = class_gen->extensions;
+#endif
     if (_x_demux_check_extension (mrl, extensions))
       break;
 
@@ -2464,7 +2566,8 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen,
       break;
 
     /* accept BluRay discs */
-    if (!strncasecmp (mrl, "bluray:/", 8)) {
+    if (!strncasecmp (mrl, "bluray:/", 8) ||
+        !strncasecmp (mrl, "bd:/", 4)) {
       hdmv = 1;
       break;
     }
@@ -2486,7 +2589,6 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen,
   this            = calloc(1, sizeof(*this));
   this->stream    = stream;
   this->input     = input;
-  this->blockSize = PKT_SIZE;
 
   this->demux_plugin.send_headers      = demux_ts_send_headers;
   this->demux_plugin.send_chunk        = demux_ts_send_chunk;
@@ -2514,14 +2616,16 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen,
     this->pmt_write_ptr[i]           = NULL;
   }
 
-  this->programNumber = INVALID_PROGRAM;
-  this->pcrPid = INVALID_PID;
   this->scrambled_npids = 0;
   this->videoPid = INVALID_PID;
   this->audio_tracks_count = 0;
   this->last_pmt_crc = 0;
 
-  this->rate = 16000; /* FIXME */
+  this->rate = xine_get_stream_info(this->stream, XINE_STREAM_INFO_BITRATE);
+  if (this->rate > 0 && this->rate < 100000000 )
+    this->rate /= 8; /* bits/s -> bytes/s */
+  else
+    this->rate = 800000; /* FIXME */
 
   this->status = DEMUX_FINISHED;
 
@@ -2547,21 +2651,29 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen,
  * ts demuxer class
  */
 
+#if DEMUXER_PLUGIN_IFACE_VERSION < 27
 static const char *get_description (demux_class_t *this_gen) {
-  return "MPEG Transport Stream demuxer";
+  return "MPEG Transport Stream demuxer (HDMV)";
 }
+#endif
 
+#if DEMUXER_PLUGIN_IFACE_VERSION < 27
 static const char *get_identifier (demux_class_t *this_gen) {
   return "MPEG_TS_HDMV";
 }
+#endif
 
+#if DEMUXER_PLUGIN_IFACE_VERSION < 27
 static const char *get_extensions (demux_class_t *this_gen) {
   return "m2ts mts";
 }
+#endif
 
+#if DEMUXER_PLUGIN_IFACE_VERSION < 27
 static const char *get_mimetypes (demux_class_t *this_gen) {
   return NULL;
 }
+#endif
 
 static void class_dispose (demux_class_t *this_gen) {
 
@@ -2579,10 +2691,17 @@ static void *init_class (xine_t *xine, void *data) {
   this->xine   = xine;
 
   this->demux_class.open_plugin     = open_plugin;
+#if DEMUXER_PLUGIN_IFACE_VERSION < 27
   this->demux_class.get_description = get_description;
   this->demux_class.get_identifier  = get_identifier;
   this->demux_class.get_mimetypes   = get_mimetypes;
   this->demux_class.get_extensions  = get_extensions;
+#else
+  this->demux_class.description     = "MPEG Transport Stream demuxer (HDMV)";
+  this->demux_class.identifier      = "MPEG_TS_HDMV";
+  this->demux_class.mimetypes       = NULL;
+  this->demux_class.extensions      = "m2ts mts";
+#endif
   this->demux_class.dispose         = class_dispose;
 
   return this;
@@ -2598,7 +2717,11 @@ static const demuxer_info_t demux_info_ts = {
 
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
+#if DEMUX_PLUGIN_IFACE_VERSION <= 26
   { PLUGIN_DEMUX, 26, "mpeg-ts-hdmv", XINE_VERSION_CODE, &demux_info_ts, init_class },
+#elif DEMUX_PLUGIN_IFACE_VERSION >= 27
+  { PLUGIN_DEMUX, 27, "mpeg-ts-hdmv", XINE_VERSION_CODE, &demux_info_ts, init_class },
+#endif
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
 
