@@ -63,8 +63,7 @@ static const char log_module_demux_xvdr[] = "[demux_vdr] ";
  * constants
  */
 
-#define DISC_TRESHOLD       90000
-#define WRAP_THRESHOLD     120000
+#define WRAP_THRESHOLD     270000  /* 3 sec */
 #define PTS_AUDIO 0
 #define PTS_VIDEO 1
 
@@ -115,29 +114,62 @@ typedef struct {
 } demux_xvdr_class_t;
 
 
-static const char * get_decoder_name(xine_t *xine, int video_type)
+static const char * get_decoder_name(xine_t *xine, int buf_type)
 {
-  int streamtype = (video_type >> 16) & 0xFF;
-  plugin_node_t *node = xine->plugin_catalog->video_decoder_map[streamtype][0];
+  uint32_t base = buf_type & 0xFF000000;
+  uint32_t type = (buf_type >> 16) & 0xFF;
+  plugin_node_t *node = NULL;
+
+  if (base == BUF_VIDEO_BASE)
+    node = xine->plugin_catalog->video_decoder_map[type][0];
+  else if (base == BUF_AUDIO_BASE)
+    node = xine->plugin_catalog->audio_decoder_map[type][0];
+
   if (node) {
     plugin_info_t *info = node->info;
     if (info)
       return info->id;
   }
-  return "";
+
+  return "<none>";
 }
 
+#define LOG_DECODER(buf_type, name)                              \
+  LOGDBG("Using %-10s decoder \"%s\"",                           \
+         name, get_decoder_name(this->stream->xine, buf_type));
+
 static void detect_video_decoders(demux_xvdr_t *this)
 {
-  if (!strcmp(get_decoder_name(this->stream->xine, BUF_VIDEO_MPEG), "ffmpegvideo"))
-    this->ffmpeg_mpeg2_decoder = 1;
-  LOGMSG("Using decoder \"%s\" for mpeg2 video",
-         this->ffmpeg_mpeg2_decoder ? "FFmpeg" : "libmpeg2");
+  const char *name;
 
-  if (!strcmp(get_decoder_name(this->stream->xine, BUF_VIDEO_H264), "dshowserver"))
+  name = get_decoder_name(this->stream->xine, BUF_VIDEO_MPEG);
+  if (!strcmp(name, "ffmpegvideo"))
+    this->ffmpeg_mpeg2_decoder = 1;
+  LOGDBG("Using MPEG video decoder \"%s\"%s",
+         name, this->ffmpeg_mpeg2_decoder ? " (FFmpeg)" : "");
+
+  name = get_decoder_name(this->stream->xine, BUF_VIDEO_H264);
+  if (!strcmp(name, "dshowserver"))
     this->coreavc_h264_decoder = 1;
-  LOGMSG("Using decoder \"%s\" for H.264 video",
-         this->coreavc_h264_decoder ? "dshowserver (CoreAVC)" : "FFmpeg");
+  LOGDBG("Using H.264      decoder \"%s\"%s",
+         name, this->coreavc_h264_decoder ? " (dshowserver (CoreAVC))" : "");
+
+  LOG_DECODER(BUF_VIDEO_VC1,     "VC-1");
+}
+
+static void detect_audio_decoders(demux_xvdr_t *this)
+{
+  LOG_DECODER(BUF_AUDIO_MPEG,    "MPEG audio");
+  LOG_DECODER(BUF_AUDIO_A52,     "AC-3");
+  LOG_DECODER(BUF_AUDIO_AAC,     "AAC");
+  LOG_DECODER(BUF_AUDIO_DTS,     "DTS");
+  LOG_DECODER(BUF_AUDIO_LPCM_BE, "LPCM");
+#ifdef BUF_AUDIO_AAC_EAC3
+  LOG_DECODER(BUF_AUDIO_AAC_EAC3, "E-AC-3");
+#endif
+#ifdef BUF_AUDIO_AAC_LATM
+  LOG_DECODER(BUF_AUDIO_AAC_LATM, "AAC LATM");
+#endif
 }
 
 static void demux_xvdr_parse_ts(demux_xvdr_t *this, buf_element_t *buf);
@@ -216,11 +248,10 @@ static void check_newpts(demux_xvdr_t *this, buf_element_t *buf, int video )
         _x_demux_control_newpts(this->stream, buf->pts, 0);
       }
       this->send_newpts = 0;
-
-      this->last_pts[1-video] = 0;
     }
 
     this->last_pts[video] = buf->pts;
+    this->last_pts[1-video] = buf->pts;
   }
 }
 
@@ -348,10 +379,23 @@ static void track_audio_stream_change(demux_xvdr_t *this, buf_element_t *buf)
 #else
   if (this->audio_type != buf->type) {
     LOGDBG("audio stream changed: %08x -> %08x", this->audio_type, buf->type);
-    this->audio_type = buf->type;
+
+    ts_data_reset_audio(this->ts_data, this->audio_fifo, buf->type & 0xff);
+
     put_control_buf(this->audio_fifo,
                     this->audio_fifo,
                     BUF_CONTROL_RESET_TRACK_MAP);
+
+    if (this->audio_type) {
+      buf_element_t *b = this->audio_fifo->buffer_pool_try_alloc(this->audio_fifo);
+      if (b) {
+        b->type = BUF_CONTROL_START;
+        b->decoder_flags = BUF_FLAG_GAPLESS_SW;
+        this->audio_fifo->put(this->audio_fifo, b);
+      }
+    }
+
+    this->audio_type = buf->type;
   }
 #endif
 }
@@ -875,55 +919,15 @@ static int32_t parse_private_stream_1(demux_xvdr_t *this, uint8_t *p, buf_elemen
 
   } else if ((p[0]&0xf0) == 0xa0) {
 
-    int pcm_offset;
-    int number_of_frame_headers;
-    int first_access_unit_pointer;
-    int audio_frame_number;
-    int bits_per_sample;
-    int sample_rate;
-    int num_channels;
-    int dynamic_range;
-
-    /*
-     * found in http://members.freemail.absa.co.za/ginggs/dvd/mpeg2_lpcm.txt
-     * appears to be correct.
-     */
-
     track = p[0] & 0x0F;
-    number_of_frame_headers = p[1];
-    /* unknown = p[2]; */
-    first_access_unit_pointer = p[3];
-    audio_frame_number = p[4];
-
-    /*
-     * 000 => mono
-     * 001 => stereo
-     * 010 => 3 channel
-     * ...
-     * 111 => 8 channel
-     */
-    num_channels = (p[5] & 0x7) + 1;
-    sample_rate = p[5] & 0x10 ? 96000 : 48000;
-    switch ((p[5]>>6) & 3) {
-    case 3: /* illegal, use 16-bits? */
-      default:
-        xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
-                 "illegal lpcm sample format (%d), assume 16-bit samples\n", (p[5]>>6) & 3 );
-      case 0: bits_per_sample = 16; break;
-      case 1: bits_per_sample = 20; break;
-      case 2: bits_per_sample = 24; break;
-    }
-    dynamic_range = p[6];
 
     /* send lpcm config byte */
     buf->decoder_flags |= BUF_FLAG_SPECIAL;
     buf->decoder_info[1] = BUF_SPECIAL_LPCM_CONFIG;
     buf->decoder_info[2] = p[5];
 
-    pcm_offset = 7;
-
-    buf->content   = p+pcm_offset;
-    buf->size      = this->packet_len-pcm_offset;
+    buf->content   = p + 7;
+    buf->size      = this->packet_len - 7;
     buf->type      = BUF_AUDIO_LPCM_BE + track;
     buf->pts       = this->pts;
 
@@ -989,7 +993,7 @@ static int32_t parse_video_stream(demux_xvdr_t *this, uint8_t *p, buf_element_t
 
   p += result;
 
-  if (this->video_type == 0 && buf->size >= 4) {
+  if (this->video_type == 0 && buf->size >= result + 4) {
     this->video_type = detect_h264(p);
   }
 
@@ -1210,6 +1214,7 @@ static demux_plugin_t *demux_xvdr_open_plugin (demux_class_t *class_gen,
   this->status     = DEMUX_FINISHED;
 
   detect_video_decoders(this);
+  detect_audio_decoders(this);
 
   return &this->demux_plugin;
 }
