@@ -4,7 +4,7 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: xvdr_metronom.c,v 1.12 2011/02/13 14:21:55 phintuka Exp $
+ * $Id: xvdr_metronom.c,v 1.16 2012/01/16 12:28:50 phintuka Exp $
  *
  */
 
@@ -32,7 +32,7 @@ static void got_video_frame(metronom_t *metronom, vo_frame_t *frame)
     this->frame_decoded(this->handle, this->video_frames, this->audio_frames);
 
   if (this->still_mode) {
-    LOGMSG("Still frame, type %d", frame->picture_coding_type);
+    LOGVERBOSE("Still frame, type %d", frame->picture_coding_type);
     frame->pts       = 0;
   }
 
@@ -91,8 +91,11 @@ static void set_option(metronom_t *metronom, int option, int64_t value)
   xvdr_metronom_t *this = (xvdr_metronom_t *)metronom;
 
   if (option == XVDR_METRONOM_LAST_VO_PTS) {
-    if (value > 0)
+    if (value > 0) {
+      pthread_mutex_lock(&this->pts_mutex);
       this->last_vo_pts = value;
+      pthread_mutex_unlock(&this->pts_mutex);
+    }
     return;
   }
 
@@ -104,7 +107,11 @@ static int64_t get_option(metronom_t *metronom, int option)
   xvdr_metronom_t *this = (xvdr_metronom_t *)metronom;
 
   if (option == XVDR_METRONOM_LAST_VO_PTS) {
-    return this->last_vo_pts;
+    int64_t pts;
+    pthread_mutex_lock(&this->pts_mutex);
+    pts = this->last_vo_pts;
+    pthread_mutex_unlock(&this->pts_mutex);
+    return pts;
   }
   if (option == XVDR_METRONOM_TRICK_SPEED) {
     return this->trickspeed;
@@ -204,6 +211,8 @@ static void xvdr_metronom_dispose(xvdr_metronom_t *this)
 {
   xvdr_metronom_unwire(this);
 
+  pthread_mutex_destroy(&this->pts_mutex);
+
   free(this);
 }
 
@@ -213,6 +222,11 @@ static void xvdr_metronom_dispose(xvdr_metronom_t *this)
 
 xvdr_metronom_t *xvdr_metronom_init(xine_stream_t *stream)
 {
+  if (stream->metronom->get_option(stream->metronom, XVDR_METRONOM_ID) == XVDR_METRONOM_ID) {
+    LOGMSG("xvdr_metronom_init(): stream already hooked !");
+    return (xvdr_metronom_t*)stream->metronom;
+  }
+
   xvdr_metronom_t *this = calloc(1, sizeof(xvdr_metronom_t));
 
   this->stream        = stream;
@@ -238,6 +252,8 @@ xvdr_metronom_t *xvdr_metronom_init(xine_stream_t *stream)
 
   this->metronom.exit = metronom_exit;
 
+  pthread_mutex_init(&this->pts_mutex, NULL);
+
   xvdr_metronom_wire(this);
 
   return this;
