@@ -4,13 +4,20 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: xvdr_metronom.h,v 1.1 2009/05/22 21:02:30 phintuka Exp $
+ * $Id: xvdr_metronom.h,v 1.7 2011/02/13 14:21:55 phintuka Exp $
  *
  */
 
 #ifndef XVDR_METRONOM_H
 #define XVDR_METRONOM_H
 
+
+#define XVDR_METRONOM_LAST_VO_PTS  0x1001
+#define XVDR_METRONOM_TRICK_SPEED  0x1002
+#define XVDR_METRONOM_STILL_MODE   0x1003
+#define XVDR_METRONOM_ID           0x1004
+
+
 typedef struct xvdr_metronom_s xvdr_metronom_t;
 
 struct xvdr_metronom_s {
@@ -24,6 +31,12 @@ struct xvdr_metronom_s {
   void (*reset_frames)(xvdr_metronom_t *);
   void (*dispose)     (xvdr_metronom_t *);
 
+  void (*set_trickspeed)(xvdr_metronom_t *, int);
+  void (*set_still_mode)(xvdr_metronom_t *, int);
+
+  void (*wire)          (xvdr_metronom_t *);
+  void (*unwire)        (xvdr_metronom_t *);
+
   /* accumulated frame data */
   volatile uint video_frames;
   volatile uint audio_frames;
@@ -40,6 +53,10 @@ struct xvdr_metronom_s {
   void *handle;
   void (*frame_decoded)(void *handle, uint video_count, uint audio_count);
 
+  int     trickspeed;    /* current trick speed */
+  int     still_mode;
+  int64_t last_vo_pts;   /* last displayed video frame PTS */
+  int     wired;         /* true if currently wired to stream */
 #endif
 };
 
