@@ -4,45 +4,23 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: xine_fbfe_frontend.c,v 1.20.2.5 2009/10/08 12:13:11 phintuka Exp $
+ * $Id: xine_fbfe_frontend.c,v 1.50 2012/03/19 11:51:22 phintuka Exp $
  *
  */
 
 #include <inttypes.h>
 #include <sys/ioctl.h>
 #include <fcntl.h>
-#include <stdio.h>
 #include <unistd.h>
 
 #if defined(__linux__)
 # include <linux/kd.h>
 #endif
 
-#ifdef boolean
-# define HAVE_BOOLEAN
-#endif
-#include <jpeglib.h>
-#undef boolean
+#define LOG_MODULENAME "[vdr-fbfe]  "
+#include "logdefs.h"
 
-/* framegrab ports */
-#define XINE_ENABLE_EXPERIMENTAL_FEATURES
-
-#include <xine.h>
-#ifndef XINE_ENGINE_INTERNAL
-#  define XINE_ENGINE_INTERNAL
-#  include <xine/xine_internal.h>
-#  undef XINE_ENGINE_INTERNAL
-#else
-#  include <xine/xine_internal.h>
-#endif
-#include <xine/xineutils.h>
-#include <xine/input_plugin.h>
-#include <xine/plugin_catalog.h>
-
-#include "xine_input_vdr.h"
-#include "xine/post.h"
-
-#include "xine_frontend.h"
+#include "xine_frontend_internal.h"
 
 
 /*
@@ -54,46 +32,9 @@ typedef struct fbfe_s {
   /* function pointers / base class */
   union {
     frontend_t fe;  /* generic frontend */
+    fe_t       x;   /* xine frontend */
   };
 
-  void   (*update_display_size_cb)(frontend_t*);
-  void   (*toggle_fullscreen_cb)  (frontend_t*);
-
-  /* from xine_frontend.c */
-  double (*dest_pixel_aspect)   (const frontend_t *,
-                                 double video_pixel_aspect,
-                                 int video_width, int video_height);
-  void   (*frame_output_handler)(void *data,
-                                 int video_width, int video_height,
-                                 double video_pixel_aspect,
-                                 int *dest_x, int *dest_y,
-                                 int *dest_width, int *dest_height,
-                                 double *dest_pixel_aspect,
-                                 int *win_x, int *win_y);
-
-  /* vdr */
-  fe_keypress_f       keypress;
-
-  /* xine stuff */
-  xine_t             *xine;
-  xine_stream_t      *stream;
-  xine_stream_t      *slave_stream;
-  vdr_input_plugin_if_t *input_plugin;
-  xine_video_port_t  *video_port;
-  xine_video_port_t  *video_port_none;
-  xine_audio_port_t  *audio_port;
-  xine_audio_port_t  *audio_port_none;
-  xine_event_queue_t *event_queue;
-
-  post_plugins_t     *postplugins;
-  char               *video_port_name;
-  char               *aspect_controller;
-
-  int                 xine_visual_type;
-  fb_visual_t         vis;
-
-  uint16_t            pes_buffers;
-
   /* stored original handlers */
   int (*fe_xine_init)(frontend_t *this_gen, const char *audio_driver,
                       const char *audio_port,
@@ -102,46 +43,26 @@ typedef struct fbfe_s {
                       const char *static_post_plugins,
                       const char *config_file);
 
-  /* frontend */
-  double      display_ratio;
-  double      video_aspect;
-  uint16_t    xpos, ypos;
-  uint16_t    width, height;
-  uint16_t    video_width, video_height;
-  uint8_t     overscan;
-  uint8_t     terminate_key_pressed;
-  uint8_t     playback_finished;
-  uint8_t     slave_playback_finished;
-  uint8_t     aspect;
-  uint8_t     cropping;
-  uint8_t     scale_video;
-  uint8_t     field_order;
-
-  /* strings */
-  char        *configfile;
-
   /* display */
 /*char   *modeline;*/
   int     fd_tty;
 
+  /* frontend */
   uint8_t fullscreen : 1;
 /*uint8_t vmode_switch : 1;*/
 
-} fbfe_t, fe_t;
+} fbfe_t;
 
-/* Common (non-X11/FB) frontend functions */
-#include "xine_frontend.c"
-
-static void fbfe_update_display_size(frontend_t *this_gen)
+static void fbfe_update_display_size(fe_t *this_gen)
 {
   fbfe_t *this = (fbfe_t*)this_gen;
-  if(this->fullscreen && this->video_port) {
-    this->width  = this->video_port->get_property(this->video_port,
-                                                  VO_PROP_WINDOW_WIDTH);
-    this->height = this->video_port->get_property(this->video_port,
-                                                  VO_PROP_WINDOW_HEIGHT);
+  if(this->fullscreen && this->x.video_port) {
+    this->x.width  = this->x.video_port->get_property(this->x.video_port, 
+						      VO_PROP_WINDOW_WIDTH);
+    this->x.height = this->x.video_port->get_property(this->x.video_port, 
+						      VO_PROP_WINDOW_HEIGHT);
     LOGDBG("Framebuffer size after initialization: %dx%d",
-	   this->width, this->height);
+	   this->x.width, this->x.height);
   }
 }
 
@@ -197,10 +118,10 @@ static void update_DFBARGS(const char *fb_dev)
  */
 static int fbfe_display_open(frontend_t *this_gen,
                              int xpos, int ypos,
-                             int width, int height, int fullscreen, int hud,
+                             int width, int height, int fullscreen, int hud, int opengl,
                              int modeswitch, const char *modeline, int aspect,
-                             fe_keypress_f keyfunc, int no_x_kbd, int gui_hotkeys,
-                             const char *video_port, int scale_video, int field_order,
+                             int no_x_kbd, int gui_hotkeys,
+                             const char *video_port, int scale_video,
                              const char *aspect_controller, int window_id)
 {
   fbfe_t *this = (fbfe_t*)this_gen;
@@ -211,40 +132,38 @@ static int fbfe_display_open(frontend_t *this_gen,
   if(this->fd_tty >= 0)
     this->fe.fe_display_close(this_gen);
 
-  if(keyfunc) {
-    this->keypress = keyfunc;
-    this->keypress("KBD", "");
+  if(this->fe.fe_message_cb) {
+    this->fe.fe_message_cb(this->fe.fe_message_h, "KBD", "");
   }
 
   LOGDBG("fbfe_display_open(width=%d, height=%d, fullscreen=%d, display=%s)",
 	 width, height, fullscreen, video_port);
 
-  this->xpos            = xpos;
-  this->ypos            = ypos;
-  this->width           = width;
-  this->height          = height;
-  this->aspect          = aspect;
-  this->cropping        = 0;
-  this->field_order     = 0/*field_order ? 1 : 0*/;
-  this->scale_video     = scale_video;
-  this->overscan        = 0;
-  this->display_ratio   = 1.0;
-  this->aspect_controller = aspect_controller ? strdup(aspect_controller) : NULL;
+  this->x.xpos          = xpos;
+  this->x.ypos          = ypos;
+  this->x.width         = width;
+  this->x.height        = height;
+  this->x.aspect        = aspect;
+/*this->x.cropping      = 0;*/
+  this->x.scale_video   = scale_video;
+  this->x.overscan      = 0;
+  this->x.display_ratio = 1.0;
+  this->x.aspect_controller = aspect_controller ? strdup(aspect_controller) : NULL;
 
   this->fullscreen      = fullscreen;
 /*this->vmode_switch    = modeswitch;*/
 /*this->modeline        = strdup(modeline ?: "");*/
 
   /* setup xine FB visual */
-  this->xine_visual_type = XINE_VISUAL_TYPE_FB;
-  this->vis.frame_output_cb = fe_frame_output_cb;
-  this->vis.user_data = this;
+  this->x.xine_visual_type = XINE_VISUAL_TYPE_FB;
+  this->x.vis_fb.frame_output_cb = this->x.frame_output_handler;
+  this->x.vis_fb.user_data = this;
 
   /* select framebuffer device ? */
   if(video_port && !strncmp(video_port, "/dev/", 5))
-    this->video_port_name = strdup(video_port);
+    this->x.video_port_name = strdup(video_port);
   else
-    this->video_port_name = NULL;
+    this->x.video_port_name = NULL;
 
   /* set console to graphics mode */
 #if defined(KDSETMODE) && defined(KD_GRAPHICS)
@@ -274,26 +193,26 @@ static int fbfe_display_config(frontend_t *this_gen,
                                int xpos, int ypos,
                                int width, int height, int fullscreen,
                                int modeswitch, const char *modeline,
-                               int aspect, int scale_video, int field_order)
+                               int aspect, int scale_video)
 {
   fbfe_t *this = (fbfe_t*)this_gen;
 
   if(!this)
     return 0;
 
-  this->xpos          = xpos   >= 0 ? xpos   : this->xpos;
-  this->ypos          = ypos   >= 0 ? ypos   : this->ypos;
-  this->width         = width  >= 0 ? width  : this->width;
-  this->height        = height >= 0 ? height : this->height;
-  this->aspect        = aspect;
-  this->scale_video   = scale_video;
-  this->field_order   = field_order;
+  this->x.xpos        = xpos   >= 0 ? xpos   : this->x.xpos;
+  this->x.ypos        = ypos   >= 0 ? ypos   : this->x.ypos;
+  this->x.width       = width  >= 0 ? width  : this->x.width;
+  this->x.height      = height >= 0 ? height : this->x.height;
+  this->x.aspect      = aspect;
+  this->x.scale_video = scale_video;
   this->fullscreen    = fullscreen;
 /*this->vmode_switch  = modeswitch;*/
 #if 0
   if(!modeswitch && strcmp(modeline, this->modeline)) {
-    strn0cpy(this->modeline, modeline, sizeof(this->modeline));
-    /* XXX TODO - switch vmode */
+    free(this->modeline);
+    this->modeline = strdup(modeline ?: "");
+    /* #warning XXX TODO - switch vmode */
   }
 #endif
 
@@ -325,7 +244,7 @@ static void fbfe_display_close(frontend_t *this_gen)
   if (!this)
     return;
 
-  if (this->xine)
+  if (this->x.xine)
     this->fe.xine_exit(this_gen);
 
   if (this->fd_tty >= 0) {
@@ -337,11 +256,15 @@ static void fbfe_display_close(frontend_t *this_gen)
     this->fd_tty = -1;
   }
 
-  free(this->video_port_name);
-  this->video_port_name = NULL;
+  free(this->x.video_port_name);
+  this->x.video_port_name = NULL;
 
-  free(this->aspect_controller);
-  this->aspect_controller = NULL;
+  free(this->x.aspect_controller);
+  this->x.aspect_controller = NULL;
+#if 0
+  free(this->modeline);
+  this->modeline = NULL;
+#endif
 }
 
 static int fbfe_xine_init(frontend_t *this_gen, const char *audio_driver,
@@ -354,7 +277,7 @@ static int fbfe_xine_init(frontend_t *this_gen, const char *audio_driver,
   fbfe_t *this = (fbfe_t*)this_gen;
 
   if (video_driver && !strcmp(video_driver, "DirectFB"))
-    update_DFBARGS(this->video_port_name);
+    update_DFBARGS(this->x.video_port_name);
 
   return this->fe_xine_init(this_gen, audio_driver, audio_port,
                             video_driver, pes_buffers, static_post_plugins, config_file);
@@ -375,7 +298,7 @@ static frontend_t *fbfe_get_frontend(void)
   this->fe.fe_run       = fbfe_run;
   this->fe.fe_interrupt = fbfe_interrupt;
 
-  this->update_display_size_cb = fbfe_update_display_size;
+  this->x.update_display_size_cb = fbfe_update_display_size;
 
   /* override */
   this->fe_xine_init  = this->fe.xine_init;
