@@ -4,44 +4,71 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: xine_frontend.c,v 1.53.2.16 2010/03/31 20:50:01 phintuka Exp $
+ * $Id: xine_frontend.c,v 1.122 2011/12/14 08:12:16 phintuka Exp $
  *
  */
 
+#include "features.h"
+
 #include <inttypes.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
+#include <sys/stat.h>
+#include <sys/types.h>
 
+#ifdef HAVE_LIBJPEG
 # ifdef boolean
 #  define HAVE_BOOLEAN
 # endif
 # include <jpeglib.h>
 # undef boolean
+#endif
 
 #define XINE_ENGINE_INTERNAL
 #include <xine.h>
 #include <xine/xine_internal.h>
 
-#ifndef XINE_VERSION_CODE
-#  define XINE_VERSION_CODE (XINE_MAJOR_VERSION*10000 + \
-                             XINE_MINOR_VERSION*100 + \
-                             XINE_SUB_VERSION)
-#endif
-
 #define LOG_MODULENAME "[vdr-fe]    "
 #include "logdefs.h"
 
-#include "xine_input_vdr.h"
-#include "xine_input_vdr_mrl.h"
+#include "xine_frontend_internal.h"
 #include "xine/post.h"
 
+#include "xine/vo_post.h"
+#include "xine/vo_osdscaler.h"
+#include "xine/vo_osdreorder.h"
+#include "xine/vo_lastpts.h"
+
 #undef  MIN
 #define MIN(a,b) ( (a) < (b) ? (a) : (b))
 #undef  MAX
 #define MAX(a,b) ( (a) > (b) ? (a) : (b))
 
-static inline char *strn0cpy(char *dest, const char *src, int n) 
+
+static void intercept_video_driver(xine_video_port_t *video_port)
+{
+  vo_driver_t *osdscaler = osdscaler_init();
+  if (! wire_video_driver(video_port, osdscaler)) {
+    LOGMSG("wire_video_driver() for osdscaler failed");
+    osdscaler->dispose(osdscaler);
+  }
+
+  vo_driver_t *osdreorder = osd_reorder_init();
+  if (! wire_video_driver(video_port, osdreorder)) {
+    LOGMSG("wire_video_driver() for osdreorder failed");
+    osdreorder->dispose(osdreorder);
+  }
+
+  vo_driver_t *lastpts = vo_lastpts_init();
+  if (! wire_video_driver(video_port, lastpts)) {
+    LOGMSG("wire_video_driver() for vo_lastpts failed");
+    lastpts->dispose(lastpts);
+  }
+}
+
+
+char *strn0cpy(char *dest, const char *src, int n) 
 {
   char *s = dest;
   for ( ; --n && (*dest = *src) != 0; dest++, src++) ;
@@ -74,6 +101,43 @@ static int guess_cpu_count(void)
   return cores;
 }
 
+static void shutdown_system(char *cmd, int user_requested)
+{
+  const char *reason = user_requested ? "User requested system shutdown" : "Inactivity timer elapsed";
+
+  if (cmd) {
+    LOGMSG("%s. Executing '%s'", reason, cmd);
+    if (-1 == system(cmd))
+      LOGERR("Can't execute %s", cmd);
+  } else {
+    LOGMSG("%s, power off comand undefined!", reason);
+  }
+}
+
+static void make_dirs(const char *file)
+{
+  struct stat st;
+  char *s = strdup(file);
+  char *p = s;
+
+  if (*p == '/') {
+    p++;
+    while ((p = strchr(p, '/')) != NULL) {
+      *p = 0;
+      if (stat(s, &st) != 0 || !S_ISDIR(st.st_mode)) {
+        if (mkdir(s, ACCESSPERMS) == -1) {
+          LOGERR("Can't create %s", s);
+          break;
+        }
+        LOGDBG("Created directory %s", s);
+      }
+      *p++ = '/';
+    }
+  }
+
+  free(s);
+}
+
 /*
  * list available plugins
  */
@@ -154,10 +218,9 @@ static void *fe_control(frontend_t *this_gen, const char *cmd);
  * xine callbacks
  */
 
-static double fe_dest_pixel_aspect(const frontend_t *this_gen, double video_pixel_aspect,
+static double fe_dest_pixel_aspect(const fe_t *this, double video_pixel_aspect,
 				   int video_width, int video_height)
 {
-  const fe_t *this = (const fe_t*)this_gen;
   /*int new_cropping = 0;*/
   double result = 1.0;
 
@@ -275,7 +338,7 @@ static void fe_frame_output_cb (void *data,
   *win_x = this->xpos;
   *win_y = this->ypos;  
   
-  *dest_pixel_aspect = this->dest_pixel_aspect((frontend_t*)this, video_pixel_aspect,
+  *dest_pixel_aspect = this->dest_pixel_aspect(this, video_pixel_aspect,
 					       video_width, video_height);
 
 #if 0
@@ -321,10 +384,11 @@ static void fe_frame_output_cb (void *data,
     xine_event_send(this->stream, &event);
     this->video_width = video_width;
     this->video_height = video_height;
-
+#if 0
     /* trigger forced redraw to make cropping changes effective */
     if(this->cropping)
       xine_set_param(this->stream, XINE_PARAM_VO_ZOOM_X, 100);      
+#endif
   }
 
   if(this->aspect_controller) {
@@ -334,7 +398,7 @@ static void fe_frame_output_cb (void *data,
       char cmd[4096];
       if(snprintf(cmd, sizeof(cmd), "%s %d", 
                   this->aspect_controller, (int)(video_aspect * 10000.0)) 
-         < sizeof(cmd)) {
+         < (int)sizeof(cmd)) {
         LOGDBG("Aspect ratio changed, executing %s", cmd);
         if(system(cmd) == -1)
 	  LOGERR("Executing /bin/sh -c %s failed", cmd);
@@ -481,8 +545,10 @@ static void configure_audio_out(const fe_t *this, const char *audio_driver, cons
       x_upd_str("audio.device.alsa_passthrough_device",	audio_port);
       x_upd_num("audio.output.speaker_arrangement",     A52_PASSTHRU);
     } else {
+#if 0
       x_upd_num("audio.output.speaker_arrangement",
 		strstr(audio_port, "surround") ? SURROUND51 : STEREO);
+#endif
     }
   }
 
@@ -562,9 +628,11 @@ static int fe_xine_init(frontend_t *this_gen, const char *audio_driver,
   if (config_file)
     this->configfile = strdup(config_file);
   else if(asprintf(&this->configfile, "%s%s", xine_get_homedir(), 
-                   "/.xine/config_xineliboutput") < 0)
+	      "/.xine/config_xineliboutput") < 0)
     return 0;
 
+  LOGDBG("Using xine-lib config file %s", this->configfile);
+  make_dirs(this->configfile);
   xine_config_load (this->xine, this->configfile);
 
   x_reg_num ("engine.buffers.video_num_buffers",
@@ -621,15 +689,17 @@ static int fe_xine_init(frontend_t *this_gen, const char *audio_driver,
   if(!this->video_port) {
     LOGMSG("fe_xine_init: xine_open_video_driver(\"%s\") failed",
 	   video_driver?video_driver:"(NULL)"); 
-    xine_exit(this->xine);
+    this->fe.xine_exit(this_gen);
     return 0;
   }
 
+  intercept_video_driver(this->video_port);
+
   this->video_port_none = NULL;
   
   /* re-configure display size (DirectFB driver changes display mode in init) */
   if(this->update_display_size_cb)
-    this->update_display_size_cb(this_gen);
+    this->update_display_size_cb(this);
 
   /* create audio port */
 
@@ -662,7 +732,7 @@ static int fe_xine_init(frontend_t *this_gen, const char *audio_driver,
 
   if(!this->stream) {
     LOGMSG("fe_xine_init: xine_stream_new failed"); 
-    xine_exit(this->xine);
+    this->fe.xine_exit(this_gen);
     return 0;
   }
 
@@ -724,6 +794,18 @@ static int fe_xine_init(frontend_t *this_gen, const char *audio_driver,
   return 1;
 }
 
+static void fe_shutdown_init(frontend_t *this_gen, const char *cmd, int timeout)
+{
+  fe_t *this = (fe_t*)this_gen;
+
+  free(this->shutdown_cmd);
+
+  this->shutdown_cmd     = cmd ? strdup(cmd) : NULL;
+  this->shutdown_timeout = timeout;
+
+  this->shutdown_time = (timeout <= 0) ? (time_t)-1 : time(NULL) + timeout;
+}
+
 /*
  * fe_xine_open
  *
@@ -812,6 +894,9 @@ static void init_dummy_ports(fe_t *this, int on)
 
 static void fe_post_unwire(fe_t *this)
 {
+  if (!this || !this->stream)
+    return;
+
   xine_post_out_t  *vo_source = xine_get_video_source(this->stream);
   xine_post_out_t  *ao_source = xine_get_audio_source(this->stream);
 
@@ -861,9 +946,11 @@ static void fe_post_rewire(const fe_t *this)
 
 static void fe_post_unload(const fe_t *this)
 {
-  LOGDBG("unloading post plugins");
-  vpplugin_unload_post(this->postplugins, NULL);
-  applugin_unload_post(this->postplugins, NULL);
+  if (this->postplugins) {
+    LOGDBG("unloading post plugins");
+    vpplugin_unload_post(this->postplugins, NULL);
+    applugin_unload_post(this->postplugins, NULL);
+  }
 }
 
 static void fe_post_close(const fe_t *this, const char *name, int which)
@@ -987,6 +1074,7 @@ static void fe_post_open(const fe_t *this, const char *name, const char *args)
       case 2: /* 4:3   */ r = 4.0/3.0; break;
       case 3: /* 16:9  */ r = 16.0/9.0; break;
       case 4: /* 16:10 */ r = 16.0/10.0; break;
+      default: LOGDBG("%s(%d): unknown aspect: %d", __FUNCTION__, __LINE__, this->aspect);
       }
       /* in finnish locale decimal separator is "," - same as post plugin parameter separator :( */
       sprintf(tmp, "%04d", (int)(r*1000.0));
@@ -1096,6 +1184,9 @@ static int fe_xine_stop(frontend_t *this_gen)
   this->input_plugin      = NULL;
   this->playback_finished = 1;
 
+  if (!this->stream)
+    return 0;
+
   xine_stop(this->stream);
 
   fe_post_unwire(this);
@@ -1116,6 +1207,9 @@ static void fe_xine_close(frontend_t *this_gen)
       this->input_plugin->f.fe_control       = NULL;
     }
 
+    if (!this->stream)
+      return;
+
     fe_xine_stop(this_gen);
 
     fe_post_unload(this);
@@ -1149,9 +1243,15 @@ static void fe_xine_exit(frontend_t *this_gen)
       xine_dispose(this->stream);
     this->stream = NULL;
 
-    if(this->postplugins->pip_stream) 
-      xine_dispose(this->postplugins->pip_stream);
-    this->postplugins->pip_stream = NULL;
+    if (this->postplugins) {
+      if (this->postplugins->pip_stream)
+        xine_dispose(this->postplugins->pip_stream);
+      this->postplugins->pip_stream = NULL;
+
+      free(this->postplugins->static_post_plugins);
+    }
+    free(this->postplugins);
+    this->postplugins = NULL;
 
     if(this->slave_stream) 
       xine_dispose(this->slave_stream);
@@ -1167,11 +1267,6 @@ static void fe_xine_exit(frontend_t *this_gen)
       xine_close_video_driver(this->xine, this->video_port);
     this->video_port = NULL;
 
-    if(this->postplugins->static_post_plugins)
-      free(this->postplugins->static_post_plugins);
-    free(this->postplugins);
-    this->postplugins = NULL;
-
     xine_exit(this->xine);
     this->xine = NULL;
   }
@@ -1183,6 +1278,7 @@ static void fe_free(frontend_t *this_gen)
     fe_t *this = (fe_t*)this_gen;
     this->fe.fe_display_close(this_gen);
     free(this->configfile);
+    free(this->shutdown_cmd);
     free(this);
   }
 }
@@ -1203,6 +1299,14 @@ static int fe_is_finished(frontend_t *this_gen, int slave_stream)
       return FE_XINE_EXIT;
   }
 
+  /* check inactivity timer */
+  if (this->shutdown_timeout > 0) {
+    if (this->shutdown_time < time(NULL)) {
+      shutdown_system(this->shutdown_cmd, 0);
+      return FE_XINE_EXIT;
+    }
+  }
+
   return FE_XINE_RUNNING;
 }
 
@@ -1231,14 +1335,14 @@ static int xine_osd_command(frontend_t *this_gen, struct osd_command_s *cmd) {
   return this->input_plugin->f.push_input_osd(this->input_plugin, cmd);
 }
 
-static int xine_queue_pes_packet(frontend_t *this_gen, const char *data, int len)
+static int xine_queue_pes_packet(frontend_t *this_gen, int stream, uint64_t pos, const char *data, int len)
 {
   fe_t *this = (fe_t*)this_gen;
 
   if(!find_input_plugin(this))
     return 0/*-1*/;
 
-  return this->input_plugin->f.push_input_write(this->input_plugin, data, len);
+  return this->input_plugin->f.push_input_write(this->input_plugin, stream, pos, data, len);
 }
 
 /*
@@ -1254,6 +1358,10 @@ static int fe_send_input_event(frontend_t *this_gen, const char *map,
   LOGDBG("Keypress: %s %s %s %s", 
 	 map, key, repeat?"Repeat":"", release?"Release":"");
 
+  /* reset inactivity timer */
+  if (this->shutdown_timeout > 0)
+    this->shutdown_time = time(NULL) + this->shutdown_timeout;
+
   /* local mode: --> vdr callback */
   if(this->keypress) {
     this->keypress(map, key);
@@ -1299,18 +1407,25 @@ static int fe_send_event(frontend_t *this_gen, const char *data)
 
   if (!strcmp(data, "TOGGLE_FULLSCREEN")) {
     if(this->toggle_fullscreen_cb)
-      this->toggle_fullscreen_cb(this_gen);
+      this->toggle_fullscreen_cb(this, -1);
+
+  } else if (!strncasecmp(data, "FULLSCREEN ", 11)) {
+    if(this->toggle_fullscreen_cb)
+      this->toggle_fullscreen_cb(this, atoi(data+11) ? 1: 0);
 
   } else if (!strcmp(data, "QUIT")) {
     this->terminate_key_pressed = 1;
 
   } else if(!strcmp(data, "TOGGLE_DEINTERLACE")) {
-    xine_set_param(this->stream, XINE_PARAM_VO_DEINTERLACE, 
+    xine_set_param(this->stream, XINE_PARAM_VO_DEINTERLACE,
 		   xine_get_param(this->stream, XINE_PARAM_VO_DEINTERLACE) ? 0 : 1);
 
   } else if(!strncasecmp(data, "DEINTERLACE ", 12)) {
     xine_set_param(this->stream, XINE_PARAM_VO_DEINTERLACE, atoi(data+12) ? 1 : 0);
 
+  } else if (!strcmp(data, "POWER_OFF")) {
+    shutdown_system(this->shutdown_cmd, 1);
+
   } else {
 
     LOGDBG("Event: %s", data);
@@ -1421,7 +1536,7 @@ static void *fe_control(frontend_t *this_gen, const char *cmd)
 					    this->video_port);
       LOGMSG("  PIP %d: %dx%d @ (%d,%d)", pid & 0x0f, w, h, x, y);
       LOGMSG("create pip stream done");
-      sprintf(mrl, MRL_ID "+slave://0x%lx#nocache;demux:mpeg_block",
+      sprintf(mrl, MRL_ID "+slave://0x%lx#nocache",
 	      (unsigned long int)this);
       if(!xine_open(posts->pip_stream, mrl) ||
 	 !xine_play(posts->pip_stream, 0, 0)) {
@@ -1511,6 +1626,7 @@ static void *fe_control(frontend_t *this_gen, const char *cmd)
   return NULL;
 }
 
+
 /*
  * --- RgbToJpeg -------------------------------------------------------------
  *
@@ -1520,6 +1636,8 @@ static void *fe_control(frontend_t *this_gen, const char *cmd)
  *        - move to xine_input_vdr ?
  */
 
+#ifdef HAVE_LIBJPEG
+
 #define JPEGCOMPRESSMEM 500000
 
 typedef struct tJpegCompressData_s {
@@ -1565,6 +1683,8 @@ static void JpegCompressTermDestination(const j_compress_ptr cinfo)
      }
 }
 
+#ifndef HAVE_XINE_GRAB_VIDEO_FRAME
+
 static vo_frame_t *yuy2_to_yv12_frame(xine_stream_t *stream, vo_frame_t *frame)
 {
   /* convert yuy12 frames to yv12 */
@@ -1695,7 +1815,10 @@ static char *frame_compress_jpeg(fe_t *this, int *size, int quality, vo_frame_t
   *size = jcd.size;
   return (char*) jcd.mem;
 }
+#endif /* HAVE_XINE_GRAB_VIDEO_FRAME */
+#endif /* HAVE_LIBJPEG */
 
+#ifndef HAVE_XINE_GRAB_VIDEO_FRAME
 static vo_frame_t *yv12_to_yuy2_frame(xine_stream_t *stream, vo_frame_t *frame)
 {
   /* convert yv12 frames to yuy2 */
@@ -1778,15 +1901,12 @@ static char *frame_compress_pnm(fe_t *this, int *size, vo_frame_t *frame)
 
   return (char*)pnm;
 }
+#endif /* HAVE_XINE_GRAB_VIDEO_FRAME */
 
-#ifdef  XINE_GUI_SEND_GRAB_FRAME
-static char *fe_grab_raw_frame(fe_t *this, int *size, int jpeg, int quality, int width, int height, xine_grab_frame_t *frame)
+#ifdef HAVE_XINE_GRAB_VIDEO_FRAME
+static char *fe_compress_grab_frame(fe_t *this, int *size, int jpeg, int quality, int width, int height, xine_grab_video_frame_t *frame)
 {
-  frame->width = width;
-  frame->height = height;
-  if (xine_port_send_gui_data(this->stream->video_out, XINE_GUI_SEND_GRAB_FRAME, frame))
-    return NULL;
-
+#ifdef HAVE_LIBJPEG
   if (jpeg) {
     /* Compress JPEG */
     struct jpeg_destination_mgr jdm;
@@ -1823,6 +1943,7 @@ static char *fe_grab_raw_frame(fe_t *this, int *size, int jpeg, int quality, int
     *size = jcd.size;
     return (char*) jcd.mem;
   }
+#endif
 
   /* convert to PNM */
 
@@ -1844,13 +1965,20 @@ static char *fe_grab_raw_frame(fe_t *this, int *size, int jpeg, int quality, int
   *size = bytes + hdrlen;
   return (char*)pnm;
 }
-#endif
+#endif /* HAVE_XINE_GRAB_VIDEO_FRAME */
+
 
 static char *fe_grab(frontend_t *this_gen, int *size, int jpeg, 
 		     int quality, int width, int height)
 {
   fe_t *this = (fe_t*)this_gen;
-  vo_frame_t *frame;
+
+#ifndef HAVE_LIBJPEG
+  if(jpeg) {
+    LOGMSG("fe_grab: JPEG grab support was not compiled in");
+    return 0;
+  }
+#endif /* HAVE_LIBJPEG */
 
   if(!find_input_plugin(this))
     return 0;
@@ -1866,18 +1994,25 @@ static char *fe_grab(frontend_t *this_gen, int *size, int jpeg,
 
   /* get last frame */
   this->stream->xine->port_ticket->acquire(this->stream->xine->port_ticket, 0);
-#ifdef XINE_GUI_SEND_GRAB_FRAME
-  xine_grab_frame_t *grab_frame;
-  if (!xine_port_send_gui_data(this->stream->video_out, XINE_GUI_SEND_ALLOC_GRAB_FRAME, &grab_frame)) {
-    char *img = fe_grab_raw_frame(this, size, jpeg, quality, width, height, grab_frame);
-    xine_port_send_gui_data(this->stream->video_out, XINE_GUI_SEND_FREE_GRAB_FRAME, grab_frame);
-    this->stream->xine->port_ticket->release(this->stream->xine->port_ticket, 0);
-    return img;
+
+#ifdef HAVE_XINE_GRAB_VIDEO_FRAME
+  char *img = NULL;
+  xine_grab_video_frame_t *grab_frame = xine_new_grab_video_frame(this->stream);
+  if (grab_frame) {
+    grab_frame->width = width;
+    grab_frame->height = height;
+    if (!grab_frame->grab(grab_frame))
+      img = fe_compress_grab_frame(this, size, jpeg, quality, width, height, grab_frame);
+    grab_frame->dispose(grab_frame);
   }
-#endif
-  frame = this->stream->video_out->get_last_frame (this->stream->video_out);
+  this->stream->xine->port_ticket->release(this->stream->xine->port_ticket, 0);
+  return img;
+#else
+  vo_frame_t *frame = this->stream->video_out->get_last_frame (this->stream->video_out);
+#if XINE_VERSION_CODE < 10190
   if(frame)
     frame->lock(frame);
+#endif
   this->stream->xine->port_ticket->release(this->stream->xine->port_ticket, 0);
 
   if(!frame) {
@@ -1894,7 +2029,12 @@ static char *fe_grab(frontend_t *this_gen, int *size, int jpeg,
   if (!jpeg)
     return frame_compress_pnm(this, size, frame);
 
+#ifdef HAVE_LIBJPEG
   return frame_compress_jpeg(this, size, quality, frame);
+#else /* HAVE_LIBJPEG */
+  return NULL;
+#endif
+#endif /* HAVE_XINE_GRAB_VIDEO_FRAME */
 }
 
 /*
@@ -1911,6 +2051,8 @@ void init_fe(fe_t *fe)
   fe->fe.xine_close       = fe_xine_close;
   fe->fe.xine_exit        = fe_xine_exit;
 
+  fe->fe.shutdown_init    = fe_shutdown_init;
+
   fe->fe.fe_free          = fe_free;
 
   fe->fe.xine_is_finished = fe_is_finished;
@@ -1928,10 +2070,3 @@ void init_fe(fe_t *fe)
   fe->frame_output_handler = fe_frame_output_cb;
 }
 
-#ifdef FE_STANDALONE
-
-/* frontend main() */
-
-#include "xine_frontend_main.c"
-
-#endif /* #ifdef FE_STANDALONE */
