@@ -4,7 +4,7 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: xine_frontend.h,v 1.25 2011/02/25 01:41:23 phintuka Exp $
+ * $Id: xine_frontend.h,v 1.29 2012/03/19 11:53:04 phintuka Exp $
  *
  */
 
@@ -17,8 +17,6 @@ extern "C" {
 
 #define FE_VERSION_STR  XINELIBOUTPUT_VERSION /*"1.0.0pre1"*/
 
-typedef void (*fe_keypress_f)(const char *keymap, const char *name);
-
 typedef struct frontend_config_s frontend_config_t;
 typedef struct frontend_s frontend_t;
 
@@ -37,7 +35,6 @@ struct frontend_config_s {
   char *video_port;
 
   int scale_video;
-  int field_order;
 
   fe_keypress_f keypresshandler;
 
@@ -60,14 +57,19 @@ struct frontend_config_s {
 #define FE_ERROR  0
 
 /* HUD OSD bitmask values */
-#define HUD_COMPOSITE 0x1
-#define HUD_XSHAPE    0x2
-#define HUD_OPENGL    0x4
+#define HUD_COMPOSITE 0x1  /* hud in transparent window */
+#define HUD_XSHAPE    0x2  /* use XShape */
+#define HUD_OPENGL    0x4  /* draw OSD and video using OpenGL */
+#define HUD_XRENDER   0x8  /* draw OSD and video using Xrender */
 
 /* Special window_id's */
 #define WINDOW_ID_NONE  -1
 #define WINDOW_ID_ROOT  -2
 
+/* Video mode switching flags */
+#define MODESWITCH_SIZE       0x1
+#define MODESWITCH_RATE       0x2
+#define MODESWITCH_INTERLACE  0x4
 
 struct frontend_s {
   /* Display */
@@ -76,17 +78,17 @@ struct frontend_s {
                          int winwidth, int winheight,
                          int fullscreen, int hud, int opengl,
                          int modeswitch, const char *modeline,
-                         int aspect, fe_keypress_f keypresshandler,
+                         int aspect,
                          int no_x_kbd, int gui_hotkeys,
                          const char *video_port,
-                         int scale_video, int field_order,
+                         int scale_video,
                          const char *aspect_controller, int window_id);
   int  (*fe_display_config)(frontend_t *,
                             int xpos, int ypos,
                             int width, int height,
                             int fullscreen,
                             int modeswitch, const char *modeline,
-                            int aspect, int scale_video, int field_order);
+                            int aspect, int scale_video);
   void (*fe_display_close)(frontend_t*);
 
   /* Xine engine */
@@ -120,11 +122,18 @@ struct frontend_s {
   char *(*grab)(frontend_t*, int *size, int jpeg, int quality,
                 int width, int height);
 
-  /* events from frontend -> xine/vdr */
+  /* frontend services: events from frontend -> xine/vdr */
   int  (*send_event)(frontend_t *fe, const char *data);
   int  (*send_input_event)(frontend_t *fe,
                            const char *map, const char *key,
                            int repeat, int release);
+
+  /* Control messages frontend -> VDR (local mode)
+   * frontend should fill following pointers
+   */
+  void               *fe_message_h;
+  void              (*fe_message_cb)(void *, const char *keymap, const char *name);
+
 #if 0
   frontend_config_t config;
 #endif
