@@ -4,18 +4,16 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: xine_frontend_main.c,v 1.40.2.17 2009/08/21 11:50:41 phintuka Exp $
+ * $Id: xine_frontend_main.c,v 1.96 2011/03/19 17:33:21 phintuka Exp $
  *
  */
 
+#include "features.h"
+
 #include <stdio.h>
 #include <inttypes.h>
 #include <stdlib.h>
 #include <string.h>
-#include <poll.h>
-#include <errno.h>
-#include <termios.h>
-#include <pthread.h>
 #include <unistd.h>
 #include <syslog.h>
 #include <getopt.h>
@@ -23,12 +21,14 @@
 
 #include <xine.h>  /* xine_get_version */
 
+#define LOG_MODULENAME "[vdr-fe]    "
 #include "logdefs.h"
 
 #include "xine_input_vdr_mrl.h"
 #include "xine_frontend.h"
 #include "tools/vdrdiscovery.h"
 #include "xine_frontend_lirc.h"
+#include "xine_frontend_kbd.h"
 
 /* static data */
 
@@ -38,288 +38,6 @@ int SysLogLevel __attribute__((visibility("default"))) = SYSLOGLEVEL_INFO; /* er
 volatile int   last_signal = 0;
 int            gui_hotkeys = 0;
 
-/*
- * stdin (keyboard/slave mode) reading
- */
-
-/* static data */
-pthread_t      kbd_thread;
-struct termios tm, saved_tm;
-
-/* include LIRC forwarding code */
-#include "xine_frontend_lirc.c"
-
-/*
- * read_key()
- *
- * Try to read single char from stdin.
- *
- * Returns: >=0  char readed
- *           -1  nothing to read
- *           -2  fatal error
- */
-#define READ_KEY_ERROR   -2
-#define READ_KEY_EAGAIN  -1
-static int read_key(void)
-{
-  unsigned char ch;
-  int err;
-  struct pollfd pfd;
-  pfd.fd = STDIN_FILENO;
-  pfd.events = POLLIN;
-
-  errno = 0;
-  pthread_testcancel();
-
-  if (1 == (err = poll(&pfd, 1, 50))) {
-    pthread_testcancel();
-
-    if (1 == (err = read(STDIN_FILENO, &ch, 1)))
-      return (int)ch;
-
-    if (err < 0)
-      LOGERR("read_key: read(stdin) failed");
-    else
-      LOGERR("read_key: read(stdin) failed: no stdin");
-    return READ_KEY_ERROR;
-
-  } else if (err < 0 && errno != EINTR) {
-    LOGERR("read_key: poll(stdin) failed");
-    return READ_KEY_ERROR;
-  }
-
-  pthread_testcancel();
-  return READ_KEY_EAGAIN;
-}
-
-/*
- * read_key_seq()
- *
- * Read a key sequence from stdin.
- * Key sequence is either normal key or escape sequence.
- *
- * Returns the key or escape sequence as uint64_t.
- */
-#define READ_KEY_SEQ_ERROR 0xffffffff
-static uint64_t read_key_seq(void)
-{
-  /* from vdr, remote.c */
-  uint64_t k = 0;
-  int key1;
-
-  if ((key1 = read_key()) >= 0) {
-    k = key1;
-    if (key1 == 0x1B) {
-      /* Start of escape sequence */
-      if ((key1 = read_key()) >= 0) {
-        k <<= 8;
-        k |= key1 & 0xFF;
-        switch (key1) {
-          case 0x4F: /* 3-byte sequence */
-            if ((key1 = read_key()) >= 0) {
-              k <<= 8;
-              k |= key1 & 0xFF;
-            }
-            break;
-          case 0x5B: /* 3- or more-byte sequence */
-            if ((key1 = read_key()) >= 0) {
-              k <<= 8;
-              k |= key1 & 0xFF;
-              switch (key1) {
-                case 0x31 ... 0x3F: /* more-byte sequence */
-                case 0x5B: /* strange, may apparently occur */
-                  do {
-                    if ((key1 = read_key()) < 0)
-                      break; /* Sequence ends here */
-                    k <<= 8;
-                    k |= key1 & 0xFF;
-                  } while (key1 != 0x7E);
-                  break;
-              }
-            }
-            break;
-        }
-      }
-    }
-  }
-
-  if (key1 == READ_KEY_ERROR)
-    return READ_KEY_SEQ_ERROR;
-
-  return k;
-}
-
-/*
- * kbd_receiver_thread()
- *
- * Read key(sequence)s from stdin and pass those to frontend.
- */
-
-static void kbd_receiver_thread_cleanup(void *arg)
-{
-  int status;
-  tcsetattr(STDIN_FILENO, TCSANOW, &saved_tm);
-  status = system("setterm -cursor on");
-  LOGMSG("Keyboard thread terminated");
-}
-
-static void *kbd_receiver_thread(void *fe_gen)
-{
-  frontend_t *fe = (frontend_t*)fe_gen;
-  uint64_t code = 0;
-  char str[64];
-  int status;
-
-  status = system("setterm -cursor off");
-  status = system("setterm -blank off");
-
-  /* Set stdin to deliver keypresses without buffering whole lines */
-  tcgetattr(STDIN_FILENO, &saved_tm);
-  if (tcgetattr(STDIN_FILENO, &tm) == 0) {
-    tm.c_iflag = 0;
-    tm.c_lflag &= ~(ICANON | ECHO);
-    tm.c_cc[VMIN] = 0;
-    tm.c_cc[VTIME] = 0;
-    tcsetattr(STDIN_FILENO, TCSANOW, &tm);
-  }
-
-  pthread_cleanup_push(kbd_receiver_thread_cleanup, NULL);
-
-  do {
-    alarm(0);
-    errno = 0;
-    code = read_key_seq();
-    alarm(3); /* watchdog */
-    if (code == 0)
-      continue;
-    if (code == READ_KEY_SEQ_ERROR)
-      break;
-    if (code == 27) {
-      fe->send_event(fe, "QUIT");
-      break;
-    }
-
-    if (gui_hotkeys) {
-      if (code == 'f' || code == 'F') {
-        fe->send_event(fe, "TOGGLE_FULLSCREEN");
-        continue;
-      } else if (code == 'd' || code == 'D') {
-        fe->send_event(fe, "TOGGLE_DEINTERLACE");
-        continue;
-      }
-    }
-
-    snprintf(str, sizeof(str), "%016" PRIX64, code);
-    fe->send_input_event(fe, "KBD", str, 0, 0);
-
-  } while (fe->xine_is_finished(fe, 0) != FE_XINE_EXIT);
-
-  alarm(0);
-
-  LOGDBG("Keyboard thread terminating");
-
-  pthread_cleanup_pop(1);
-
-  pthread_exit(NULL);
-  return NULL; /* never reached */
-}
-
-/*
- * slave_receiver_thread()
- *
- * Read slave mode commands from stdin
- * Interpret and execute valid commands
- */
-
-static void slave_receiver_thread_cleanup(void *arg)
-{
-  /* restore terminal settings */
-  tcsetattr(STDIN_FILENO, TCSANOW, &saved_tm);
-  LOGDBG("Slave mode receiver terminated");
-}
-
-static void *slave_receiver_thread(void *fe_gen)
-{
-  frontend_t *fe = (frontend_t*)fe_gen;
-  char str[128], *pt;
-
-  tcgetattr(STDIN_FILENO, &saved_tm);
-
-  pthread_cleanup_push(slave_receiver_thread_cleanup, NULL);
-
-  do {
-    errno = 0;
-    str[0] = 0;
-
-    pthread_testcancel();
-    if (!fgets(str, sizeof(str), stdin))
-      break;
-    pthread_testcancel();
-
-    if (NULL != (pt = strchr(str, '\r')))
-      *pt = 0;
-    if (NULL != (pt = strchr(str, '\n')))
-      *pt = 0;
-
-    if (!strncasecmp(str, "QUIT", 4)) {
-      fe->send_event(fe, "QUIT");
-      break;
-    }
-    if (!strncasecmp(str, "FULLSCREEN", 10)) {
-      fe->send_event(fe, "TOGGLE_FULLSCREEN");
-      continue;
-    }
-    if (!strncasecmp(str, "DEINTERLACE ", 12)) {
-      fe->send_event(fe, str);
-      continue;
-    }
-    if (!strncasecmp(str, "HITK ", 5)) {
-      fe->send_input_event(fe, NULL, str+5, 0, 0);
-      continue;
-    }
-
-    LOGMSG("Unknown slave mode command: %s", str);
-
-  } while (fe->xine_is_finished(fe, 0) != FE_XINE_EXIT);
-
-  LOGDBG("Slave mode receiver terminating");
-
-  pthread_cleanup_pop(1);
-
-  pthread_exit(NULL);
-  return NULL; /* never reached */
-}
-
-/*
- * kbd_start()
- *
- * Start keyboard/slave mode reader thread
- */
-static void kbd_start(frontend_t *fe, int slave_mode)
-{
-  int err;
-  if ((err = pthread_create (&kbd_thread,
-                             NULL,
-                             slave_mode ? slave_receiver_thread : kbd_receiver_thread,
-                             (void*)fe)) != 0) {
-    fprintf(stderr, "Can't create new thread for keyboard (%s)\n",
-            strerror(err));
-  }
-}
-
-/*
- * kbd_stop()
- *
- * Stop keyboard/slave mode reader thread
- */
-static void kbd_stop(void)
-{
-  void *p;
-
-  pthread_cancel(kbd_thread);
-  pthread_join(kbd_thread, &p);
-}
-
 /*
  * SignalHandler()
  */
@@ -372,54 +90,69 @@ static char *strcatrealloc(char *dest, const char *src)
 static const char help_str[] =
 "When server address is not given, server is searched from local network.\n"
 "If server is not found, localhost (127.0.0.1) is used as default.\n\n"
-    "   --help                        Show (this) help message\n"
-    "   --audio=audiodriver[:device]  Select audio driver and optional port\n"
-    "   --video=videodriver[:device]  Select video driver and optional port\n"
+    "   -H, --help                    Show (this) help message\n"
+    "   -A, --audio=audiodriver[:device]\n"
+    "                                 Select audio driver and optional port\n"
+    "   -V, --video=videodriver[:device]\n"
+    "                                 Select video driver and optional port\n"
 #ifndef IS_FBFE
-    "   --display=displayaddress      X11 display address\n"
-    "   --wid=id                      Use existing X11 window\n"
+    "   -d, --display=displayaddress  X11 display address\n"
+    "   -W, --wid=id                  Use existing X11 window\n"
 #endif
-    "   --aspect=[auto|4:3|16:9|16:10|default]\n"
+    "   -a, --aspect=[auto|4:3|16:9|16:10|default]\n"
     "                                 Display aspect ratio\n"
     "                                 Use script to control HW aspect ratio:\n"
-    "                                   --aspect=auto:path_to_script\n"
-    "   --fullscreen                  Fullscreen mode\n"
+    "                                 --aspect=auto:path_to_script\n"
+    "   -f, --fullscreen              Fullscreen mode\n"
 #ifdef HAVE_XRENDER
-    "   --hud                         Head Up Display OSD mode\n"
+    "   -D, --hud[=flag[,flag]]       Head Up Display OSD mode using compositing\n"
+    "                                 Optional flags:\n"
+#  ifdef HAVE_XSHAPE
+    "                                 xshape  Use XShape instead of compositing\n"
+#  endif
+#  ifdef HAVE_OPENGL
+    "                                 opengl  Use OpenGL instead of compositing\n"
+#  endif
 #endif
-    "   --width=x                     Video window width\n"
-    "   --height=x                    Video window height\n"
-    "   --geometry=WxH[+X+Y]          Set output window geometry (X style)\n"
-    "   --buffers=x                   Number of PES buffers\n"
-    "   --noscaling                   Disable all video scaling\n"
-    "   --post=name[:arg=val[,arg=val]] Load and use xine post plugin(s)\n"
-    "                                 examples:\n"
+#ifdef HAVE_OPENGL
+    "   -O, --opengl                  Use OpenGL for video and Head Up Display OSD\n"
+#endif
+    "   -w, --width=x                 Video window width\n"
+    "   -h, --height=x                Video window height\n"
+    "   -g, --geometry=WxH[+X+Y]      Set output window geometry (X style)\n"
+    "   -B, --buffers=x               Number of PES buffers\n"
+    "   -n, --noscaling               Disable all video scaling\n"
+    "   -P, --post=name[:arg=val[,arg=val]]\n"
+    "                                 Load and use xine post plugin(s)\n"
+    "                                 Examples:\n"
     "                                 --post=upmix\n"
     "                                 --post=upmix;tvtime:enabled=1,cheap_mode=1\n"
-    "   --lirc[=devicename]           Use lirc input device\n"
+    "   -L, --lirc[=devicename]       Use lirc input device\n"
     "                                 Optional lirc socket name can be given\n"
-    "   --config=file                 Use config file (default: ~/.xine/config_xineliboutput).\n"
-    "   --verbose                     Verbose debug output\n"
-    "   --silent                      Silent mode (report only errors)\n"
-    "   --syslog                      Write all output to system log\n"
-    "   --nokbd                       Disable console keyboard input\n"
+    "   -C, --config=file             Use config file (default: ~/.xine/config_xineliboutput).\n"
+    "   -v, --verbose                 Verbose debug output\n"
+    "   -s, --silent                  Silent mode (report only errors)\n"
+    "   -l, --syslog                  Write all output to system log\n"
+    "   -k, --nokbd                   Disable console keyboard input\n"
 #ifndef IS_FBFE
-    "   --noxkbd                      Disable X11 keyboard input\n"
+    "   -x, --noxkbd                  Disable X11 keyboard input\n"
 #endif
-    "   --hotkeys                     Enable frontend GUI hotkeys\n"
-    "   --terminal=dev                Controlling tty"
-    "   --daemon                      Run as daemon (disable keyboard,\n"
+    "   -o, --hotkeys                 Enable frontend GUI hotkeys\n"
+    "   -p, --shutdown=MIN[:CMD]      Shutdown after MIN minutes of inactivity\n"
+    "                                 Use CMD to perform shutdown (default: /sbin/shutdown)\n"
+    "   -T, --terminal=dev            Controlling TTY\n"
+    "   -b, --daemon                  Run as daemon (disable keyboard,\n"
     "                                 log to syslog and fork to background)\n"
-    "   --slave                       Enable slave mode (read commands from stdin)\n"
-    "   --reconnect                   Automatically reconnect when connection has been lost\n"
-    "   --tcp                         Use TCP transport\n"
-    "   --udp                         Use UDP transport\n"
-    "   --rtp                         Use RTP transport\n\n"
+    "   -S, --slave                   Enable slave mode (read commands from stdin)\n"
+    "   -R, --reconnect               Automatically reconnect when connection has been lost\n"
+    "   -t, --tcp                     Use TCP transport\n"
+    "   -u, --udp                     Use UDP transport\n"
+    "   -r, --rtp                     Use RTP transport\n\n"
     "                                 If no transport options are given, transports\n"
     "                                 are tried in following order:\n"
     "                                 local pipe, rtp, udp, tcp\n\n";
 
-static const char short_options[] = "HA:V:d:W:a:fg:Dw:h:B:nP:L:C:T:vsxlkobSRtur";
+static const char short_options[] = "HA:V:d:W:a:fg:Dw:h:B:nP:L:C:T:p:vsxlkobSRtur";
 
 static const struct option long_options[] = {
   { "help",       no_argument,       NULL, 'H' },
@@ -430,7 +163,8 @@ static const struct option long_options[] = {
   { "aspect",     required_argument, NULL, 'a' },
   { "fullscreen", no_argument,       NULL, 'f' },
   { "geometry",   required_argument, NULL, 'g' },
-  { "hud",        no_argument,       NULL, 'D' },
+  { "hud",        optional_argument, NULL, 'D' },
+  { "opengl",     no_argument,       NULL, 'O' },
   { "width",      required_argument, NULL, 'w' },
   { "height",     required_argument, NULL, 'h' },
   { "buffers",    required_argument, NULL, 'B' },
@@ -439,6 +173,7 @@ static const struct option long_options[] = {
   { "lirc",       optional_argument, NULL, 'L' },
   { "config",     required_argument, NULL, 'C' },
   { "terminal",   required_argument, NULL, 'T' },
+  { "shutdown",   required_argument, NULL, 'p' },
 
   { "verbose", no_argument,  NULL, 'v' },
   { "silent",  no_argument,  NULL, 's' },
@@ -453,7 +188,7 @@ static const struct option long_options[] = {
   { "tcp",       no_argument,  NULL, 't' },
   { "udp",       no_argument,  NULL, 'u' },
   { "rtp",       no_argument,  NULL, 'r' },
-  { NULL }
+  { NULL,        no_argument,  NULL,  0  }
 };
 
 #define PRINTF(x...) do { if(SysLogLevel>1) printf(x); } while(0)
@@ -461,7 +196,7 @@ static const struct option long_options[] = {
 int main(int argc, char *argv[])
 {
   int ftcp = 0, fudp = 0, frtp = 0, reconnect = 0, firsttry = 1;
-  int fullscreen = 0, hud = 0, xpos = 0, ypos = 0, width = 720, height = 576;
+  int fullscreen = 0, hud = 0, opengl = 0, xpos = 0, ypos = 0, width = 720, height = 576;
   int pes_buffers = 250;
   int scale_video = 1, aspect = 1;
   int daemon_mode = 0, nokbd = 0, noxkbd = 0, slave_mode = 0;
@@ -470,17 +205,20 @@ int main(int argc, char *argv[])
   int xmajor, xminor, xsub;
   int c;
   int xine_finished = FE_XINE_ERROR;
+  int inactivity_timer = 0;
   char *mrl = NULL;
   char *video_driver = NULL;
   char *audio_driver = NULL;
-  char *audio_device = NULL;
-  char *video_port = NULL;
   char *static_post_plugins = NULL;
   char *lirc_dev = NULL;
-  char *aspect_controller = NULL;
+  char *p;
+  const char *audio_device = NULL;
+  const char *video_port = NULL;
+  const char *aspect_controller = NULL;
   const char *tty = NULL;
   const char *exec_name = argv[0];
   const char *config_file = NULL;
+  const char *power_off_cmd = NULL;
 
   extern const fe_creator_f fe_creator;
   frontend_t *fe = NULL;
@@ -507,17 +245,19 @@ int main(int argc, char *argv[])
               list_xine_plugins(NULL, SysLogLevel>2);
               exit(0);
     case 'A': audio_driver = strdup(optarg);
-              audio_device = strchr(audio_driver, ':');
-              if (audio_device)
-                *(audio_device++) = 0;
+              if (NULL != (p = strchr(audio_driver, ':'))) {
+                *p = 0;
+                audio_device = p + 1;
+              }
               PRINTF("Audio driver: %s\n", audio_driver);
               if (audio_device)
                 PRINTF("Audio device: %s\n", audio_device);
               break;
     case 'V': video_driver = strdup(optarg);
-              video_port   = strchr(video_driver, ':');
-              if (video_port)
-                *(video_port++) = 0;
+              if (NULL != (p = strchr(video_driver, ':'))) {
+                *p = 0;
+                video_port = p + 1;
+              }
               PRINTF("Video driver: %s\n", video_driver);
               if (video_port)
                 PRINTF("Video port: %s\n", video_port);
@@ -525,7 +265,7 @@ int main(int argc, char *argv[])
 #ifndef IS_FBFE
     case 'W': window_id = atoi(optarg);
               break;
-    case 'd': video_port = strdup(optarg);
+    case 'd': video_port = optarg;
               break;
     case 'x': noxkbd = 1;
               break;
@@ -539,22 +279,41 @@ int main(int argc, char *argv[])
               if (!strncmp(optarg, "16:10", 5))
                 aspect = 4;
               if (aspect == 0 && optarg[4] == ':')
-                aspect_controller = strdup(optarg+5);
+                aspect_controller = optarg + 5;
               PRINTF("Aspect ratio: %s\n",
                      aspect==0?"Auto":aspect==2?"4:3":aspect==3?"16:9":
                      aspect==4?"16:10":"Default");
               if (aspect_controller)
-                PRINTF("Using %s to switch aspect ratio\n",
-                       aspect_controller);
+                PRINTF("Using %s to switch aspect ratio\n", aspect_controller);
               break;
     case 'f': fullscreen=1;
               PRINTF("Fullscreen mode\n");
               break;
-    case 'D': hud=1;
+    case 'D': hud = HUD_COMPOSITE;
 #ifdef HAVE_XRENDER
               PRINTF("HUD OSD mode\n");
 #else
               PRINTF("HUD OSD not supported\n");
+#endif
+              if (optarg && strstr(optarg, "xshape")) {
+                hud |= HUD_XSHAPE;
+#ifndef HAVE_XSHAPE
+                PRINTF("XShape HUD OSD not supported\n");
+#endif
+              }
+              if (optarg && strstr(optarg, "opengl")) {
+                hud |= HUD_OPENGL;
+#ifndef HAVE_OPENGL
+                PRINTF("OpenGL HUD OSD not supported\n");
+#endif
+              }
+              break;
+    case 'O': opengl = 1;
+              hud |= HUD_OPENGL;
+#ifdef HAVE_OPENGL
+              PRINTF("Using OpenGL to draw video and HUD OSD\n");
+#else
+              PRINTF("OpenGL not supported\n");
 #endif
               break;
     case 'w': width = atoi(optarg);
@@ -574,11 +333,16 @@ int main(int argc, char *argv[])
                 PRINTF("Can't access terminal: %s\n", tty);
                 return -2;
               }
-              PRINTF("Terminal: %s", tty);
+              PRINTF("Terminal: %s\n", tty);
               break;
     case 'n': scale_video = 0;
               PRINTF("Video scaling disabled\n");
               break;
+    case 'p': inactivity_timer = atoi(optarg);
+              power_off_cmd = strchr(optarg, ':');
+              power_off_cmd = power_off_cmd ? power_off_cmd+1 : "/sbin/shutdown";
+              PRINTF("Shutdown after %d minutes of inactivity using %s\n", inactivity_timer, power_off_cmd);
+              break;
     case 'P': if (static_post_plugins)
                 strcatrealloc(static_post_plugins, ";");
               static_post_plugins = strcatrealloc(static_post_plugins, optarg);
@@ -614,8 +378,10 @@ int main(int argc, char *argv[])
               PRINTF("GUI hotkeys enabled\n"
                      "  mapping keyboard f,F     -> fullscreen toggle\n"
                      "          keyboard d,D     -> deinterlace toggle\n"
+                     "          keyboard p,P     -> power off\n"
                      "          LIRC Deinterlace -> deinterlace toggle\n"
                      "          LIRC Fullscreen  -> fullscreen toggle\n"
+                     "          LIRC PowerOff    -> power off\n"
                      "          LIRC Quit        -> exit\n");
               break;
     case 'b': nokbd = daemon_mode = 1;
@@ -696,7 +462,7 @@ int main(int argc, char *argv[])
       strncmp(mrl, MRL_ID ":", MRL_ID_LEN+1) &&
       strncmp(mrl, MRL_ID "+", MRL_ID_LEN+1)) {
     char *mrl2 = mrl;
-    PRINTF("WARNING: MRL does not start with \'" MRL_ID ":\' (%s)", mrl);
+    PRINTF("WARNING: MRL does not start with \'" MRL_ID ":\' (%s)\n", mrl);
     if (asprintf(&mrl, MRL_ID "://%s", mrl) < 0)
       return -1;
     free(mrl2);
@@ -733,7 +499,7 @@ int main(int argc, char *argv[])
   }
 
   /* Initialize display */
-  if (!fe->fe_display_open(fe, xpos, ypos, width, height, fullscreen, hud, 0,
+  if (!fe->fe_display_open(fe, xpos, ypos, width, height, fullscreen, hud, opengl, 0,
                            "", aspect, NULL, noxkbd, gui_hotkeys,
                            video_port, scale_video, 0,
                            aspect_controller, window_id)) {
@@ -750,6 +516,9 @@ int main(int argc, char *argv[])
     fe->fe_free(fe);
     return -5;
   }
+  if (power_off_cmd) {
+    fe->shutdown_init(fe, power_off_cmd, inactivity_timer * 60);
+  }
 
   if (SysLogLevel > 2)
     list_xine_plugins(fe, SysLogLevel>2);
@@ -805,8 +574,6 @@ int main(int argc, char *argv[])
 
     /* Main loop */
 
-    sleep(2);  /* give input_vdr some time to establish connection */
-
     fflush(stdout);
     fflush(stderr);
 
@@ -840,7 +607,6 @@ int main(int argc, char *argv[])
   free(mrl);
   free(audio_driver);
   free(video_driver);
-  free(aspect_controller);
   free(lirc_dev);
 
   return xine_finished==FE_XINE_EXIT ? 0 : 1;
