@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_post_swscale.c,v 1.9 2008/12/13 14:24:03 phintuka Exp $
+ * $Id: xine_post_swscale.c,v 1.15 2011/03/02 12:42:22 phintuka Exp $
  *
  * Simple (faster) resize for avisynth
  *     Copyright (C) 2002 Tom Barry
@@ -36,12 +36,37 @@
  *    - Added PIC support (do not clobber ebx in x86, access only local variables from asm)
  *    - Fixed yv12 stretched warp tables generation
  */
- 
+
 #include <xine/xine_internal.h>
 #include <xine/post.h>
+#include <float.h>  /* DBL_MIN */
+
+/*
+ * tools
+ */
+
+#ifndef ALIGN
+#  define ALIGN(b,p) ((void*)((((unsigned long)(p)) + (b)-1) & (~((b)-1))))
+#endif
+#ifndef MIN
+#  define MIN(a,b) ((a) < (b) ? (a) : (b))
+#endif
+#ifndef MAX
+#  define MAX(a,b) ((a) > (b) ? (a) : (b))
+#endif
+#ifndef FABS
+#  define FABS(x) ((x) < 0.0 ? -(x) : (x))
+#endif
+
+static int doublecmp(double a, double b) {
+  if      (FABS(a-b) <= 2*DBL_MIN) return 0; // equal
+  else if (a > b) return 1;
+  else            return -1;
+}
+
 
 /*#define DBG(x...)*/
-#define DBG(x...) fprintf(stderr, "post_warp: " x)
+#define DBG(x...) do { if(this->config.debug) fprintf(stderr, "post_warp: " x); } while (0)
 
 /*#define STREAMING_STORE_TMP*/
 /*#define STREAMING_STORE*/
@@ -114,8 +139,6 @@ static void init_tables_yv12(int newwidth, int newheight, int oldwidth, int oldh
   int k;
   int wY1;
   int wY2;
-  DBG("init_yv12: %dx%d->%dx%d hWarp %1.3lf vWarp %1.3lf\n", 
-      oldwidth, oldheight, newwidth, newheight, hWarp, vWarp);
 
   /* First set up horizontal table, use for both luma & chroma since 
    * it seems to have the same equation.
@@ -125,7 +148,7 @@ static void init_tables_yv12(int newwidth, int newheight, int oldwidth, int oldh
 
   for(i=0; i < newwidth; i+=2) {
     /* first make even pixel control */
-    if (hWarp==1)  /*if no warp factor */
+    if (doublecmp(hWarp, 1.0) == 0)  /* if no warp factor */
       j = i * 256 * (oldwidth-1) / (newwidth-1);
     else           /* stretch and warp somehow */
       j = (int) (256 * WarpFactor(i / (newwidth-1.0), hWarp) * (oldwidth-1));
@@ -143,7 +166,7 @@ static void init_tables_yv12(int newwidth, int newheight, int oldwidth, int oldh
     }
 
     /* now make odd pixel control */
-    if (hWarp==1)   /* if no warp factor */
+    if (doublecmp(hWarp, 1.0) == 0)   /* if no warp factor */
       j = (i+1) * 256 * (oldwidth-1) / (newwidth-1);
     else        /* stretch and warp somehow */
       j = (int) (256 * WarpFactor((i+1) / (newwidth-1.0), hWarp) * (oldwidth-1));
@@ -167,7 +190,7 @@ static void init_tables_yv12(int newwidth, int newheight, int oldwidth, int oldh
   // UV
   for(i=0; i < newwidth/2; i+=2) {
     /* first make even pixel control */
-    if (hWarp==1)  /*if no warp factor */
+    if (doublecmp(hWarp, 1.0) == 0)  /* if no warp factor */
       j = i * 256 * (oldwidth/2-1) / (newwidth/2-1);
     else           /* stretch and warp somehow */
       j = (int) (256 * WarpFactor(i / (newwidth/2-1.0), hWarp) * (oldwidth/2-1));
@@ -185,7 +208,7 @@ static void init_tables_yv12(int newwidth, int newheight, int oldwidth, int oldh
     }
 
     /* now make odd pixel control */
-    if (hWarp==1)   /* if no warp factor */
+    if (doublecmp(hWarp, 1.0) == 0)   /* if no warp factor */
       j = (i+1) * 256 * (oldwidth/2-1) / (newwidth/2-1);
     else        /* stretch and warp somehow */
       j = (int) (256 * WarpFactor((i+1) / (newwidth/2-1.0), hWarp) * (oldwidth/2-1));
@@ -214,7 +237,7 @@ static void init_tables_yv12(int newwidth, int newheight, int oldwidth, int oldh
 
   /* First Luma Table */
   for(i=0; i< newheight; ++i) {
-    if (vWarp==1)  /* if no warp factor */
+    if (doublecmp(vWarp, 1.0) == 0)  /* if no warp factor */
       j = i * 256 * (oldheight-1) / (newheight-1);
     else           /* stretch and warp somehow */
       j = (int) (256 * WarpFactor(i / (newheight-1.0), vWarp) * (oldheight-1));
@@ -246,7 +269,7 @@ static void init_tables_yv12(int newwidth, int newheight, int oldwidth, int oldh
 
   /* Vertical table for chroma */
   for(i=0; i< newheight/2; ++i) {
-    if (vWarp==1)  /* if no warp factor */
+    if (doublecmp(vWarp, 1.0) == 0)  /* if no warp factor */
 #ifdef VANILLA
       j = (int) ( (i+.25) * 256 * (oldheight-1) / (newheight-1.0) - 64 );
 #else
@@ -319,13 +342,11 @@ static void init_tables_yuy2(int newwidth, int newheight, int oldwidth, int oldh
   int wY2;
   int wUV1;
   int wUV2;
-  DBG("init_yuy2: %dx%d->%dx%d hWarp %1.3lf vWarp %1.3lf\n", 
-      oldwidth, oldheight, newwidth, newheight, hWarp, vWarp);
   
   /* First set up horizontal table */
   for(i=0; i < newwidth; i+=2) {
     /* first make even pixel control */
-    if (hWarp==1)          /* if no warp factor */
+    if (doublecmp(hWarp, 1.0) == 0)          /* if no warp factor */
       j = i * 256 * (oldwidth-1) / (newwidth-1);
     else                   /* stretch and warp somehow */
       j = (int) (256 * WarpFactor(i / (newwidth-1.0), hWarp) * (oldwidth-1));
@@ -347,7 +368,7 @@ static void init_tables_yuy2(int newwidth, int newheight, int oldwidth, int oldh
     }
     
     /* now make odd pixel control */
-    if (hWarp==1)                          /* if no warp factor */
+    if (doublecmp(hWarp, 1.0) == 0)  /* if no warp factor */
       j = (i+1) * 256 * (oldwidth-1) / (newwidth-1);
     else                                   /* stretch and warp somehow */
       j = (int) (256 * WarpFactor((i+1) / (newwidth-1.0), hWarp) * (oldwidth-1));
@@ -377,7 +398,7 @@ static void init_tables_yuy2(int newwidth, int newheight, int oldwidth, int oldh
   /* Next set up vertical table. The offsets are measured in lines and will be mult */
   /* by the source pitch later */
   for(i=0; i< newheight; ++i) {
-    if (vWarp==1)                   /* if no warp factor */
+    if (doublecmp(vWarp, 1.0) == 0)                   /* if no warp factor */
       j = i * 256 * (oldheight-1) / (newheight-1);
     else                            /* stretch and warp somehow */
       j = (int) (256 * WarpFactor(i / (newheight-1.0), vWarp) * (oldheight-1));
@@ -481,7 +502,7 @@ typedef union {
 
 static int do_warp_yuy2(uint8_t *dst, const uint8_t *src,
 			const int dst_pitch, const int src_pitch,
-			const int dst_width, const int dst_height,			 
+			const int dst_width, const int dst_height,
 			const int src_width, const int src_height,
 			const int Interlaced, const uint32_t * const hControl, 
 			const uint32_t * const vOffsets, const uint32_t * const vWeights,
@@ -517,7 +538,7 @@ static int do_warp_yuy2(uint8_t *dst, const uint8_t *src,
 
   for (y = dst_start; y < dst_height; y++) {
 
-    if(vOffsets[y] >= src_height) {
+    if((int)vOffsets[y] >= src_height) {
       /* slice completed */
       /*DBG("do_warp_yuy2: max input height reached: need line %d, height %d\n -> Returning next output line: %d\n",
 	vOffsets[y], src_height, y);*/
@@ -563,10 +584,10 @@ static int do_warp_yuy2(uint8_t *dst, const uint8_t *src,
 	    
 	    ".align 16                   \n\t"
 
-	    "movdqa "_FPround1", %%xmm0  \n\t"
-	    "movdqa "_vWeight1", %%xmm5  \n\t"
-	    "movdqa "_vWeight2", %%xmm6  \n\t"
-	    "movdqa "_YMask",    %%xmm7  \n"
+	    "movdqu "_FPround1", %%xmm0  \n\t"
+	    "movdqu "_vWeight1", %%xmm5  \n\t"
+	    "movdqu "_vWeight2", %%xmm6  \n\t"
+	    "movdqu "_YMask",    %%xmm7  \n"
 
 	    ""vLoopSSE2_Fetch":          \n\t"
 #ifdef PREFETCH
@@ -859,7 +880,7 @@ static int do_warp_yv12(uint8_t *dst, const uint8_t * const src,
    */
 
   for (y = dst_start; y < dst_height; y++) {
-    if(vOffsets[y] >= src_height) {
+    if((int)vOffsets[y] >= src_height) {
       /* slice completed */
       /*DBG("do_warp_yv12: max input height reached: need line %d, height %d\n -> Returning next output line: %d , start was %d\n",
 	(int)vOffsets[y], (int)src_height, (int)y, (int)dst_start);*/
@@ -904,9 +925,9 @@ static int do_warp_yv12(uint8_t *dst, const uint8_t * const src,
 	     "shr   $1, %%"REGC"          \n\t"  /* do 16 bytes at a time instead */
 	     "dec   %%"REGC"              \n\t"  /* jigger loop ct */
 			 
-	     "movdqa "_FPround1", %%xmm0  \n\t"
-	     "movdqa "_vWeight1", %%xmm5  \n\t"
-	     "movdqa "_vWeight2", %%xmm6  \n\t"
+	     "movdqu "_FPround1", %%xmm0  \n\t"
+	     "movdqu "_vWeight1", %%xmm5  \n\t"
+	     "movdqu "_vWeight2", %%xmm6  \n\t"
 	     "pxor        %%xmm7, %%xmm7  \n"
 
 	     ".align 16                   \n"
@@ -1272,23 +1293,6 @@ static int do_warp_yv12(uint8_t *dst, const uint8_t * const src,
   return 0; 
 }
 
-/*
- * tools
- */
-
-#ifndef ALIGN
-#  define ALIGN(b,p) ((void*)((((unsigned long)(p)) + (b)-1) & (~((b)-1))))
-#endif
-#ifndef MIN
-#  define MIN(a,b) ((a) < (b) ? (a) : (b))
-#endif
-#ifndef MAX
-#  define MAX(a,b) ((a) > (b) ? (a) : (b))
-#endif
-#ifndef FABS
-#  define FABS(x) ((x) < 0.0 ? -(x) : (x))
-#endif
-
 /*
  * xine plugin
  */
@@ -1300,23 +1304,9 @@ static int do_warp_yv12(uint8_t *dst, const uint8_t * const src,
 /*#define POST_SLICES*/
 #include "xine/post_util.h"
 
-
-/* plugin class initialization function */
-void *warp_init_plugin(xine_t *xine, void *);
-
-/* plugin class functions */
-static post_plugin_t *open_plugin(post_class_t *class_gen, int inputs,
-				  xine_audio_port_t **audio_target,
-				  xine_video_port_t **video_target);
-
 /* plugin instance functions */
 static void        warp_dispose(post_plugin_t *this_gen);
 
-/* vo_frame functions */
-static vo_frame_t *got_frame(vo_frame_t *frame);
-static void        draw_internal(vo_frame_t *frame, vo_frame_t *new_frame);
-
-
 /* parameter functions */
 static xine_post_api_descr_t *warp_get_param_descr(void);
 static int                    warp_set_parameters(xine_post_t *this_gen, void *param_gen);
@@ -1329,6 +1319,7 @@ typedef struct warp_parameters_s {
   int    output_height;
   double output_aspect;
   int    no_downscaling;
+  int    debug;
 } warp_parameters_t;
 
 START_PARAM_DESCR(warp_parameters_t)
@@ -1340,6 +1331,8 @@ PARAM_ITEM(POST_PARAM_TYPE_DOUBLE, output_aspect, NULL,   1,    3, 0,
   "output video aspect ratio")
 PARAM_ITEM(POST_PARAM_TYPE_BOOL,   no_downscaling,NULL,   0,    1, 0,
   "disable downscaling")
+PARAM_ITEM(POST_PARAM_TYPE_BOOL,   debug,         NULL,   0,    1, 0,
+  "debug mode")
 END_PARAM_DESCR(warp_param_descr)
 
 
@@ -1407,6 +1400,11 @@ static void init_tables(warp_plugin_t *this)
     this->vWeightsUV = (uint32_t*)ALIGN(128, BP(this->vOffsetsUV) + this->output_height * sizeof(uint32_t) + 128);
     this->hControlUV = (uint32_t*)ALIGN(128, BP(this->vWeightsUV) + this->output_height * sizeof(uint32_t) + 128);
 
+    DBG("init_yv12: %dx%d->%dx%d hWarp %1.3lf vWarp %1.3lf\n",
+        this->input_width,  this->input_height,
+        this->output_width, this->output_height,
+        this->factor_x, this->factor_y);
+
     init_tables_yv12(this->output_width, this->output_height,
 		     this->input_width,  this->input_height,
 		     this->input_interlaced, this->factor_x, this->factor_y, 
@@ -1415,6 +1413,11 @@ static void init_tables(warp_plugin_t *this)
 
   } else if (this->input_format == XINE_IMGFMT_YUY2) {
 
+    DBG("init_yuy2: %dx%d->%dx%d hWarp %1.3lf vWarp %1.3lf\n",
+        this->input_width,  this->input_height,
+        this->output_width, this->output_height,
+        this->factor_x, this->factor_y);
+
     init_tables_yuy2(this->output_width, this->output_height,
 		     this->input_width,  this->input_height,
 		     this->input_interlaced, this->factor_x, this->factor_y, 
@@ -1575,8 +1578,11 @@ static vo_frame_t *got_frame(vo_frame_t *frame)
     this->input_aspect = frame->ratio;
     this->input_interlaced = !!(frame->flags & VO_INTERLACED_FLAG);
 
-    /* re-configure target size and aspect ratio */ 
-    this->output_aspect = this->config.output_aspect ?: frame->ratio;
+    /* re-configure target size and aspect ratio */
+    if (doublecmp(this->config.output_aspect, 0.0) == 0)
+      this->output_aspect = frame->ratio;
+    else
+      this->output_aspect = this->config.output_aspect;
     if (!this->config.no_downscaling) {
       this->output_width  = this->config.output_width  ?: frame->width;
       this->output_height = this->config.output_height ?: frame->height;
@@ -1679,9 +1685,9 @@ static int warp_set_parameters(xine_post_t *this_gen, void *param_gen)
     this->config.output_aspect /= 1000.0;
 
   DBG("warp_set_parameters: "
-      "output_width=%d, output_height=%d, output_aspect=%4.3lf, no_downscaling=%d\n",
+      "output_width=%d, output_height=%d, output_aspect=%4.3lf, no_downscaling=%d, debug=%d\n",
       this->config.output_width, this->config.output_height, this->config.output_aspect,
-      this->config.no_downscaling);
+      this->config.no_downscaling, this->config.debug);
 
   return 1;
 }
@@ -1710,6 +1716,7 @@ static char *warp_get_help(void) {
 	   "  output_aspect:      Adjust aspect ratio using non-linear scaling\n"
 	   "                      (0 -> do not change video aspect ratio)\n"
 	   "  no_downscaling:     Do not downscale video\n"
+           "  debug:              Output debug messages\n"
            "\n"
          );
 }
