@@ -21,39 +21,37 @@
  *
  * xineliboutput.c: VDR Plugin interface
  *
- * $Id: xineliboutput.c,v 1.30.2.11 2010/02/07 01:21:22 phintuka Exp $
+ * $Id: xineliboutput.c,v 1.55 2012/03/24 09:25:53 phintuka Exp $
  *
  */
 
+#include "features.h"
+
 #include <vdr/plugin.h>
+#include <vdr/i18n.h>
 
 #include "logdefs.h"
-#include "i18n.h"
 #include "config.h"
 #include "device.h"
 #include "setup_menu.h"
 #include "menu.h"
 #include "media_player.h"
 
-#if VDRVERSNUM < 10400
-# error VDR versions < 1.4.0 are not supported !
-#endif
-#if VDRVERSNUM > 10700
-# error VDR versions > 1.7.0 are not supported !
+#if defined(APIVERSNUM) && (APIVERSNUM < 10600)
+# error VDR API versions < 1.6.0 are not supported !
 #endif
 
 //---------------------------------plugin-------------------------------------
 
-static const char *VERSION        = "1.0.6";
+static const char *VERSION        = "1.0.90-cvs";
 static const char *DESCRIPTION    = trNOOP("X11/xine-lib output plugin");
 static const char *MAINMENUENTRY  = trNOOP("Media Player");
 
-cOsdObject *g_PendingMenuAction = NULL;
-
 class cPluginXinelibOutput : public cPlugin 
 {
   private:
     // Add any member variables or functions you may need here.
+    cXinelibDevice *m_Dev;
 
   public:
     cPluginXinelibOutput(void);
@@ -88,6 +86,8 @@ cPluginXinelibOutput::cPluginXinelibOutput(void)
   // Initialize any member variables here.
   // DON'T DO ANYTHING ELSE THAT MAY HAVE SIDE EFFECTS, REQUIRE GLOBAL
   // VDR OBJECTS TO EXIST OR PRODUCE ANY OUTPUT!
+
+  m_Dev = NULL;
 }
 
 cPluginXinelibOutput::~cPluginXinelibOutput()
@@ -125,13 +125,25 @@ const char cmdLineHelp[] =
 "                           (example: )\n" 
 #endif
 "  -f        --fullscreen   Fullscreen mode (X11)\n"
-#ifdef HAVE_XRENDER
-"  -D        --hud          Head Up Display OSD (X11)\n"
+#if defined(HAVE_XRENDER) || defined(HAVE_OPENGL)
+"  -D        --hud[=flag[,flag]]\n"
+"                           Head Up Display OSD (X11)\n"
+"                           flags:\n"
+#endif
+#if defined(HAVE_XRENDER) && defined(HAVE_XSHAPE)
+"                           xshape  Use XShape instead of compositing\n"
+#endif
+#ifdef HAVE_OPENGL
+"                           opengl  Use OpenGL instead of compositing\n"
+"   -O       --opengl       Use OpenGL for video and Head Up Display OSD\n"
 #endif
 "  -w        --width=x      Window width\n"
 "  -h        --height=x     Window width\n"
+"  -g,       --geometry=WxH[+X+Y] Set output window geometry (X style)\n"
 "  -d DISP   --display=DISP Use X11 display DISP\n"
 "                           (or framebuffer device name)\n"
+"  -W ID     --wid=ID       Use existing X11 window\n"
+"                           Special ID for root window: --wid=root\n"
 "  -P NAME   --post=NAME    Use xine post plugin NAME\n"
 "                           format: pluginname[:arg=val[,arg=val]][,...]\n"
 "                           example: \n"
@@ -159,32 +171,36 @@ bool cPluginXinelibOutput::Initialize(void)
   // Initialize any background activities the plugin shall perform.
   TRACEF("cPluginXinelibOutput::Initialize");
 
-#if VDRVERSNUM < 10507
-  RegisterI18n(Phrases);
-#endif
+  m_Dev = &(cXinelibDevice::Instance());
 
-  cXinelibDevice::Instance();
-  return true;
+  return m_Dev ? m_Dev->InitDevice() : false;
 }
 
 bool cPluginXinelibOutput::Start(void)
 {
   // Start any background activities the plugin shall perform.
   TRACEF("cPluginXinelibOutput::Start");
-  return cXinelibDevice::Instance().StartDevice();
+
+  return m_Dev ? m_Dev->StartDevice() : false;
 }
 
 void cPluginXinelibOutput::MainThreadHook(void)
 {
   TRACEF("cPluginXinelibOutput::MainThreadHook");
-  return cXinelibDevice::Instance().MainThreadHook();
+
+  if (m_Dev) {
+    m_Dev->MainThreadHook();
+  }
 }
 
 void cPluginXinelibOutput::Stop(void)
 {
   // Start any background activities the plugin shall perform.
   TRACEF("cPluginXinelibOutput::Stop");
-  return cXinelibDevice::Instance().StopDevice();
+
+  if (m_Dev) {
+    m_Dev->StopDevice();
+  }
 }
 
 cOsdObject *cPluginXinelibOutput::MainMenuAction(void)
@@ -192,34 +208,32 @@ cOsdObject *cPluginXinelibOutput::MainMenuAction(void)
   // Perform the action when selected from the main VDR menu.
   TRACEF("cPluginXinelibOutput::MainMenuAction");
 
-  if(xc.main_menu_mode == CloseOsd) {
+  if (!m_Dev) {
+    return NULL;
+  }
+  if (xc.main_menu_mode == CloseOsd) {
     xc.main_menu_mode = ShowMenu;
     return NULL;
   }
 
-  if(g_PendingMenuAction) {
-    cOsdObject *tmp = g_PendingMenuAction;
-    g_PendingMenuAction = NULL;
+  if (xc.pending_menu_action) {
+    cOsdObject *tmp = xc.pending_menu_action;
+    xc.pending_menu_action = NULL;
     return tmp;
   }
 
-  if(xc.hide_main_menu)
+  if (xc.hide_main_menu)
     return NULL;
 
-#ifdef HAVE_XV_FIELD_ORDER
-  xc.field_order = xc.field_order ? 0 : 1;
-  cXinelibDevice::Instance().ConfigureWindow(xc.fullscreen, xc.width, xc.height, 
-					     xc.modeswitch, xc.modeline, xc.display_aspect, 
-					     xc.scale_video, xc.field_order);
-#endif
-  return new cMenuXinelib();
+  return m_Dev ? new cMenuXinelib(m_Dev) : NULL;
 }
 
 cMenuSetupPage *cPluginXinelibOutput::SetupMenu(void)
 {
   // Return a setup menu in case the plugin supports one.
   TRACEF("cPluginXinelibOutput::SetupMenu");
-  return new cMenuSetupXinelib();
+
+  return m_Dev ? new cMenuSetupXinelib(m_Dev) : NULL;
 }
 
 bool cPluginXinelibOutput::SetupParse(const char *Name, const char *Value)
@@ -235,9 +249,9 @@ bool cPluginXinelibOutput::Service(const char *Id, void *Data)
 
     if(!strcmp(Id, "MediaPlayer-1.0")) {
       if(CData && *CData) {
-	LOGMSG("Service(%s, %s)", Id, CData);
-	cControl::Launch(new cXinelibPlayerControl(ShowFiles, CData));
-	return true;
+        LOGMSG("Service(%s, %s)", Id, CData);
+        cPlayerFactory::Launch(m_Dev, pmAudioVideo, CData);
+        return true;
       }
       LOGMSG("Service(%s) -> true", Id);
       return true;
@@ -245,9 +259,9 @@ bool cPluginXinelibOutput::Service(const char *Id, void *Data)
 
     else if(!strcmp(Id, "MusicPlayer-1.0")) {
       if(CData && *CData) {
-	LOGMSG("Service(%s, %s)", Id, CData);
-	cControl::Launch(new cXinelibPlayerControl(ShowMusic, CData));
-	return true;
+        LOGMSG("Service(%s, %s)", Id, CData);
+        cPlayerFactory::Launch(m_Dev, pmAudioOnly, CData);
+        return true;
       }
       LOGMSG("Service(%s) -> true", Id);
       return true;
@@ -255,9 +269,9 @@ bool cPluginXinelibOutput::Service(const char *Id, void *Data)
 
     else if(!strcmp(Id, "DvdPlayer-1.0")) {
       if(Data && *CData) {
-	LOGMSG("Service(%s, %s)", Id, CData);
-	cControl::Launch(new cXinelibDvdPlayerControl(CData));
-	return true;
+        LOGMSG("Service(%s, %s)", Id, CData);
+        cPlayerFactory::Launch(m_Dev, pmNone, CData);
+        return true;
       }
       LOGMSG("Service(%s) -> true", Id);
       return true;
@@ -265,12 +279,9 @@ bool cPluginXinelibOutput::Service(const char *Id, void *Data)
 
     else if(!strcmp(Id, "ImagePlayer-1.0")) {
       if(CData && *CData) {
-	LOGMSG("Service(%s, %s)", Id, CData);
-	char **list = new char*[2];
-	list[0] = strdup(CData);
-	list[1] = NULL;
-	cControl::Launch(new cXinelibImagesControl(list, 0, 1));
-	return true;
+        LOGMSG("Service(%s, %s)", Id, CData);
+        cPlayerFactory::Launch(m_Dev, pmVideoOnly, CData);
+        return true;
       }
       LOGMSG("Service(%s) -> true", Id);
       return true;
@@ -282,9 +293,9 @@ bool cPluginXinelibOutput::Service(const char *Id, void *Data)
         int local_frontend = strstra(CData, xc.s_frontends, -1);
         if (local_frontend >= 0 && local_frontend < FRONTEND_count && strcmp(CData, xc.local_frontend)) {
           strn0cpy(xc.local_frontend, xc.s_frontends[local_frontend], sizeof(xc.local_frontend));
-          cXinelibDevice::Instance().ConfigureWindow(
+          m_Dev->ConfigureWindow(
                xc.fullscreen, xc.width, xc.height, xc.modeswitch, xc.modeline,
-               xc.display_aspect, xc.scale_video, xc.field_order);
+               xc.display_aspect, xc.scale_video);
         }
         return true;
       }
@@ -321,7 +332,7 @@ cString cPluginXinelibOutput::SVDRPCommand(const char *Command, const char *Opti
   if(strcasecmp(Command, "PMDA") == 0) {
     if(*Option) {
       LOGMSG("SVDRP(%s, %s)", Command, Option);
-      cControl::Launch(new cXinelibPlayerControl(ShowFiles, Option));
+      cPlayerFactory::Launch(m_Dev, pmAudioVideo, Option);
       return cString("Playing video file");
     } else {
       ReplyCode = 550; // Requested action not taken
@@ -332,17 +343,17 @@ cString cPluginXinelibOutput::SVDRPCommand(const char *Command, const char *Opti
   else if(strcasecmp(Command, "PDVD") == 0) {
     if(*Option) {
       LOGMSG("SVDRP(%s, %s)", Command, Option);
-      cControl::Launch(new cXinelibDvdPlayerControl(Option));
-      return cString("Playing DVD disc");
+      cPlayerFactory::Launch(m_Dev, pmNone, Option);
     } else {
-      ReplyCode = 550; // Requested action not taken
+      cPlayerFactory::Launch(m_Dev, pmNone, "dvd:/");
     }
+    return cString("Playing DVD disc");
   }
 
   else if(strcasecmp(Command, "PMSC") == 0) {
     if(*Option) {
       LOGMSG("SVDRP(%s, %s)", Command, Option);
-      cControl::Launch(new cXinelibPlayerControl(ShowMusic, Option));
+      cPlayerFactory::Launch(m_Dev, pmAudioOnly, Option);
       return cString("Playing music file");
     } else {
       ReplyCode = 550; // Requested action not taken
@@ -352,11 +363,8 @@ cString cPluginXinelibOutput::SVDRPCommand(const char *Command, const char *Opti
 
   else if(strcasecmp(Command, "PIMG") == 0) {
     if(*Option) {
-      char **list = new char*[2];
-      list[0] = strdup(Option);
-      list[1] = NULL;
       LOGMSG("SVDRP(%s, %s)", Command, Option);
-      cControl::Launch(new cXinelibImagesControl(list, 0, 1));
+      cPlayerFactory::Launch(m_Dev, pmVideoOnly, Option);
       return cString("Showing image file");
     } else {
       ReplyCode = 550; // Requested action not taken
@@ -367,7 +375,7 @@ cString cPluginXinelibOutput::SVDRPCommand(const char *Command, const char *Opti
   else if(strcasecmp(Command, "QMSC") == 0) {
     if(*Option) {
       LOGMSG("SVDRP(%s, %s)", Command, Option);
-      cXinelibPlayerControl::Queue(Option);
+      cPlayerFactory::Queue(m_Dev, Option);
       return cString("Queueing music file");
     } else {
       ReplyCode = 550; // Requested action not taken
